@@ -27,7 +27,7 @@
 #include "upsmon.h"
 #include "parseconf.h"
 #include "timehead.h"
-#
+
 #ifdef HAVE_STDARG_H
 #include <stdarg.h>
 #endif
@@ -171,7 +171,9 @@ static void notify(const char *notice, int flags, const char *ntype,
 				setenv("UPSNAME", "", 1);
 
 			setenv("NOTIFYTYPE", ntype, 1);
-			system(exec);
+			if (system(exec) == -1) {
+				upslog_with_errno(LOG_ERR, "%s", __func__);
+			}
 		}
 	}
 
@@ -255,9 +257,6 @@ static int do_upsd_auth(utype_t *ups)
 		return 0;
 	}
 
-	if (ups->pv == 0)	/* monitor only, no need to login */
-		return 1;
-
 	snprintf(buf, sizeof(buf), "USERNAME %s\n", ups->un);
 	if (upscli_sendline(&ups->conn, buf, strlen(buf)) < 0) {
 		upslogx(LOG_ERR, "Can't set username on [%s]: %s",
@@ -787,9 +786,6 @@ static void drop_connection(utype_t *ups)
 	clearflag(&ups->status, ST_LOGIN);
 	clearflag(&ups->status, ST_CONNECTED);
 
-	/* Attempt to logout from this UPS. If this doesn't work,
-	don't bother (it may not be listening to us anymore) */
-	upscli_sendline(&ups->conn, "LOGOUT\n", 7);
 	upscli_disconnect(&ups->conn);
 }
 
@@ -1617,7 +1613,9 @@ static int pdflag_status(void)
 
 	/* if it exists, see if it has the right text in it */
 
-	fgets(buf, sizeof(buf), pdf);
+	if (fgets(buf, sizeof(buf), pdf) == NULL) {
+		upslog_with_errno(LOG_ERR, "'%s' exists, but we can't read from it", powerdownflag);
+	}
 	fclose(pdf);
 
 	/* reasoning: say upsmon.conf is world-writable (!) and some nasty
