@@ -595,13 +595,13 @@ static int try_connect(void)
 	int	pipefd, ret;
 	struct	sockaddr_un saddr;
 
-	memset(&saddr, '\0', sizeof(saddr));
+	memset(&sa, '\0', sizeof(saddr));
 	saddr.sun_family = AF_UNIX;
 	snprintf(saddr.sun_path, sizeof(saddr.sun_path), "%s", pipefn);
 
 	pipefd = socket(AF_UNIX, SOCK_STREAM, 0);
 
-	if (pipefd == -1)
+	if (pipefd < 0)
 		fatal_with_errno(EXIT_FAILURE, "socket");
 
 	ret = connect(pipefd, (const struct sockaddr *) &saddr, sizeof(saddr));
