@@ -30,24 +30,23 @@
 #define MAX_CGI_STRLEN 128
 #define MAX_PARSE_ARGS 16
 
-static	char	*monhost = NULL;
-static	int	use_celsius = 1, refreshdelay = -1, treemode = 0;
+static char	*monhost = NULL;
+static int	use_celsius = 1, refreshdelay = -1, treemode = 0;
 
 	/* from cgilib's checkhost() */
-static	char	*monhostdesc = NULL;
+static char	*monhostdesc = NULL;
 
-static	int	port;
-static	char	*upsname, *hostname;
-static	char	*upsimgpath="upsimage.cgi", *upsstatpath="upsstats.cgi";
-static	UPSCONN_t	ups;
+static int	port;
+static char	*upsname, *hostname;
+static char	*upsimgpath="upsimage.cgi", *upsstatpath="upsstats.cgi";
+static UPSCONN_t	ups;
 
-static	FILE	*tf;
-static	long	forofs = 0;
+static FILE	*tf;
+static long	forofs = 0;
 
-static	ulist_t	*ulhead = NULL, *currups = NULL;
-
-static	int	skip_clause = 0, skip_block = 0;
+static ulist_t	*ulhead = NULL, *currups = NULL;
 
+static int	skip_clause = 0, skip_block = 0;
 
 void parsearg(char *var, char *value)
 {
@@ -99,7 +98,7 @@ static int check_ups_fd(int do_report)
 
 	/* must be OK */
 	return 1;
-}	
+}
 
 static int get_var(const char *var, char *buf, size_t buflen, int verbose)
 {
@@ -150,32 +149,30 @@ static void parse_var(const char *var)
 	if (!get_var(var, answer, sizeof(answer), 1))
 		return;
 
-	printf("%s\n", answer);
+	printf("%s", answer);
 }
 
 static void do_status(void)
 {
 	int	i;
-	char	status[SMALLBUF], *sp, *ptr;
+	char	status[SMALLBUF], *ptr, *last = NULL;
 
-	if (!get_var("ups.status", status, sizeof(status), 1))
+	if (!get_var("ups.status", status, sizeof(status), 1)) {
 		return;
+	}
 
-	sp = status;
-
-	while (sp) {
-		ptr = strchr(sp, ' ');
-		if (ptr)
-			*ptr++ = '\0';
+	for (ptr = strtok_r(status, " \n", &last); ptr != NULL; ptr = strtok_r(NULL, " \n", &last)) {
 
 		/* expand from table in status.h */
-		for (i = 0; stattab[i].name != NULL; i++)
-			if (!strcmp(stattab[i].name, sp))
-				printf("%s", stattab[i].desc);
+		for (i = 0; stattab[i].name != NULL; i++) {
 
-		sp = ptr;
+			if (!strcasecmp(ptr, stattab[i].name)) {
+				printf("%s<br>", stattab[i].desc);
+			}
+		}
 	}
 }
+
 static void do_runtime(void)
 {
 	int 	total, hours, minutes, seconds;
@@ -201,7 +198,7 @@ static int do_date(const char *buf)
 
 	time(&tod);
 	if (strftime(datebuf, sizeof(datebuf), buf, localtime(&tod))) {
-		printf("%s\n", datebuf);
+		printf("%s", datebuf);
 		return 1;
 	}
 
@@ -221,8 +218,7 @@ static int get_img_val(const char *var, const char *desc, const char *imgargs)
 	if ((imgargs) && (strlen(imgargs) > 0))
 		printf("&amp;%s", imgargs);
 
-	printf("\" ALT=\"%s: %s\">\n",
-		desc, answer);
+	printf("\" ALT=\"%s: %s\">", desc, answer);
 
 	return 1;
 }
@@ -380,7 +376,6 @@ static void ups_connect(void)
 		free(newhost);
 	}
 
-	upscli_sendline(&ups, "LOGOUT\n", 7);
 	upscli_disconnect(&ups);
 
 	free(upsname);
@@ -388,34 +383,38 @@ static void ups_connect(void)
 
 	if (upscli_splitname(currups->sys, &upsname, &hostname, &port) != 0) {
 		printf("Unusable UPS definition [%s]\n", currups->sys);
-		fprintf(stderr, "Unusable UPS definition [%s]\n", 
-			currups->sys);
+		fprintf(stderr, "Unusable UPS definition [%s]\n", currups->sys);
 		exit(EXIT_FAILURE);
 	}
 
 	if (upscli_connect(&ups, hostname, port, 0) < 0)
-		fprintf(stderr, "UPS [%s]: can't connect to server: %s\n",
-			currups->sys, upscli_strerror(&ups));
+		fprintf(stderr, "UPS [%s]: can't connect to server: %s\n", currups->sys, upscli_strerror(&ups));
 
 	lastups = currups;
 }
 
 static void do_hostlink(void)
 {
-	if (!currups)
+	if (!currups) {
 		return;
+	}
 
-	printf("<a href=\"%s?host=%s\">%s</a>\n",
-		upsstatpath, currups->sys, currups->desc);
+	printf("<a href=\"%s?host=%s", upsstatpath, currups->sys);
+
+	if (refreshdelay > 0) {
+		printf("&amp;refresh=%d", refreshdelay);
+	}
+
+	printf("\">%s</a>", currups->desc);
 }
 
 static void do_treelink(void)
 {
-	if (!currups)
+	if (!currups) {
 		return;
+	}
 
-	printf("<a href=\"%s?host=%s&amp;treemode\">All data</a>\n",
-		upsstatpath, currups->sys);
+	printf("<a href=\"%s?host=%s&amp;treemode\">All data</a>", upsstatpath, currups->sys);
 }
 
 /* see if the UPS supports this variable - skip to the next ENDIF if not */
@@ -562,12 +561,12 @@ static void do_temp(const char *var)
 		return;
 
 	if (use_celsius) {
-		printf("%s\n", tempc);
+		printf("%s", tempc);
 		return;
 	}
 
 	tempf = (strtod(tempc, (char **) NULL) * 1.8) + 32;
-	printf("%.1f\n", tempf);
+	printf("%.1f", tempf);
 }
 
 static void do_degrees(void)
@@ -575,9 +574,9 @@ static void do_degrees(void)
 	printf("&deg;");
 
 	if (use_celsius)
-		printf("C\n");
+		printf("C");
 	else
-		printf("F\n");
+		printf("F");
 }
 
 /* plug in the right color string (like #FF0000) for the UPS status */
@@ -589,13 +588,13 @@ static void do_statuscolor(void)
 	if (!check_ups_fd(0)) {
 
 		/* can't print the warning here - give a red error condition */
-		printf("#FF0000\n");
+		printf("#FF0000");
 		return;
 	}
 
 	if (!get_var("ups.status", stat, sizeof(stat), 0)) {
 		/* status not available - give yellow as a warning */
-		printf("#FFFF00\n");
+		printf("#FFFF00");
 		return;
 	}
 
@@ -617,47 +616,15 @@ static void do_statuscolor(void)
 	}
 
 	switch(severity) {
-		case 0:	printf("#00FF00\n"); break;	/* green  : OK      */
-		case 1:	printf("#FFFF00\n"); break;	/* yellow : warning */
+		case 0:	printf("#00FF00"); break;	/* green  : OK      */
+		case 1:	printf("#FFFF00"); break;	/* yellow : warning */
 
-		default: printf("#FF0000\n"); break;	/* red    : error   */
+		default: printf("#FF0000"); break;	/* red    : error   */
 	}
 }
 
-
-/* look for lines starting and ending with @ containing valid commands */
-static int parse_line(const char *buf)
+static int do_command(char *cmd)
 {
-	static	char	*cmd = NULL;
-
-	/* deal with extremely short lines as a special case */
-	if (strlen(buf) < 3) {
-
-		if (skip_clause || skip_block)
-			return 1;
-
-		return 0;
-	}
-
-	if (buf[0] != '@') {
-
-		/* if skipping a section, act like we parsed the line */
-		if (skip_clause || skip_block)
-			return 1;
-		
-		/* otherwise pass it through for normal printing */
-		return 0;
-	}
-
-	if (buf[strlen(buf) - 1] != '@')
-		return 0;
-
-	free(cmd);
-	cmd = xstrdup(&buf[1]);
-
-	/* strip off final @ */
-	cmd[strlen(cmd) - 1] = '\0';
-
 	/* ending an if block? */
 	if (!strcmp(cmd, "ENDIF")) {
 		skip_clause = 0;
@@ -671,19 +638,19 @@ static int parse_line(const char *buf)
 	}
 
 	/* Toggle state when we run across ELSE */
-	if(!strcmp(cmd, "ELSE")) {
-		if(skip_clause) {
+	if (!strcmp(cmd, "ELSE")) {
+		if (skip_clause) {
 			skip_clause = 0;
-		}
-		else {
+		} else {
 			skip_block = 1;
 		}
 		return 1;
 	}
 
 	/* don't do any commands if skipping a section */
-	if (skip_clause == 1)
+	if (skip_clause == 1) {
 		return 1;
+	}
 
 	if (!strncmp(cmd, "VAR ", 4)) {
 		parse_var(&cmd[4]);
@@ -691,12 +658,12 @@ static int parse_line(const char *buf)
 	}
 
 	if (!strcmp(cmd, "HOST")) {
-		printf("%s\n", currups->sys);
+		printf("%s", currups->sys);
 		return 1;
 	}
 
 	if (!strcmp(cmd, "HOSTDESC")) {
-		printf("%s\n", currups->desc);
+		printf("%s", currups->desc);
 		return 1;
 	}
 
@@ -725,22 +692,23 @@ static int parse_line(const char *buf)
 		return 1;
 	}
 
-	if (!strncmp(cmd, "DATE ", 5))
+	if (!strncmp(cmd, "DATE ", 5)) {
 		return do_date(&cmd[5]);
+	}
 
-	if (!strncmp(cmd, "IMG ", 4))
+	if (!strncmp(cmd, "IMG ", 4)) {
 		return do_img(&cmd[4]);
+	}
 
 	if (!strcmp(cmd, "VERSION")) {
-		printf("%s\n", UPS_VERSION);
+		printf("%s", UPS_VERSION);
 		return 1;
 	}
 
 	if (!strcmp(cmd, "REFRESH")) {
-		if (refreshdelay > 0)
-			printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"%d\">\n", 
-				refreshdelay);
-
+		if (refreshdelay > 0) {
+			printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"%d\">", refreshdelay);
+		}
 		return 1;
 	}
 
@@ -755,9 +723,10 @@ static int parse_line(const char *buf)
 	if (!strcmp(cmd, "ENDFOR")) {
 
 		/* if not in a for, ignore this */
-		if (forofs == 0)
+		if (forofs == 0) {
 			return 1;
-			
+		}
+
 		currups = currups->next;
 
 		if (currups) {
@@ -766,7 +735,7 @@ static int parse_line(const char *buf)
 		}
 
 		return 1;
-	}		
+	}
 
 	if (!strcmp(cmd, "HOSTLINK")) {
 		do_hostlink();
@@ -788,6 +757,11 @@ static int parse_line(const char *buf)
 		return 1;
 	}
 
+	if (!strcmp(cmd, "BATTTEMP")) {
+		do_temp("battery.temperature");
+		return 1;
+	}
+
 	if (!strcmp(cmd, "AMBTEMP")) {
 		do_temp("ambient.temperature");
 		return 1;
@@ -802,25 +776,61 @@ static int parse_line(const char *buf)
 		do_ifeq(&cmd[5]);
 		return 1;
 	}
-		
+
 	if (!strncmp(cmd, "IFBETWEEN ", 10)) {
 		do_ifbetween(&cmd[10]);
 		return 1;
 	}
 
-	if(!strncmp(cmd, "UPSSTATSPATH ", 13)) {
+	if (!strncmp(cmd, "UPSSTATSPATH ", 13)) {
 		do_upsstatpath(&cmd[13]);
 		return 1;
 	}
-		
-	if(!strncmp(cmd, "UPSIMAGEPATH ", 13)) {
+
+	if (!strncmp(cmd, "UPSIMAGEPATH ", 13)) {
 		do_upsimgpath(&cmd[13]);
 		return 1;
 	}
-		
+
 	return 0;
 }
 
+static void parse_line(const char *buf)
+{
+	char	cmd[SMALLBUF];
+	int	i, len, do_cmd = 0;
+
+	for (i = 0; buf[i]; i += len) {
+
+		len = strcspn(&buf[i], "@");
+
+		if (len == 0) {
+			if (do_cmd) {
+				do_command(cmd);
+				do_cmd = 0;
+			} else {
+				cmd[0] = '\0';
+				do_cmd = 1;
+			}
+			i++;	/* skip over the '@' character */
+			continue;
+		}
+
+		if (do_cmd) {
+			snprintf(cmd, sizeof(cmd), "%.*s", len, &buf[i]);
+			continue;
+		}
+
+		if (skip_clause || skip_block) {
+			/* ignore this */
+			continue;
+		}
+
+		/* pass it trough */
+		printf("%.*s", len, &buf[i]);
+	}
+}
+
 static void display_template(const char *tfn)
 {
 	char	fn[SMALLBUF], buf[LARGEBUF];	
@@ -830,8 +840,7 @@ static void display_template(const char *tfn)
 	tf = fopen(fn, "r");
 
 	if (!tf) {
-		fprintf(stderr, "upsstats: Can't open %s: %s\n", 
-			fn, strerror(errno));
+		fprintf(stderr, "upsstats: Can't open %s: %s\n", fn, strerror(errno));
 
 		printf("Error: can't open template file (%s)\n", tfn);
 
@@ -839,10 +848,7 @@ static void display_template(const char *tfn)
 	}
 
 	while (fgets(buf, sizeof(buf), tf)) {
-		buf[strlen(buf) - 1] = '\0';
-
-		if (!parse_line(buf))
-			printf("%s\n", buf);
+		parse_line(buf);
 	}
 
 	fclose(tf);
@@ -850,7 +856,6 @@ static void display_template(const char *tfn)
 
 static void display_tree(int verbose)
 {
-	int	ret;
 	unsigned int	numq, numa;
 	const	char	*query[4];
 	char	**answer;
@@ -858,7 +863,6 @@ static void display_tree(int verbose)
 	if (!upsname) {
 		if (verbose)
 			printf("[No UPS name specified]\n");
-
 		return;
 	}
 
@@ -866,16 +870,12 @@ static void display_tree(int verbose)
 	query[1] = upsname;
 	numq = 2;
 
-	ret = upscli_list_start(&ups, numq, query);
-
-	if (ret < 0) {
+	if (upscli_list_start(&ups, numq, query) < 0) {
 		if (verbose)
 			report_error();
 		return;
 	}
 
-	ret = upscli_list_next(&ups, numq, query, &numa, &answer);
-
 	printf("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"\n");
 	printf("	\"http://www.w3.org/TR/REC-html40/loose.dtd\">\n");
 	printf("<HTML>\n");
@@ -895,7 +895,7 @@ static void display_tree(int verbose)
 
 	printf("<TR><TH COLSPAN=3 BGCOLOR=\"#60B0B0\"></TH></TR>\n");
 
-	while (ret == 1) {
+	while (upscli_list_next(&ups, numq, query, &numa, &answer) == 1) {
 
 		/* VAR <upsname> <varname> <val> */
 		if (numa < 4) {
@@ -912,8 +912,6 @@ static void display_tree(int verbose)
 		printf("<TD>%s<br></TD>\n", answer[3]);
 
 		printf("</TR>\n");
-
-		ret = upscli_list_next(&ups, numq, query, &numa, &answer);
 	}
 
 	printf("</TABLE>\n");
@@ -1029,7 +1027,6 @@ static void display_single(void)
 	else
 		display_template("upsstats-single.html");
 
-	upscli_sendline(&ups, "LOGOUT\n", 7);
 	upscli_disconnect(&ups);
 }
 
@@ -1055,7 +1052,6 @@ int main(int argc, char **argv)
 
 	display_template("upsstats.html");
 
-	upscli_sendline(&ups, "LOGOUT\n", 7);
 	upscli_disconnect(&ups);
 
 	return 0;
