@@ -19,13 +19,31 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-#define APC_DRIVER_VERSION	"1.99.8"
-
 #include "main.h"
 #include "serial.h"
-
 #include "apcsmart.h"
 
+#define DRIVER_NAME	"APC Smart protocol driver"
+#define DRIVER_VERSION	"2.00"
+
+static upsdrv_info_t table_info = {
+	"APC command table",
+	APC_TABLE_VERSION,
+	NULL,
+	0,
+	{ NULL }
+};
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Russell Kroll <rkroll@exploits.org>\n" \
+	"Nigel Metheringham <Nigel.Metheringham@Intechnology.co.uk>",
+	DRV_STABLE,
+	{ &table_info, NULL }
+};
+
 #define ALT_CABLE_1 "940-0095B"
 
 	static	int	ups_status = 0, quirk_capability_overflow = 0;
@@ -170,7 +188,7 @@ static int read_buf(char *buf, size_t buflen)
 		POLL_ALERT, alert_handler, SER_WAIT_SEC, SER_WAIT_USEC);
 
 	if (ret < 1) {
-		ser_comm_fail(NULL);
+		ser_comm_fail("%s", ret ? strerror(errno) : "timeout");
 		return ret;
 	}
 
@@ -247,7 +265,7 @@ static int query_ups(const char *var, int first)
 		SER_WAIT_USEC);
 
 	if ((ret < 1) && (first == 0)) {
-		ser_comm_fail(NULL);
+		ser_comm_fail("%s", ret ? strerror(errno) : "timeout");
 		return 0;
 	}
 
@@ -533,19 +551,27 @@ static int firmware_table_lookup(void)
 		ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, IGNCHARS,
 			SER_WAIT_SEC, SER_WAIT_USEC);
 
+		if (ret < 1) {
+			upslog_with_errno(LOG_ERR, "firmware_table_lookup: ser_get_line failed");
+			return 0;
+		}
+
+		upsdebugx(2, "Firmware: [%s]", buf);
+
 		/* found one, force the model information */
 		if (!strcmp(buf, "6QD") || /* (APC600.) */
 				!strcmp(buf, "8QD") || /* (SmartUPS 1250, vintage 07/94.) */
+				!strcmp(buf, "8TI") || /* (SmartUPS 1250, vintage 08/24/95.) */
 				!strcmp(buf, "6TI") || /* (APC600.) */
-				!strcmp(buf, "6QI")) { /* (APC600.) */
+				!strcmp(buf, "6QI") || /* (APC600.) */
+				!strcmp(buf, "0ZI") || /* (APC Matrix 3000, vintage 10/99.) */
+				!strcmp(buf, "5UI")) { /* (APC Matrix 5000, vintage 10/93.) */
 			upsdebugx(1, "Found Smart-UPS");
 			dstate_setinfo("ups.model", "Smart-UPS");
 		}
 		else return 0;
 	}
 
-	upsdebugx(2, "Firmware: [%s]", buf);
-
 	/* this will be reworked if we get a lot of these things */
 	if (!strcmp(buf, "451.2.I")) {
 		quirk_capability_overflow = 1;
@@ -1192,15 +1218,6 @@ static void setuphandlers(void)
 
 /* functions that interface with main.c */
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools (version %s) - APC Smart protocol driver\n",
-		UPS_VERSION);
-	printf("\tDriver version %s, command table %s\n",
-		APC_DRIVER_VERSION,
-		APC_TABLE_VERSION);
-}
-
 void upsdrv_makevartable(void)
 {
 	addvar(VAR_VALUE, "cable", "Specify alternate cable (940-0095B)");
@@ -1248,8 +1265,6 @@ void upsdrv_initinfo(void)
 	/* manufacturer ID - hardcoded in this particular module */
 	dstate_setinfo("ups.mfr", "APC");
 
-	dstate_setinfo("driver.version.internal", "%s", APC_DRIVER_VERSION);
-
 	getbaseinfo();
 
 	printf("Detected %s [%s] on %s\n", dstate_getinfo("ups.model"),
@@ -1265,7 +1280,7 @@ void upsdrv_updateinfo(void)
 
 	/* try to wake up a dead ups once in awhile */
 	if ((dstate_is_stale()) && (!smartmode())) {
-		ser_comm_fail(NULL);
+		ser_comm_fail("Communications with UPS lost - check cabling");
 
 		/* reset this so a full update runs when the UPS returns */
 		last_full = 0;
