@@ -37,18 +37,6 @@ TODO List:
 
 	Extend the parsing of the Standard ID Block, to read:
 
-		Size of outlet monitoring block: (High priority)
-		To check if a outlet block is present.
-		Parse the outlet block to get the info about the
-		number of outlets and load segment state
-		(On, Off, On pending Off, Off pending On, Failed and closed, Failed and Open)
-		And the timers (Auto off delay, Auto on delay)
-		If this exist it is possible to use the
-		'Set outlet parameter command (0x97)' to alter the delay
-		settings or turn the outlet on or off with a delay (0 - 32767 seconds)
-		Also enable the outlet on off or shutdown-return commands in the driver.
-		(Check 'Communication Port List Block' down the list for more info)
-
 		Config Block Length: (High priority)
 		Give information if config block is
 		present, and how long it is, if it exist.
@@ -113,6 +101,9 @@ TODO List:
 		Number of outlets. (Number of Outlets "assigned to" (controlled by) this Comm Port)
 		Outlet number. (Each assigned Outlet is listed (1-64))
 
+		'Set outlet parameter command (0x97)' to alter the delay
+		settings or turn the outlet on or off with a delay (0 - 32767 seconds)
+
 
 	Rewrite some parts of the driver, to minimise code duplication. (Like the inst commands) 
 
@@ -126,16 +117,33 @@ TODO List:
 #include "bcmxcp_io.h"
 #include "bcmxcp.h"
 
-#define DRV_VERSION "0.14"
+#define DRIVER_NAME	"BCMXCP UPS driver"
+#define DRIVER_VERSION	"0.21"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Martin Schroeder <emes@geomer.de>\n" \
+	"Kjell Claesson <kjell.claesson@epost.tidanet.se>\n" \
+	"Tore Ørpetveit <tore@orpetveit.net>\n" \
+	"Wolfgang Ocker <weo@weo1.de>\n" \
+	"Oliver Wilcock",
+	DRV_STABLE,
+	{ &comm_upsdrv_info, NULL }
+};
 
 static int get_word(const unsigned char*);
 static long int get_long(const unsigned char*);
 static float get_float(const unsigned char *data);
 static void init_meter_map(void);
 static void init_alarm_map(void);
+static void init_config(void);
+static void init_limit(void);
 static void init_ups_meter_map(const unsigned char *map, unsigned char len);
 static void init_ups_alarm_map(const unsigned char *map, unsigned char len);
 static void decode_meter_map_entry(const unsigned char *entry, const unsigned char format, char* value);
+static int init_outlet(unsigned char len);
 static int instcmd(const char *cmdname, const char *extra);
 
 
@@ -143,7 +151,8 @@ char *FreqTol[3] = {"+/-2%", "+/-5%", "+/-7"};
 char *ABMStatus[4] = {"Charging", "Discharging", "Floating", "Resting"};
 unsigned char AUTHOR[4] = {0xCF, 0x69, 0xE8, 0xD5};		/* Autorisation	command	*/
 int nphases = 0;
-char *cpu_name[] = {"Cont:", "Inve:", "Rect:", "Netw:", "Disp:"};
+int outlet_block_len = 0;
+char *cpu_name[5] = {"Cont:", "Inve:", "Rect:", "Netw:", "Disp:"};
 
 /* get_word funktion from nut driver metasys.c */
 int get_word(const unsigned char *buffer)	/* return an integer reading a word in the supplied buffer */
@@ -273,17 +282,22 @@ void init_meter_map()
 	bcmxcp_meter_map[5].nut_entity = "input.L3-L1.voltage";
 	bcmxcp_meter_map[19].nut_entity = "input.L2.current";
 	bcmxcp_meter_map[20].nut_entity = "input.L3.current";
+	bcmxcp_meter_map[23].nut_entity = "ups.power";
 
 if (nphases == 1) {
 	bcmxcp_meter_map[18].nut_entity = "input.current";
 	bcmxcp_meter_map[56].nut_entity = "input.voltage";
 	bcmxcp_meter_map[65].nut_entity = "output.current";
+	bcmxcp_meter_map[68].nut_entity = "output.current.nominal";
 	bcmxcp_meter_map[78].nut_entity = "output.voltage";
+	bcmxcp_meter_map[82].nut_entity = "ups.realpower";
 }else{
 	bcmxcp_meter_map[18].nut_entity = "input.L1.current";
 	bcmxcp_meter_map[56].nut_entity = "input.L1-N.voltage";
 	bcmxcp_meter_map[65].nut_entity = "output.L1.current";
+	bcmxcp_meter_map[68].nut_entity = "output.L1.current.nominal";
 	bcmxcp_meter_map[78].nut_entity = "output.L1-N.voltage";
+	bcmxcp_meter_map[82].nut_entity = "ups.L1-N.realpower";
 }
 	bcmxcp_meter_map[27].nut_entity = "output.frequency";
 	bcmxcp_meter_map[28].nut_entity = "input.frequency";
@@ -291,15 +305,21 @@ if (nphases == 1) {
 	bcmxcp_meter_map[33].nut_entity = "battery.voltage";
 	bcmxcp_meter_map[34].nut_entity = "battery.charge";
 	bcmxcp_meter_map[35].nut_entity = "battery.runtime";
+	bcmxcp_meter_map[43].nut_entity = "battery.charge.low";
 	bcmxcp_meter_map[57].nut_entity = "input.L2-N.voltage";
 	bcmxcp_meter_map[58].nut_entity = "input.L3-N.voltage";
 	bcmxcp_meter_map[62].nut_entity = "ambient.temperature";
 	bcmxcp_meter_map[63].nut_entity = "ups.temperature";
 	bcmxcp_meter_map[66].nut_entity = "output.L2.current";
 	bcmxcp_meter_map[67].nut_entity = "output.L3.current";
+	bcmxcp_meter_map[69].nut_entity = "output.L2.current.nominal";
+	bcmxcp_meter_map[70].nut_entity = "output.L3.current.nominal";
 	bcmxcp_meter_map[77].nut_entity = "battery.temperature";
 	bcmxcp_meter_map[79].nut_entity = "output.L2-N.voltage";
 	bcmxcp_meter_map[80].nut_entity = "output.L3-N.voltage";
+	bcmxcp_meter_map[83].nut_entity = "ups.L2-N.realpower";
+	bcmxcp_meter_map[84].nut_entity = "ups.L3-N.realpower";
+	bcmxcp_meter_map[85].nut_entity = "ups.realpower.nominal";
 }
 
 void init_alarm_map()
@@ -810,15 +830,189 @@ void init_ups_alarm_map(const unsigned char *map, unsigned char len)
 	upsdebugx(2, "\n");
 }
 
+int init_outlet(unsigned char len)
+{
+	unsigned char answer[len];
+	int iIndex = 0, res, num;
+	int num_outlet, size_outlet;
+	int outlet_num, outlet_state;
+	short auto_dly_off, auto_dly_on;
+	char outlet_name[25];
+
+	res = command_read_sequence(PW_OUT_MON_BLOCK_REQ, answer);
+	if (res <= 0)
+		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
+
+	num_outlet = answer[iIndex++];
+	upsdebugx(2, "Number of outlets: %d\n", num_outlet); 
+
+	size_outlet = answer[iIndex++];
+	upsdebugx(2, "Number of bytes: %d\n", size_outlet); 
+
+	for(num = 1 ; num <= num_outlet ; num++) {
+		outlet_num = answer[iIndex++];
+		upsdebugx(2, "Outlet number: %d\n", outlet_num);
+		snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.id", num);
+		dstate_setinfo(outlet_name, "%d", outlet_num);
+
+		outlet_state = answer[iIndex++];
+		upsdebugx(2, "Outlet state: %d\n", outlet_state);
+		snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.status", num);
+		dstate_setinfo(outlet_name, "%s", (outlet_state & 0x01 ? "On" : "Off"));
+
+		auto_dly_off = get_word(answer+iIndex);
+		iIndex += 2;
+		upsdebugx(2, "Auto delay off: %d\n", auto_dly_off);
+		snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.delay.shutdown", num);
+		dstate_setinfo(outlet_name, "%d", auto_dly_off);
+
+		auto_dly_on = get_word(answer+iIndex);
+		iIndex += 2;
+		upsdebugx(2, "Auto delay on: %d\n", auto_dly_on);
+		snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.delay.start", num);
+		dstate_setinfo(outlet_name, "%d", auto_dly_on);
+	}	
+
+	return num_outlet;
+
+}
+
+void init_config(void)
+{
+	unsigned char answer[256];
+	int voltage = 0, res, len;
+	char sValue[17];
+
+	res = command_read_sequence(PW_CONFIG_BLOC_REQ, answer);
+	if (res <= 0)
+		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
+
+	/* Get validation mask for status bitmap */
+	bcmxcp_status.topology_mask = answer[BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_BYTE3];
+
+	/* Nominal output voltage of ups */
+	voltage = get_word((answer + BCMXCP_CONFIG_BLOCK_NOMINAL_OUTPUT_VOLTAGE));
+	
+	if (voltage != 0)
+		dstate_setinfo("output.voltage.nominal", "%d", voltage);
+
+	/* UPS serial number */
+	sValue[16] = 0;
+	
+	snprintf(sValue, 16, "%s", answer + BCMXCP_CONFIG_BLOCK_SERIAL_NUMBER);
+	len = 0;
+
+	for (len = 0; len < 16; len++) {
+		if (sValue[len] == 0x20) {
+			sValue[len] = 0;
+			break;
+		}
+	}
+	
+	dstate_setinfo("ups.serial", "%s", sValue);
+	
+}
+
+void init_limit(void)
+{
+	unsigned char answer[256];
+	int value, res;
+	char *horn_stat[3] = {"disabled", "enabled", "muted"};
+
+	res = command_read_sequence(PW_LIMIT_BLOCK_REQ, answer);
+	if (res <= 0) {
+		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
+	}
+
+	/* Nominal input voltage */
+	value = get_word((answer + BCMXCP_EXT_LIMITS_BLOCK_NOMINAL_INPUT_VOLTAGE));
+
+	if (value != 0) {
+		dstate_setinfo("input.voltage.nominal", "%d", value);
+	}
+
+	/* Nominal input frequency */
+	value = get_word((answer + BCMXCP_EXT_LIMITS_BLOCK_NOMINAL_INPUT_FREQ));
+
+	if (value != 0) {
+		int	fnom = value;
+		dstate_setinfo("input.frequency.nominal", "%d", value);
+
+		/* Input frequency deviation */
+		value = get_word((answer + BCMXCP_EXT_LIMITS_BLOCK_FREQ_DEV_LIMIT));
+
+		if (value != 0) {
+			value /= 100;
+			dstate_setinfo("input.frequency.low", "%d", fnom - value);
+			dstate_setinfo("input.frequency.high", "%d", fnom + value);
+		}
+	}
+
+	/* Bypass Voltage Low Deviation Limit / Transfer to Boost Voltage */
+	value = get_word((answer + BCMXCP_EXT_LIMITS_BLOCK_VOLTAGE_LOW_DEV_LIMIT));
+
+	if (value != 0) {
+		dstate_setinfo("input.transfer.boost.high", "%d", value);
+	}
+
+	/* Bypass Voltage High Deviation Limit / Transfer to Buck Voltage */
+	value = get_word((answer + BCMXCP_EXT_LIMITS_BLOCK_VOLTAGE_HIGE_DEV_LIMIT));
+
+	if (value != 0) {
+		dstate_setinfo("input.transfer.trim.low", "%d", value);
+	}
+
+	/* Low battery warning */
+	bcmxcp_status.lowbatt = answer[BCMXCP_EXT_LIMITS_BLOCK_LOW_BATT_WARNING] * 60;
+
+	/* Check if we should warn the user that her shutdown delay is to long? */
+	if (bcmxcp_status.shutdowndelay > bcmxcp_status.lowbatt)
+		upslogx(LOG_WARNING, "Shutdown delay longer than battery capacity when Low Battery warning is given. (max %d seconds)", bcmxcp_status.lowbatt);
+
+	/* Horn Status: */
+	value = answer[BCMXCP_EXT_LIMITS_BLOCK_HORN_STATUS];
+
+	if (value >= 0 && value <= 2) {
+		dstate_setinfo("ups.beeper.status", "%s", horn_stat[value]);
+	}
+
+	/* Minimum Supported Input Voltage */
+	value = get_word((answer + BCMXCP_EXT_LIMITS_BLOCK_MIN_INPUT_VOLTAGE));
+
+	if (value != 0) {
+		dstate_setinfo("input.transfer.low", "%d", value);
+	}
+
+	/* Maximum Supported Input Voltage */
+	value = get_word((answer + BCMXCP_EXT_LIMITS_BLOCK_MAX_INPUT_VOLTAGE));
+
+	if (value != 0) {
+		dstate_setinfo("input.transfer.high", "%d", value);
+	}
+
+	/* Ambient Temperature Lower Alarm Limit  */
+	value = answer[BCMXCP_EXT_LIMITS_BLOCK_AMBIENT_TEMP_LOW];
+
+	if (value != 0) {
+		dstate_setinfo("ambient.temperature.low", "%d", value);
+	}
+
+	/* AAmbient Temperature Upper Alarm Limit  */
+	value = answer[BCMXCP_EXT_LIMITS_BLOCK_AMBIENT_TEMP_HIGE];
+
+	if (value != 0) {
+		dstate_setinfo("ambient.temperature.high", "%d", value);
+	}
+}
+
 void upsdrv_initinfo(void)
 {
 	unsigned char answer[256];
-	char *pTmp, sValue[17];
+	char *pTmp;
+	char outlet_name[27];
 	int iRating = 0, iIndex = 0, res, len;
-	int voltage = 0, ncpu = 0, buf;
-
-	/* Set driver version info */
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
+	int ncpu = 0, buf;
+	int conf_block_len = 0, alarm_block_len = 0, cmd_list_len = 0;
 
 	/* Init BCM/XCP alarm descriptions */
 	init_alarm_map();
@@ -828,8 +1022,8 @@ void upsdrv_initinfo(void)
 		bcmxcp_status.shutdowndelay = atoi(getval("shutdown_delay"));
 	else
 		bcmxcp_status.shutdowndelay = 120;
-
-	/* Get information on UPS from UPS */
+		
+	/* Get information on UPS from UPS ID block */
 	res = command_read_sequence(PW_ID_BLOCK_REQ, answer);
 	if (res <= 0)
 		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
@@ -894,16 +1088,12 @@ void upsdrv_initinfo(void)
 
 	/* Make a nice model string and tell NUT about it */
 	snprintfcat(pTmp, len+10, " %dVA", iRating);
-	dstate_setinfo("ups.model", pTmp);
+	dstate_setinfo("ups.model", "%s", pTmp);
 	free(pTmp);
 
-	/* Display startup banner */
-	printf("Model = %s\n", dstate_getinfo("ups.model"));
-	printf("Firmware = %s\n", dstate_getinfo("ups.firmware"));
-	upslogx(LOG_INFO,"Shutdown delay =  %d seconds", bcmxcp_status.shutdowndelay);
-	
 	/* Get meter map info from ups, and init our map */
 	len = answer[iIndex++];
+	upsdebugx(2, "Length of meter map: %d\n", len); 
 	init_ups_meter_map(answer+iIndex, len);
 	iIndex += len;
 
@@ -913,54 +1103,69 @@ void upsdrv_initinfo(void)
 	init_ups_alarm_map(answer+iIndex, len);
 	iIndex += len;
 
-	/* Get information on UPS configuration */
-	res = command_read_sequence(PW_CONFIG_BLOC_REQ, answer);
-	if (res <= 0)
-		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
+	/* Then the Config_block_length */
+	conf_block_len = get_word(answer+iIndex);
+	upsdebugx(2, "Lengt of Config_block: %d\n", conf_block_len);
+	iIndex += 2;
 
-	/* Get validation mask for status bitmap */
-	bcmxcp_status.topology_mask = answer[BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_BYTE4];
+	/* Next is statistics map */
+	len = answer[iIndex++];
+	upsdebugx(2, "Length of statistics map: %d\n", len); 
+	/* init_statistics_map(answer+iIndex, len); */
+	iIndex += len;
 
-	/* Nominal output voltage of ups */
-	voltage = get_word((answer + BCMXCP_CONFIG_BLOCK_NOMINAL_OUTPUT_VOLTAGE));
-	
-	if (voltage != 0)
-		dstate_setinfo("output.voltage.nominal", "%d", voltage);
+	/* Size of the alarm history log */
+	iIndex += 2;
 
-	/* UPS serial number */
-	sValue[16] = 0;
-	
-	snprintf(sValue, 16, "%s", answer + BCMXCP_CONFIG_BLOCK_SERIAL_NUMBER);
-	len = 0;
+	/* Size of custom event log */
+	iIndex += 2;
 
-	for (len = 0; len < 16; len++) {
-		if (sValue[len] == 0x20) {
-			sValue[len] = 0;
-			break;
+	/* Size of topology block */
+	iIndex += 2;
+
+	/* Maximum supported command length */
+	iIndex += 1;
+
+	/* Size of command list block */
+	cmd_list_len = get_word(answer+iIndex);
+	upsdebugx(2, "Lengt of command list: %d\n", cmd_list_len);
+	iIndex += 2;
+
+	/* Size of outlet monitoring block */
+	outlet_block_len = get_word(answer+iIndex);
+	upsdebugx(2, "Lengt of outlet_block: %d\n", outlet_block_len);
+	iIndex += 2;
+
+	/* Size of the alarm block */
+	alarm_block_len = get_word(answer+iIndex);
+	upsdebugx(2, "Lengt of alarm_block: %d\n", alarm_block_len);
+	/* End of UPS ID block request */
+
+	/* Due to a bug in PW5115 firmware, we need to use blocklength > 8.
+	The protocol state that outlet block is only implemented if ther is
+	at least 2 outlet block. 5115 has only one outlet, but has outlet block. */ 
+	if (outlet_block_len > 8) {
+		len = init_outlet(outlet_block_len);
+
+		for(res = 1 ; res <= len ; res++) {
+			snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.shutdown.return", res);
+			dstate_addcmd(outlet_name);
 		}
 	}
-	
-	dstate_setinfo("ups.serial", sValue);
-	
+
+	/* Get information on UPS configuration */
+	init_config();
+
 	/* Get information on UPS extended limits */
-	res = command_read_sequence(PW_LIMIT_BLOCK_REQ, answer);
-	if (res <= 0)
-		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
-
-	/* Low battery warning */
-	bcmxcp_status.lowbatt = answer[BCMXCP_EXT_LIMITS_BLOCK_LOW_BATT_WARNING] * 60;
-
-	/* Check if we should warn the user that her shutdown delay is to long? */
-	if (bcmxcp_status.shutdowndelay > bcmxcp_status.lowbatt)
-		upslogx(LOG_WARNING, "Shutdown delay longer than battery capacity when Low Battery warning is given. (max %d seconds)", bcmxcp_status.lowbatt);
+	init_limit();
 	
 	/* Add instant commands */
 	dstate_addcmd("shutdown.return");
 	dstate_addcmd("shutdown.stayoff");
-	dstate_addcmd("outlet.1.shutdown.return"); /* ojw0000 */
-	dstate_addcmd("outlet.2.shutdown.return"); /* ojw0000 */
 	dstate_addcmd("test.battery.start");
+
 	upsh.instcmd = instcmd;
+
 	return;
 }
 
@@ -987,7 +1192,7 @@ void upsdrv_updateinfo(void)
 						 bcmxcp_meter_map[iIndex].format, sValue);
 
 			/* Set result */
-			dstate_setinfo(bcmxcp_meter_map[iIndex].nut_entity, sValue);
+			dstate_setinfo(bcmxcp_meter_map[iIndex].nut_entity, "%s", sValue);
 		}
 	}
 
@@ -1023,6 +1228,13 @@ void upsdrv_updateinfo(void)
 		dstate_setinfo("ups.load", "%5.1f", fValue);
 	}
 
+	/* Due to a bug in PW5115 firmware, we need to use blocklength > 8.
+	The protocol state that outlet block is only implemented if ther is
+	at least 2 outlet block. 5115 has only one outlet, but has outlet block. */ 
+	if (outlet_block_len > 8) {
+		init_outlet(outlet_block_len);
+	}
+
 	/* Get alarm info from UPS */
 	res = command_read_sequence(PW_CUR_ALARM_REQ, answer);
 	if (res <= 0){
@@ -1090,16 +1302,20 @@ void upsdrv_updateinfo(void)
 					status_set("OB");
 				break;
 			case 0xe0: /* Overload */
+				status_set("OL");
 				status_set("OVER");
 				break;
 			case 0x63: /* Trim */
+				status_set("OL");
 				status_set("TRIM");
 				break;
 			case 0x62:
 			case 0x61: /* Boost */
+				status_set("OL");
 				status_set("BOOST");
 				break;
 			case 0x60: /* Bypass */
+				status_set("OL");
 				status_set("BYPASS");
 				break;
 			case 0x10: /* Mostly off */
@@ -1201,7 +1417,8 @@ static int instcmd(const char *cmdname, const char *extra)
 	/* ojw0000 outlet power cycle for PW5125 and perhaps others */
 	if (!strcasecmp(cmdname, "outlet.1.shutdown.return")
 		|| !strcasecmp(cmdname, "outlet.2.shutdown.return")
-	) {
+		|| !strcasecmp(cmdname, "outlet.3.shutdown.return")
+	    ) {
 		send_write_command(AUTHOR, 4);
 
 		sleep(1);	/* Need to. Have to wait at least 0,25 sec max 16 sec */
@@ -1392,10 +1609,3 @@ void upsdrv_makevartable(void)
 	addvar(VAR_VALUE, "baud_rate", "Specify communication speed (ex: 9600)");
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - BCMXCP UPS driver %s (%s)\n\n",
-		DRV_VERSION, UPS_VERSION);
-
-	experimental_driver = 1;	
-}
