@@ -39,8 +39,21 @@
 #define PW_INIT_SYS_TEST	(unsigned char)0xB2 /* Initiate general system test command. length 2 */
 
 /* Config block offsets */
+#define BCMXCP_CONFIG_BLOCK_MACHINE_TYPE_CODE 		0
+#define BCMXCP_CONFIG_BLOCK_MODEL_NUMBER	 	2
+#define BCMXCP_CONFIG_BLOCK_MODEL_CONF_WORD 		4
+#define BCMXCP_CONFIG_BLOCK_INPUT_FREQ_DEV_LIMIT 	6
 #define BCMXCP_CONFIG_BLOCK_NOMINAL_OUTPUT_VOLTAGE 	8
-#define BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_BYTE4 	16
+#define BCMXCP_CONFIG_BLOCK_NOMINAL_OUTPUT_FREQ 	10
+#define BCMXCP_CONFIG_BLOCK_OUTPUT_PHASE_ANGLE	 	12
+#define BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_WORD1 	14
+#define BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_BYTE3 	16 /* KEEP THIS UNTILL PARSING OK. USE THIS BYTE. */
+#define BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_WORD2 	16
+#define BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_WORD3 	18
+#define BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_WORD4 	20
+#define BCMXCP_CONFIG_BLOCK_BATTERY_DATA_WORD1 		22 /* Undefined at this time.*/
+#define BCMXCP_CONFIG_BLOCK_BATTERY_DATA_WORD2 		24 /* Per cell inverter shutdown voltage at full rated load. (volt/cell)* 100 */
+#define BCMXCP_CONFIG_BLOCK_BATTERY_DATA_WORD3 		26 /* LOW BYTE Number of battery strings. HIGH BYTE undefined at this time.*/
 #define BCMXCP_CONFIG_BLOCK_EXTENDED_BLOCK_LENGTH	47
 #define BCMXCP_CONFIG_BLOCK_SERIAL_NUMBER		64
 
