@@ -1,7 +1,7 @@
 /* cps-hid.c - subdriver to monitor CPS USB/HID devices with NUT
  *
  *  Copyright (C)
- *  2003 - 2005 Arnaud Quette <arnaud.quette@free.fr>
+ *  2003 - 2008 Arnaud Quette <arnaud.quette@free.fr>
  *  2005 - 2006 Peter Selinger <selinger@users.sourceforge.net>         
  *
  *  Note: this subdriver was initially generated as a "stub" by the
@@ -28,11 +28,52 @@
 #include "extstate.h" /* for ST_FLAG_STRING */
 #include "main.h"     /* for getval() */
 #include "common.h"
+#include "usb-common.h"
 
-#define CPS_HID_VERSION      "CyberPower HID 0.1"
+#define CPS_HID_VERSION      "CyberPower HID 0.2"
 
+/* Cyber Power Systems */
 #define CPS_VENDORID 0x0764
 
+/*
+ * For some devices, the reported battery voltage is off by factor
+ * of 1.5 so we need to apply a scale factor to it to get the real
+ * battery voltage. By default, the factor is 1 (no scaling).
+ */
+static double	battery_scale = 1;
+
+static void *cps_battery_scale(void)
+{
+	battery_scale = 0.667;
+	return NULL;
+}
+
+/* USB IDs device table */
+static usb_device_id_t cps_usb_device_table[] = {
+	/* 900AVR/BC900D, CP1200AVR/BC1200D */
+	{ USB_DEVICE(CPS_VENDORID, 0x0005), NULL },
+	/* Dynex DX-800U? */
+	{ USB_DEVICE(CPS_VENDORID, 0x0501), &cps_battery_scale },
+
+	/* Terminating entry */
+	{ -1, -1, NULL }
+};
+
+/* returns statically allocated string - must not use it again before
+   done with result! */
+static char *cps_battvolt_fun(double value)
+{
+	static char	buf[8];
+
+	snprintf(buf, sizeof(buf), "%.1f", battery_scale * value);
+
+	return buf;
+}
+
+static info_lkp_t cps_battvolt[] = {
+	{ 0, NULL, &cps_battvolt_fun }
+};
+
 /* --------------------------------------------------------------- */
 /*      Vendor-specific usage table */
 /* --------------------------------------------------------------- */
@@ -53,10 +94,6 @@ static usage_tables_t cps_utab[] = {
 /* --------------------------------------------------------------- */
 
 static hid_info_t cps_hid2nut[] = {
-  /* Server side variables */
-  { "driver.version.internal", ST_FLAG_STRING, sizeof(DRIVER_VERSION), NULL, NULL, DRIVER_VERSION, HU_FLAG_ABSENT, NULL },
-  { "driver.version.data", ST_FLAG_STRING, sizeof(CPS_HID_VERSION), NULL, NULL, CPS_HID_VERSION, HU_FLAG_ABSENT, NULL },
-
   /* { "unmapped.ups.powersummary.rechargeable", 0, 0, "UPS.PowerSummary.Rechargeable", NULL, "%.0f", 0, NULL }, */
   /* { "unmapped.ups.powersummary.capacitymode", 0, 0, "UPS.PowerSummary.CapacityMode", NULL, "%.0f", 0, NULL }, */
   /* { "unmapped.ups.powersummary.designcapacity", 0, 0, "UPS.PowerSummary.DesignCapacity", NULL, "%.0f", 0, NULL }, */
@@ -73,7 +110,7 @@ static hid_info_t cps_hid2nut[] = {
   { "battery.runtime", 0, 0, "UPS.PowerSummary.RunTimeToEmpty", NULL, "%.0f", 0, NULL },
   { "battery.runtime.low", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerSummary.RemainingTimeLimit", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
   { "battery.voltage.nominal", 0, 0, "UPS.PowerSummary.ConfigVoltage", NULL, "%.0f", 0, NULL },
-  { "battery.voltage", 0, 0, "UPS.PowerSummary.Voltage", NULL, "%.1f", 0, NULL },
+  { "battery.voltage", 0, 0, "UPS.PowerSummary.Voltage", NULL, "%s", 0, cps_battvolt },
 
   /* UPS page */
   { "ups.load", 0, 0, "UPS.Output.PercentLoad", NULL, "%.0f", 0, NULL },
@@ -82,9 +119,9 @@ static hid_info_t cps_hid2nut[] = {
   { "ups.realpower.nominal", 0, 0, "UPS.Output.ConfigActivePower", NULL, "%.0f", 0, NULL },
   { "ups.delay.start", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.Output.DelayBeforeStartup", NULL, DEFAULT_ONDELAY, HU_FLAG_ABSENT, NULL},
   { "ups.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.Output.DelayBeforeShutdown", NULL, DEFAULT_OFFDELAY, HU_FLAG_ABSENT, NULL},
-  { "ups.timer.start", 0, 0, "UPS.Output.DelayBeforeStartup", NULL, "%.0f", 0, NULL},
-  { "ups.timer.shutdown", 0, 0, "UPS.Output.DelayBeforeShutdown", NULL, "%.0f", 0, NULL},
-  { "ups.timer.reboot", 0, 0, "UPS.Output.DelayBeforeReboot", NULL, "%.0f", 0, NULL},
+  { "ups.timer.start", 0, 0, "UPS.Output.DelayBeforeStartup", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
+  { "ups.timer.shutdown", 0, 0, "UPS.Output.DelayBeforeShutdown", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
+  { "ups.timer.reboot", 0, 0, "UPS.Output.DelayBeforeReboot", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
 
   /* Special case: ups.status & ups.alarm */
   { "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ACPresent", NULL, NULL, HU_FLAG_QUICK_POLL, online_info },
@@ -139,25 +176,26 @@ static char *cps_format_serial(HIDDevice_t *hd) {
 /* this function allows the subdriver to "claim" a device: return 1 if
  * the device is supported by this subdriver, else 0. */
 static int cps_claim(HIDDevice_t *hd) {
-	if (hd->VendorID != CPS_VENDORID) {
-		return 0;
-	}
-	switch (hd->ProductID) {
 
-	/* accept any known UPS - add devices here as needed */
-	case 0x0005:	/* Cyber Power 900AVR/BC900D, CP1200AVR/BC1200D */
-			/* fixme: are the above really HID devices? */
-			/* Dynex DX-800U */
-	case 0x0501:
-		return 1;
+	int status = is_usb_device_supported(cps_usb_device_table, hd->VendorID,
+								 hd->ProductID);
 
-	/* by default, reject, unless the productid option is given */
-	default:
-		if (getval("productid")) {
+	switch (status) {
+
+		case POSSIBLY_SUPPORTED:
+			/* by default, reject, unless the productid option is given */
+			if (getval("productid")) {
+				return 1;
+			}
+			possibly_supported("CyberPower", hd);
+			return 0;
+
+		case SUPPORTED:
 			return 1;
-		}
-		possibly_supported("CyberPower", hd);
-		return 0;
+
+		case NOT_SUPPORTED:
+		default:
+			return 0;
 	}
 }
 
