@@ -29,10 +29,6 @@
  *   editing the definition file?
  */
 
-
-#define DRV_VERSION	"0.03"
-
-
 /* from usbhid-ups.h */
 /* --------------------------------------------------------------- */
 /* Struct & data for ups.status processing                         */
@@ -45,16 +41,16 @@ typedef struct {
 
 #define STATUS_CAL		1       /* calibration */
 #define STATUS_TRIM		2       /* SmartTrim */
-#define STATUS_BOOST		4       /* SmartBoost */
+#define STATUS_BOOST	4       /* SmartBoost */
 #define STATUS_OL		8       /* on line */
 #define STATUS_OB		16      /* on battery */
 #define STATUS_OVER		32      /* overload */
 #define STATUS_LB		64      /* low battery */
 #define STATUS_RB		128     /* replace battery */
-#define STATUS_BYPASS		256	/* on bypass */
-#define STATUS_OFF		512	/* ups is off */
+#define STATUS_BYPASS	256		/* on bypass */
+#define STATUS_OFF		512		/* ups is off */
 #define STATUS_CHRG		1024	/* charging */
-#define STATUS_DISCHRG		2048	/* discharging */
+#define STATUS_DISCHRG	2048	/* discharging */
 
 status_lkp_t status_info[] = {
   { "CAL", STATUS_CAL },
@@ -84,14 +80,14 @@ typedef struct {
 /* --------------------------------------------------------------- */
 
 typedef struct {
-	char	*info_type;		/* NUT variable name */
-	int	info_flags;		/* NUT flags (to set in addinfo) */
-	float	info_len;		/* if ST_FLAG_STRING: length of the string */
-					/* if HU_TYPE_CMD: command value ; multiplier (or max len) otherwise */
-	char	*default_value;		/* if HU_FLAG_ABSENT: default value ; format otherwise */
-	int	drv_flags;		/* */
-	char **var_values;		/* all possible values for this variable (allows to check data...) */
-					/* FIXME: "void *" so we can have bound or enum */
+	char	*info_type;	/* NUT variable name */
+	int	info_flags;	/* NUT flags (to set in addinfo) */
+	float	info_len;	/* if ST_FLAG_STRING: length of the string */
+				/* if HU_TYPE_CMD: command value ; multiplier (or max len) otherwise */
+	char	*default_value;	/* if HU_FLAG_ABSENT: default value ; format otherwise */
+	int	drv_flags;	/* */
+	char	**var_values;	/* all possible values for this variable (allows to check data...) */
+				/* FIXME: "void *" so we can have bound or enum */
 /*	interpreter interpret;	*/	/* FFE: interpreter fct, NULL if not needed  */
 } dummy_info_t;
 
@@ -108,7 +104,6 @@ typedef struct {
 dummy_info_t nut_data[] =
 {
 	/* Essential variables, loaded before parsing the definition file */
-	{ "driver.version.internal", ST_FLAG_STRING, 5, DRV_VERSION, DU_FLAG_INIT, NULL },
 	{ "ups.mfr", ST_FLAG_STRING | ST_FLAG_RW, 32, "Dummy Manufacturer", DU_FLAG_INIT, NULL },
 	{ "ups.model", ST_FLAG_STRING | ST_FLAG_RW, 32, "Dummy UPS", DU_FLAG_INIT, NULL },
 	{ "ups.status", ST_FLAG_STRING | ST_FLAG_RW, 32, "OL", DU_FLAG_INIT, NULL },
@@ -172,14 +167,12 @@ battery.packs.bad
 
 ambient.temperature
 ambient.temperature.alarm
-ambient.temperature.alarm.maximum
-ambient.temperature.alarm.minimum
-ambient.temperature.alarm.enable
+ambient.temperature.high
+ambient.temperature.low
 ambient.humidity
 ambient.humidity.alarm
-ambient.humidity.alarm.maximum
-ambient.humidity.alarm.minimum
-ambient.humidity.alarm.enable
+ambient.humidity.high
+ambient.humidity.low
 
 FIXME: how to manage these?
 outlet.n.id
