@@ -6,7 +6,7 @@
  *   2001 John Marley <John.Marley@alcatel.com.au>
  *   2002 Jules Taplin <jules@netsitepro.co.uk>
  *   2002 Eric Lawson <elawson@inficad.com>
- *   2005 Arnaud Quette <http://arnaud.quette.free.fr/contact.html>
+ *   2005 Arnaud Quette <arnaud.quette@gmail.com>
  *   2005 Nadav Moskovitch <blutz@walla.com / http://www.gamatronic.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,12 +29,26 @@
 #include "serial.h" 
 #include "gamatronic.h"
 
+#define DRIVER_NAME	"Gamatronic UPS driver"
+#define DRIVER_VERSION	"0.02"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"John Marley <John.Marley@alcatel.com.au>\n" \
+	"Jules Taplin <jules@netsitepro.co.uk>\n" \
+	"Eric Lawson <elawson@inficad.com>\n" \
+	"Arnaud Quette <arnaud.quette@gmail.com>\n" \
+	"Nadav Moskovitch <blutz@walla.com / http://www.gamatronic.com>",
+	DRV_STABLE,
+	{ NULL }
+};
 
 #define ENDCHAR	'\r'	
 #define IGNCHARS ""
 #define SER_WAIT_SEC    1	/* allow 3.0 sec for ser_get calls */
 #define SER_WAIT_USEC	0
-#define DRV_VERSION	"0.01"
 
 int sec_upsrecv (char *buf)
 {
@@ -82,7 +96,7 @@ int sec_cmd(const char mode, const char *command, char *msgbuf, int *buflen)
 		mode, command);
     }	
     upsdebugx(1, "PC-->UPS: \"%s\"",msg);
-    ret = ser_send(upsfd,msg);
+    ret = ser_send(upsfd, "%s", msg);
     
     upsdebugx(1, " send returned: %d",ret);
 
@@ -237,8 +251,6 @@ void upsdrv_initinfo(void)
     int msglen, e, v;
     char *a,*p,avail_list[300];
  
-    dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
- 
     /* find out which variables/commands this UPS supports */
     msglen = 0;
     sec_cmd(SEC_POLLCMD, SEC_AVAILP1, avail_list, &msglen);
@@ -325,12 +337,6 @@ void upsdrv_makevartable(void)
 	/* addvar(VAR_VALUE, "foo", "Override foo setting"); */
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Gamatronic UPS driver %s (%s)\n\n", 
-		DRV_VERSION, UPS_VERSION);
-}
-
 void setup_serial(const char *port)
 {
     char temp[140];
