@@ -17,15 +17,24 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-#define DRV_VERSION "1.34"
-
 #include <sys/ioctl.h>
 
 #include "main.h"
 #include "serial.h"
-
 #include "genericups.h"
 
+#define DRIVER_NAME	"Generic contact-closure UPS driver"
+#define DRIVER_VERSION	"1.35"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Russell Kroll <rkroll@exploits.org>",
+	DRV_STABLE,
+	{ NULL }
+};
+
 	static	int	upstype = -1;
 
 static void parse_output_signals(const char *value, int *line)
@@ -107,8 +116,6 @@ void upsdrv_initinfo(void)
 {
 	char	*v;
 
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
-
 	/* setup the basics */
 
 	dstate_setinfo("ups.mfr", "%s", ((v = getval("mfr")) != NULL) ? v : upstab[upstype].mfr);
@@ -258,12 +265,6 @@ void upsdrv_help(void)
 	listtypes();
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Generic UPS driver %s (%s)\n",
-		DRV_VERSION, UPS_VERSION);
-}
-
 void upsdrv_makevartable(void)
 {
 	addvar(VAR_VALUE, "upstype", "Set UPS type (required)");
