@@ -61,8 +61,6 @@
 
 	/* signal handling */
 	int	exit_flag = 0;
-	static	sigset_t		main_sigmask;
-	static	struct	sigaction	main_sa;
 
 	/* everything else */
 	static	char	*pidfn = NULL;
@@ -167,18 +165,19 @@ static void exit_cleanup(int sig)
 
 static void setup_signals(void)
 {
-	sigemptyset(&main_sigmask);
-	main_sa.sa_mask = main_sigmask;
-	main_sa.sa_flags = SA_RESTART | SA_NOCLDSTOP;
+	struct sigaction	sa;
 
-	main_sa.sa_handler = exit_cleanup;
-	sigaction(SIGTERM, &main_sa, NULL);
-	sigaction(SIGINT, &main_sa, NULL);
-	sigaction(SIGQUIT, &main_sa, NULL);
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = SA_RESTART | SA_NOCLDSTOP;
 
-/*	main_sa.sa_handler = SIG_IGN;
-	sigaction(SIGHUP, &main_sa, NULL);
-	sigaction(SIGPIPE, &main_sa, NULL);*/
+	sa.sa_handler = exit_cleanup;
+	sigaction(SIGTERM, &sa, NULL);
+	sigaction(SIGINT, &sa, NULL);
+	sigaction(SIGQUIT, &sa, NULL);
+
+/*	sa.sa_handler = SIG_IGN;
+	sigaction(SIGHUP, &sa, NULL);
+	sigaction(SIGPIPE, &sa, NULL);*/
 }
 
 /* (*GSourceFunc) wrapper */
@@ -196,8 +195,6 @@ int main(int argc, char **argv)
 	char *hal_debug_level;
 /*	int	i, do_forceshutdown = 0; */
 
-	upsdrv_banner();
-
 	if (experimental_driver) {
 		printf("Warning: This is an experimental driver.\n");
 		printf("Some features may not function correctly.\n\n");
@@ -333,7 +330,11 @@ int main(int argc, char **argv)
 	}
 
 	/* add a timer for data update */
+#ifdef HAVE_GLIB_2_14
 	g_timeout_add_seconds (poll_interval,
+#else
+	g_timeout_add (1000 * poll_interval,				/* seconds */
+#endif
 							(GSourceFunc)update_data,
 							NULL);
 
@@ -511,6 +512,7 @@ DBusHandlerResult dbus_filter_function(DBusConnection *connection,
 					      void *user_data)
 {
 	DBusError dbus_error;
+	int retcode = -1;
 	const char	*member		= dbus_message_get_member(message);
 	const char	*path		= dbus_message_get_path(message);
 /*	int ret = DBUS_HANDLER_RESULT_NOT_YET_HANDLED; */
@@ -535,7 +537,7 @@ DBusHandlerResult dbus_filter_function(DBusConnection *connection,
 			
 	 		fprintf(stdout, "executing Shutdown\n");
 			upsdrv_shutdown();
-			dbus_send_reply(connection, message, DBUS_TYPE_INT32, 0);
+			dbus_send_reply(connection, message, DBUS_TYPE_INVALID, NULL);
 
 		} else if (dbus_message_is_method_call(message,
 												DBUS_INTERFACE, "SetBeeper")) {
@@ -550,13 +552,13 @@ DBusHandlerResult dbus_filter_function(DBusConnection *connection,
 			
 			if (b_enable==TRUE) {
 				if (upsh.instcmd("beeper.enable", NULL) != STAT_INSTCMD_HANDLED) {
-					dbus_send_reply(connection, message, DBUS_TYPE_INT32, -1);
+					dbus_send_reply(connection, message, DBUS_TYPE_INT32, &retcode);
 					return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 				}
 			}
 			else {
 				if (upsh.instcmd("beeper.disable", NULL) != STAT_INSTCMD_HANDLED) {
-					dbus_send_reply(connection, message, DBUS_TYPE_INT32, -1);
+					dbus_send_reply(connection, message, DBUS_TYPE_INT32, &retcode);
 					return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 				}
 			}
@@ -564,6 +566,6 @@ DBusHandlerResult dbus_filter_function(DBusConnection *connection,
 			return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 		}
 	}
-	dbus_send_reply(connection, message, DBUS_TYPE_INT32, 0);
+	dbus_send_reply(connection, message, DBUS_TYPE_INVALID, NULL);
 	return DBUS_HANDLER_RESULT_HANDLED;
 }
