@@ -21,10 +21,9 @@
 #include "dstate.h"
 
 	/* data which may be useful to the drivers */	
-	int	upsfd = -1;
-	char	*device_path = NULL;
-	const	char	*progname = NULL, *upsname = NULL, 
-			*device_name = NULL;
+	int		upsfd = -1;
+	char		*device_path = NULL;
+	const char	*progname = NULL, *upsname = NULL, *device_name = NULL;
 
 	/* may be set by the driver to wake up while in dstate_poll_fds */
 	int	extrafd = -1;
@@ -32,10 +31,6 @@
 	/* for ser_open */
 	int	do_lock_port = 1;
 
-	/* set by the drivers */
-	int	experimental_driver = 0;
-	int	broken_driver = 0;
-
 	/* for detecting -a values that don't match anything */
 	static	int	upsname_found = 0;
 
@@ -47,11 +42,32 @@
 
 	/* signal handling */
 	int	exit_flag = 0;
-	static	sigset_t		main_sigmask;
-	static	struct	sigaction	main_sa;
 
 	/* everything else */
-	static	char	*pidfn = NULL;
+	static char	*pidfn = NULL;
+
+/* print the driver banner */
+void upsdrv_banner (void)
+{
+	int i;
+
+	printf("Network UPS Tools - %s %s (%s)\n", upsdrv_info.name, upsdrv_info.version, UPS_VERSION);
+
+	/* process sub driver(s) information */
+	for (i = 0; upsdrv_info.subdrv_info[i]; i++) {
+
+		if (!upsdrv_info.subdrv_info[i]->name) {
+			continue;
+		}
+
+		if (!upsdrv_info.subdrv_info[i]->version) {
+			continue;
+		}
+
+		printf("%s %s\n", upsdrv_info.subdrv_info[i]->name,
+			upsdrv_info.subdrv_info[i]->version);
+	}
+}
 
 /* power down the attached load immediately */
 static void forceshutdown(void)
@@ -125,6 +141,17 @@ static void storeval(const char *var, char *val)
 {
 	vartab_t	*tmp, *last;
 
+	if (!strncasecmp(var, "override.", 9)) {
+		dstate_setinfo(var+9, "%s", val);
+		dstate_setflags(var+9, ST_FLAG_IMMUTABLE);
+		return;
+	}
+
+	if (!strncasecmp(var, "default.", 8)) {
+		dstate_setinfo(var+8, "%s", val);
+		return;
+	}
+
 	tmp = last = vartab_h;
 
 	while (tmp) {
@@ -398,8 +425,6 @@ static void vartab_free(void)
 
 static void exit_cleanup(void)
 {
-	upsdrv_cleanup();
-
 	free(chroot_path);
 	free(device_path);
 	free(user);
@@ -420,18 +445,19 @@ static void set_exit_flag(int sig)
 
 static void setup_signals(void)
 {
-	sigemptyset(&main_sigmask);
-	main_sa.sa_mask = main_sigmask;
-	main_sa.sa_flags = 0;
+	struct sigaction	sa;
 
-	main_sa.sa_handler = set_exit_flag;
-	sigaction(SIGTERM, &main_sa, NULL);
-	sigaction(SIGINT, &main_sa, NULL);
-	sigaction(SIGQUIT, &main_sa, NULL);
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = 0;
 
-	main_sa.sa_handler = SIG_IGN;
-	sigaction(SIGHUP, &main_sa, NULL);
-	sigaction(SIGPIPE, &main_sa, NULL);
+	sa.sa_handler = set_exit_flag;
+	sigaction(SIGTERM, &sa, NULL);
+	sigaction(SIGINT, &sa, NULL);
+	sigaction(SIGQUIT, &sa, NULL);
+
+	sa.sa_handler = SIG_IGN;
+	sigaction(SIGHUP, &sa, NULL);
+	sigaction(SIGPIPE, &sa, NULL);
 }
 
 int main(int argc, char **argv)
@@ -439,6 +465,8 @@ int main(int argc, char **argv)
 	struct	passwd	*new_uid = NULL;
 	int	i, do_forceshutdown = 0;
 
+	atexit(exit_cleanup);
+
 	/* pick up a default from configure --with-user */
 	user = xstrdup(RUN_AS_USER);	/* xstrdup: this gets freed at exit */
 
@@ -447,7 +475,7 @@ int main(int argc, char **argv)
 
 	upsdrv_banner();
 
-	if (experimental_driver) {
+	if (upsdrv_info.status == DRV_EXPERIMENTAL) {
 		printf("Warning: This is an experimental driver.\n");
 		printf("Some features may not function correctly.\n\n");
 	}
@@ -544,11 +572,12 @@ int main(int argc, char **argv)
 
 	upsdrv_initups();
 
+	/* UPS is detected now, cleanup upon exit */
+	atexit(upsdrv_cleanup);
+
 	/* now see if things are very wrong out there */
-	if (broken_driver) {
-		fatalx(EXIT_FAILURE,
-			"Fatal error: broken driver. It probably needs to be converted.\n"
-			"Search for 'broken_driver = 1' in the source for more details.");
+	if (upsdrv_info.status == DRV_BROKEN) {
+		fatalx(EXIT_FAILURE, "Fatal error: broken driver. It probably needs to be converted.\n");
 	}
 
 	if (do_forceshutdown)
@@ -563,6 +592,7 @@ int main(int argc, char **argv)
 
 	/* publish the top-level data: version number, driver name */
 	dstate_setinfo("driver.version", "%s", UPS_VERSION);
+	dstate_setinfo("driver.version.internal", "%s", upsdrv_info.version);
 	dstate_setinfo("driver.name", "%s", progname);
 
 	/* The poll_interval may have been changed from the default */
@@ -573,17 +603,21 @@ int main(int argc, char **argv)
 		writepid(pidfn);
 	}
 
-	/* safe to do this now that the parent has exited */
-	atexit(exit_cleanup);
+	while (!exit_flag) {
+
+		struct timeval	timeout;
+	
+		gettimeofday(&timeout, NULL);
+		timeout.tv_sec += poll_interval;
 
-	while (exit_flag == 0) {
 		upsdrv_updateinfo();
 
-		dstate_poll_fds(poll_interval, extrafd);
+		while (!dstate_poll_fds(timeout, extrafd) && !exit_flag) {
+			/* repeat until time is up or extrafd has data */
+		}
 	}
 
 	/* if we get here, the exit flag was set by a signal handler */
-
 	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
 
 	exit(EXIT_SUCCESS);
