@@ -37,11 +37,27 @@
 #include <string.h>
 #include <stdlib.h>
 
+#define DRIVER_NAME	"Megatec protocol driver"
+#define DRIVER_VERSION	"1.6" /* FIXME: doesn't conform to "X.YZ" */
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Carlos Rodrigues <carlos.efr@mail.telepac.pt>",
+	DRV_STABLE,
+#ifdef MEGATEC_SUBDRV
+	{ &megatec_subdrv_info, NULL }
+#else
+	{ NULL }
+#endif
+};
 
 #define ENDCHAR  '\r'
 #define IGNCHARS ""
 
 #define RECV_BUFFER_LEN 128
+#define FIELD_BUFFER_LEN 16
 
 /* The expected reply lengths */
 #define F_CMD_REPLY_LEN  21
@@ -51,7 +67,6 @@
 #define IDENT_MAXTRIES   5
 #define IDENT_MINSUCCESS 3
 
-#define SEND_PACE    100000  /* interval between chars on send (usec) */
 #define READ_TIMEOUT 2       /* timeout on read (seconds) */
 #define READ_PACE    300000  /* interval to wait between sending a command and reading the response (usec) */
 
@@ -138,6 +153,12 @@ static BatteryVolts_t batteries[] = {{ 12.0,  9.0, 16.0,  9.7, 13.7,  0.0 },   /
                                      { 96.0,  1.5,  3.0, 1.63, 2.29,  1.8 },   /* Ablerex MS3000RT (LB at 25% charge) */
                                      {  0.0,  0.0,  0.0,  0.0,  0.0,  0.0 }};  /* END OF DATA */
 
+/* Interval between chars on send (usec) */
+static int send_pace = 100000;
+
+/* Some models need this */
+static char state_dtr = 1;
+static char state_rts = 0;
 
 /* Workaround for buggy models */
 static char ignore_off = 0;  /* ignore FL_LOAD_OFF if it behaves strangely */
@@ -146,6 +167,9 @@ static char ignore_off = 0;  /* ignore FL_LOAD_OFF if it behaves strangely */
 static float battvolt_empty = -1;  /* unknown */
 static float battvolt_full = -1;   /* unknown */
 
+/* Battery voltage multiplier (to match the nominal voltage on some models) */
+static float battvolt_mult = 1;
+
 /* Minimum and maximum voltage seen on input */
 static float ivolt_min = INT_MAX;  /* unknown */
 static float ivolt_max = -1;       /* unknown */
@@ -288,12 +312,21 @@ static int get_ups_info(UPSInfo_t *info)
 
 	upsdebugx(2, "Asking for UPS information [I]...");
 	ser_flush_io(upsfd);
-	ser_send_pace(upsfd, SEND_PACE, "I%c", ENDCHAR);
+	ser_send_pace(upsfd, send_pace, "I%c", ENDCHAR);
 	usleep(READ_PACE);
 
+	/*
+	 * Expected reply: "#UPS_MFR........ UPS_MODEL. UPS_VER...<cr>"
+	 */
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
 
 	if (ret < 0) {
+		upsdebug_with_errno(2, "I => FAILED");
+
+		return -1;
+	}
+
+	if (ret == 0) {
 		upsdebugx(2, "I => FAILED [timeout]");
 
 		return -1;
@@ -326,6 +359,8 @@ static int get_ups_info(UPSInfo_t *info)
 	anchor = copy_field(info->model, anchor + 1, UPS_MODEL_CHARS);
 	copy_field(info->version, anchor + 1, UPS_VERSION_CHARS);
 
+	upsdebugx(2, "I VALUES => [%s %s %s]", info->mfr, info->model, info->version);
+
 	return 0;
 }
 
@@ -333,16 +368,27 @@ static int get_ups_info(UPSInfo_t *info)
 static int get_firmware_values(FirmwareValues_t *values)
 {
 	char buffer[RECV_BUFFER_LEN];
+	char field[FIELD_BUFFER_LEN];
+	char *anchor;
 	int ret;
 
 	upsdebugx(2, "Asking for UPS power ratings [F]...");
 	ser_flush_io(upsfd);
-	ser_send_pace(upsfd, SEND_PACE, "F%c", ENDCHAR);
+	ser_send_pace(upsfd, send_pace, "F%c", ENDCHAR);
 	usleep(READ_PACE);
 
+	/*
+	 * Expected reply: "#MMM.M QQQ SS.SS RR.R<cr>"
+	 */
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
 
 	if (ret < 0) {
+		upsdebug_with_errno(2, "F => FAILED");
+
+		return -1;
+	}
+
+	if (ret == 0) {
 		upsdebugx(2, "F => FAILED [timeout]");
 
 		return -1;
@@ -365,8 +411,20 @@ static int get_firmware_values(FirmwareValues_t *values)
 
 	upsdebugx(2, "F => OK [%s]", buffer);
 
-	sscanf(buffer, "#%f %f %f %f", &values->volt, &values->current,
-	       &values->battvolt, &values->freq);
+	anchor = copy_field(field, &buffer[1], 5);
+	values->volt = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 3);
+	values->current = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 5);
+	values->battvolt = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 4);
+	values->freq = atof(field);
+
+	upsdebugx(2, "F VALUES => [%.1f %.1f %.1f %.1f]", values->volt,
+	          values->current, values->battvolt, values->freq);
 
 	return 0;
 }
@@ -375,17 +433,27 @@ static int get_firmware_values(FirmwareValues_t *values)
 static int run_query(QueryValues_t *values)
 {
 	char buffer[RECV_BUFFER_LEN];
-	char temperature[8];
+	char field[FIELD_BUFFER_LEN];
+	char *anchor;
 	int ret;
 
 	upsdebugx(2, "Asking for UPS status [Q1]...");
 	ser_flush_io(upsfd);
-	ser_send_pace(upsfd, SEND_PACE, "Q1%c", ENDCHAR);
+	ser_send_pace(upsfd, send_pace, "Q1%c", ENDCHAR);
 	usleep(READ_PACE);
 
+	/*
+	 * Expected reply: "(MMM.M NNN.N PPP.P QQQ RR.R S.SS TT.T b7b6b5b4b3b2b1b0<cr>"
+	 */
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
 
 	if (ret < 0) {
+		upsdebug_with_errno(2, "Q1 => FAILED");
+
+		return -1;
+	}
+
+	if (ret == 0) {
 		upsdebugx(2, "Q1 => FAILED [timeout]");
 
 		return -1;
@@ -407,16 +475,38 @@ static int run_query(QueryValues_t *values)
 
 	upsdebugx(2, "Q1 => OK [%s]", buffer);
 
-	sscanf(buffer, "(%f %f %f %f %f %f %s %s", &values->ivolt, &values->fvolt, &values->ovolt,
-	       &values->load, &values->freq, &values->battvolt, temperature, values->flags);
+	anchor = copy_field(field, &buffer[1], 5);
+	values->ivolt = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 5);
+	values->fvolt = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 5);
+	values->ovolt = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 3);
+	values->load = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 4);
+	values->freq = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 4);
+	values->battvolt = atof(field);
+
+	anchor = copy_field(field, anchor + 1, 4);
+	values->temp = atof(field);
+
+	anchor = copy_field(values->flags, anchor + 1, N_FLAGS);
+
+	if (strlen(values->flags) < N_FLAGS) {
+		upsdebugx(2, "Q1 => FAILED [flags error]");
+		return -1;
+	}
+
+	upsdebugx(2, "Q1 VALUES => [%.1f %.1f %.1f %.1f %.1f %.1f %.1f %s]",
+	          values->ivolt, values->fvolt, values->ovolt, values->load,
+	          values->freq, values->battvolt, values->temp, values->flags);
 
-	/*
-	 * UPS temperature must be parsed on its own. Some models put
-	 * something other than a float in this field, meaning there is no
-	 * temperature reading available.
-	 */
-	values->temp = atof(temperature);
-	
 	return 0;
 }
 
@@ -429,12 +519,30 @@ void upsdrv_initinfo(void)
 	QueryValues_t status;
 	UPSInfo_t info;
 
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
+	/*
+	 * Some models apparently time-out with the default send pace, so we must
+	 * allow the user to override it if needed be. The configuration parameter
+	 * is specified in milliseconds for the user's benefit.
+	 */
+	if (getval("sendpace")) {
+		upsdebugx(2, "Default command send pace is %d usec.", send_pace);
+		upsdebugx(2, "Parameter [sendpace]: [%s]", getval("sendpace"));
+
+		/* Having 1 second as the upper-bound is an arbitrary choice... */
+		send_pace = CLAMP(atoi(getval("sendpace")), 1, 1000) * 1000;
+
+		upslogx(LOG_NOTICE, "Command send pace changed to %d usec.", send_pace);
+	}
 
 	/*
 	 * UPS detection sequence.
 	 */
 	upsdebugx(1, "Starting UPS detection process...");
+	
+	/* Some models seem to need this. We'll just discard the ouput for now... */
+	get_ups_info(&info);
+
+	/* Check for a compatible UPS and for a reliable connection... */
 	for (i = 0; i < IDENT_MAXTRIES; i++) {
 		if (check_ups(&status) == 0) {
 			success++;
@@ -455,7 +563,7 @@ void upsdrv_initinfo(void)
 	dstate_setinfo("ups.type", status.flags[FL_UPS_TYPE] == '1' ? "standby" : "online");
 
 	upsdebugx(1, "Cancelling any pending shutdown or battery test.");
-	ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
+	ser_send_pace(upsfd, send_pace, "C%c", ENDCHAR);
 
 	/*
 	 * Try to identify the UPS.
@@ -501,6 +609,15 @@ void upsdrv_initinfo(void)
 		}
 	}
 
+	if (getval("battvoltmult")) {
+		upsdebugx(2, "Parameter [battvoltmult]: [%s]", getval("battvoltmult"));
+
+		/* Having SHRT_MAX as the upper-bound is an arbitrary choice... */
+		battvolt_mult = CLAMP(atof(getval("battvoltmult")), 1, SHRT_MAX);
+
+		upslogx(LOG_NOTICE, "The battery voltage reported by the UPS will be multiplied by %.1f.", battvolt_mult);
+	}
+
 	if (getval("battvolts")) {
 		upsdebugx(2, "Parameter [battvolts]: [%s]", getval("battvolts"));
 
@@ -594,7 +711,13 @@ void upsdrv_updateinfo(void)
 	dstate_setinfo("output.voltage", "%.1f", query.ovolt);
 	dstate_setinfo("ups.load", "%.1f", query.load);
 	dstate_setinfo("input.frequency", "%.1f", query.freq);
-	dstate_setinfo("battery.voltage", "%.2f", query.battvolt);
+
+	/*
+	 * The battery voltage multiplier should only be applied to battery.voltage
+	 * in order not to break the charge calculation (that uses the 'raw' value
+	 * that is reported by the UPS).
+	 */
+	dstate_setinfo("battery.voltage", "%.2f", battvolt_mult * query.battvolt);
 
 	if (query.temp > 0.01) {
 		dstate_setinfo("ups.temperature", "%.1f", query.temp);
@@ -682,8 +805,8 @@ void upsdrv_shutdown(void)
 
 	upslogx(LOG_INFO, "Shutting down UPS.");
 
-	ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
-	ser_send_pace(upsfd, SEND_PACE, "S%02dR%04d%c", s_wait, r_wait, ENDCHAR);
+	ser_send_pace(upsfd, send_pace, "C%c", ENDCHAR);
+	ser_send_pace(upsfd, send_pace, "S%02dR%04d%c", s_wait, r_wait, ENDCHAR);
 }
 
 
@@ -699,7 +822,7 @@ int instcmd(const char *cmdname, const char *extra)
 
 	if (strcasecmp(cmdname, "test.battery.start.deep") == 0) {
 		ser_flush_io(upsfd);
-		ser_send_pace(upsfd, SEND_PACE, "TL%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "TL%c", ENDCHAR);
 		usleep(READ_PACE);
 
 		if (ser_get_line(upsfd, buffer, 2 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
@@ -713,7 +836,7 @@ int instcmd(const char *cmdname, const char *extra)
 
 	if (strcasecmp(cmdname, "test.battery.start") == 0) {
 		ser_flush_io(upsfd);
-		ser_send_pace(upsfd, SEND_PACE, "T%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "T%c", ENDCHAR);
 		usleep(READ_PACE);
 
 		if (ser_get_line(upsfd, buffer, 1 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
@@ -727,7 +850,7 @@ int instcmd(const char *cmdname, const char *extra)
 
 	if (strcasecmp(cmdname, "test.battery.stop") == 0) {
 		ser_flush_io(upsfd);
-		ser_send_pace(upsfd, SEND_PACE, "CT%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "CT%c", ENDCHAR);
 		usleep(READ_PACE);
 
 		if (ser_get_line(upsfd, buffer, 2 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
@@ -740,10 +863,10 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "shutdown.return") == 0) {
-		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "C%c", ENDCHAR);
 		watchdog_enabled = 0;
 
-		ser_send_pace(upsfd, SEND_PACE, "S%02dR%04d%c", shutdown_delay, start_delay, ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "S%02dR%04d%c", shutdown_delay, start_delay, ENDCHAR);
 
 		upslogx(LOG_INFO, "Shutdown (return) initiated.");
 
@@ -751,15 +874,15 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "shutdown.stayoff") == 0) {
-		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "C%c", ENDCHAR);
 		watchdog_enabled = 0;
 
 		ser_flush_io(upsfd);
-		ser_send_pace(upsfd, SEND_PACE, "S%02d%c", shutdown_delay, ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "S%02d%c", shutdown_delay, ENDCHAR);
 		usleep(READ_PACE);
 
 		if (ser_get_line(upsfd, buffer, 3 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
-			ser_send_pace(upsfd, SEND_PACE, "S%02dR9999%c", shutdown_delay, ENDCHAR);
+			ser_send_pace(upsfd, send_pace, "S%02dR9999%c", shutdown_delay, ENDCHAR);
 			upslogx(LOG_NOTICE, "UPS refuses to turn the load off indefinitely. Will turn off for 9999 minutes instead.");
 		}
 
@@ -769,7 +892,7 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "shutdown.stop") == 0) {
-		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "C%c", ENDCHAR);
 		watchdog_enabled = 0;
 
 		upslogx(LOG_INFO, "Shutdown canceled.");
@@ -778,7 +901,7 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "load.on") == 0) {
-		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "C%c", ENDCHAR);
 		watchdog_enabled = 0;
 
 		upslogx(LOG_INFO, "Turning the load on.");
@@ -787,15 +910,15 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "load.off") == 0) {
-		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "C%c", ENDCHAR);
 		watchdog_enabled = 0;
 
 		ser_flush_io(upsfd);
-		ser_send_pace(upsfd, SEND_PACE, "S00%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "S00%c", ENDCHAR);
 		usleep(READ_PACE);
 
 		if (ser_get_line(upsfd, buffer, 3 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
-			ser_send_pace(upsfd, SEND_PACE, "S00R9999%c", ENDCHAR);
+			ser_send_pace(upsfd, send_pace, "S00R9999%c", ENDCHAR);
 			upslogx(LOG_NOTICE, "UPS refuses to turn the load off indefinitely. Will turn off for 9999 minutes instead.");
 		}
 
@@ -817,8 +940,8 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "reset.watchdog") == 0) {
-		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
-		ser_send_pace(upsfd, SEND_PACE, "S%02dR0001%c", watchdog_timeout, ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "C%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "S%02dR0001%c", watchdog_timeout, ENDCHAR);
 
 		if (watchdog_enabled) {
 			upsdebugx(2, "Resetting the UPS watchdog.");
@@ -832,7 +955,7 @@ int instcmd(const char *cmdname, const char *extra)
 
 	if (strcasecmp(cmdname, "beeper.toggle") == 0) {
 		ser_flush_io(upsfd);
-		ser_send_pace(upsfd, SEND_PACE, "Q%c", ENDCHAR);
+		ser_send_pace(upsfd, send_pace, "Q%c", ENDCHAR);
 		usleep(READ_PACE);
 
 		if (ser_get_line(upsfd, buffer, 1 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
@@ -863,36 +986,52 @@ void upsdrv_help(void)
 
 void upsdrv_makevartable(void)
 {
-	addvar(VAR_VALUE, "mfr"      , "Manufacturer name");
-	addvar(VAR_VALUE, "model"    , "Model name");
-	addvar(VAR_VALUE, "serial"   , "UPS serial number");
-	addvar(VAR_VALUE, "lowbatt"  , "Low battery level (%)");
-	addvar(VAR_VALUE, "ondelay"  , "Minimum delay before UPS startup (minutes)");
-	addvar(VAR_VALUE, "offdelay" , "Delay before UPS shutdown (minutes)");
+	addvar(VAR_VALUE, "mfr", "Manufacturer name");
+	addvar(VAR_VALUE, "model", "Model name");
+	addvar(VAR_VALUE, "serial", "UPS serial number");
+	addvar(VAR_VALUE, "lowbatt", "Low battery level (%)");
+	addvar(VAR_VALUE, "ondelay", "Min. delay before UPS startup (minutes)");
+	addvar(VAR_VALUE, "offdelay", "Delay before UPS shutdown (minutes)");
 	addvar(VAR_VALUE, "battvolts", "Battery voltages (empty:full)");
-	addvar(VAR_FLAG , "ignoreoff", "Ignore the OFF status reported by the UPS.");
+	addvar(VAR_VALUE, "battvoltmult", "Battery voltage multiplier");
+	addvar(VAR_FLAG , "ignoreoff", "Ignore the OFF status from the UPS");
+	addvar(VAR_VALUE, "sendpace", "Interval between command chars (msec)");
+	addvar(VAR_VALUE, "dtr", "Serial DTR line state (0/1)");
+	addvar(VAR_VALUE, "rts", "Serial RTS line state (0/1)");
 
 	megatec_subdrv_makevartable();
 }
 
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools %s - Megatec protocol driver %s [%s]\n", UPS_VERSION, DRV_VERSION, progname);
-	printf("Carlos Rodrigues (c) 2003-2008\n\n");
-
-	megatec_subdrv_banner();
-}
-
-
 void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B2400);
 
-	/* Some UPS models need this. */
-	ser_set_dtr(upsfd, 1);
-	ser_set_rts(upsfd, 0);
+	if (getval("dtr")) {
+		upsdebugx(2, "Parameter [dtr]: [%s]", getval("dtr"));
+
+		if (strcmp(getval("dtr"), "0") != 0 && strcmp(getval("dtr"), "1") != 0) {
+			fatalx(EXIT_FAILURE, "Error in \"dtr\" parameter.");
+		}
+
+		state_dtr = atoi(getval("dtr"));
+	}
+
+	if (getval("rts")) {
+		upsdebugx(2, "Parameter [rts]: [%s]", getval("rts"));
+
+		if (strcmp(getval("rts"), "0") != 0 && strcmp(getval("rts"), "1") != 0) {
+			fatalx(EXIT_FAILURE, "Error in \"rts\" parameter.");
+		}
+
+		state_rts = atoi(getval("rts"));
+	}
+
+	upsdebugx(2, "DTR=%d, RTS=%d", state_dtr, state_rts);
+
+	ser_set_dtr(upsfd, state_dtr);
+	ser_set_rts(upsfd, state_rts);
 }
 
 
