@@ -24,8 +24,18 @@
 
 #include "main.h"
 #include "serial.h"
-#include "metasys.h"
-#include <string.h>
+
+#define DRIVER_NAME	"Metasystem UPS driver"
+#define DRIVER_VERSION	"0.07"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Fabio Di Niro <fabio.diniro@email.it>",
+	DRV_STABLE,
+	{ NULL }
+};
 
 /* Autorestart flag */
 int autorestart = 0;
@@ -158,7 +168,7 @@ int get_answer(unsigned char *data) {
 	int packet_lenght, checksum, i, res;
 	/* Read STX byte */
 	res = ser_get_char(upsfd, my_buf, 1, 0);
-	if (res != 1) {
+	if (res < 1) {
 		ser_comm_fail("Receive error (STX): %d!!!\n", res);
 		return -1;	
 	}
@@ -168,7 +178,7 @@ int get_answer(unsigned char *data) {
 	}
 	/* Read data lenght byte */
 	res = ser_get_char(upsfd, my_buf, 1, 0);
-	if (res != 1) {
+	if (res < 1) {
 		ser_comm_fail("Receive error (lenght): %d!!!\n", res);
 		return -1;	
 	}
@@ -308,7 +318,7 @@ void upsdrv_initinfo(void)
 	  dstate_setaux("ups.test.result", 20);
 	
 	/* Fixed variables */
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
+	dstate_setinfo("driver.version.internal", "%s", DRIVER_VERSION);
 	/* UPS INFO READ */
 	res = command_read_sequence(UPS_INFO, my_answer);
 	if (res < 0) fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
@@ -535,7 +545,7 @@ void upsdrv_initinfo(void)
 	memcpy(serial, my_answer + 7, res - 7);
 	/* serial number start from the 8th byte */
 	serial[12]=0;		/* terminate string */
-	dstate_setinfo("ups.serial", serial);
+	dstate_setinfo("ups.serial", "%s", serial);
 	
 	/* get the ups firmware. The major number is in the 5th byte, the minor is in the 6th */
 	dstate_setinfo("ups.firmware", "%u.%u", my_answer[5], my_answer[6]);
@@ -1030,12 +1040,6 @@ void upsdrv_makevartable(void)
 
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Metasystem UPS driver %s \n\n", 
-		DRV_VERSION);
-}
-
 void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path); 
