@@ -1,6 +1,6 @@
 /*  mge-hid.c - data to monitor MGE UPS SYSTEMS HID (USB and serial) devices
  *
- *  Copyright (C) 2003 - 2005
+ *  Copyright (C) 2003 - 2008
  *  			Arnaud Quette <arnaud.quette@mgeups.fr>
  *
  *  Sponsored by MGE UPS SYSTEMS <http://www.mgeups.com>
@@ -25,12 +25,28 @@
 #include "usbhid-ups.h"
 #include "mge-hid.h"
 #include "extstate.h"	/* for ST_FLAG_STRING */
-#include "main.h"	/* for getval() */
+#include "main.h"		/* for getval() */
 #include "common.h"
 
-#define MGE_HID_VERSION		"MGE HID 1.10"
+#define MGE_HID_VERSION		"MGE HID 1.12"
+
+#ifndef SHUT_MODE
+#include "usb-common.h"
+
+/* MGE Office Protection Systems, prev. MGE UPS Systems */
 #define MGE_VENDORID		0x0463
 
+/* USB IDs device table */
+static usb_device_id_t mge_usb_device_table[] = {
+	/* various models */
+	{ USB_DEVICE(MGE_VENDORID, 0x0001), NULL },
+	{ USB_DEVICE(MGE_VENDORID, 0xffff), NULL },
+	
+	/* Terminating entry */
+	{ -1, -1, NULL }
+};
+#endif
+
 typedef enum {
 	MGE_DEFAULT = 0,
 	MGE_EVOLUTION = 0x100,		/* MGE Evolution series */
@@ -144,6 +160,13 @@ static info_lkp_t mge_emergency_stop[] = {
 	{ 0, NULL, NULL }
 };
 
+/* FIXME: limit to ups.model ~= Protection Station... */
+static info_lkp_t pegasus_threshold_info[] = {
+	{ 10, "10", NULL },
+	{ 25, "25", NULL },
+	{ 60, "60", NULL },
+	{ 0, NULL, NULL }
+};
 
 /* --------------------------------------------------------------- */
 /*      Vendor-specific usage table */
@@ -283,10 +306,15 @@ static usage_tables_t mge_utab[] = {
 typedef struct {
 	const char	*iProduct;
 	const char	*iModel;
-	models_type_t	type;		/* Enumerated model type */
-	const char	*name;
+	models_type_t	type;	/* enumerated model type */
+	const char	*name;		/* optional (defaults to "<iProduct> <iModel>" if NULL) */
 } models_name_t;
 
+/*
+ * Do not remove models from this list, but instead comment them
+ * out if not needed. This allows us to quickly add overrides for
+ * specific models only, should this be needed.
+ */
 static models_name_t mge_model_names [] =
 {
 	/* Ellipse models */
@@ -308,17 +336,22 @@ static models_name_t mge_model_names [] =
 	{ "ELLIPSE", "1000", MGE_DEFAULT, "Ellipse 1000" },
 	{ "ELLIPSE", "1500", MGE_DEFAULT, "Ellipse 1500" },
 
-	/* Ellipse "MAX" */
-	{ "Ellipse MAX", "600", MGE_DEFAULT, "Ellipse MAX 600" },
-	{ "Ellipse MAX", "850", MGE_DEFAULT, "Ellipse MAX 850" },
-	{ "Ellipse MAX", "1100", MGE_DEFAULT, "Ellipse MAX 1100" },
-	{ "Ellipse MAX", "1500", MGE_DEFAULT, "Ellipse MAX 1500" },
+	/* Ellipse "MAX" (TBR) */
+/*	{ "Ellipse MAX", "600", MGE_DEFAULT, NULL }, */
+/*	{ "Ellipse MAX", "850", MGE_DEFAULT, NULL }, */
+/*	{ "Ellipse MAX", "1100", MGE_DEFAULT, NULL }, */
+/*	{ "Ellipse MAX", "1500", MGE_DEFAULT, NULL }, */
 
 	/* Protection Center */
 	{ "PROTECTIONCENTER", "420", MGE_DEFAULT, "Protection Center 420" },
 	{ "PROTECTIONCENTER", "500", MGE_DEFAULT, "Protection Center 500" },
 	{ "PROTECTIONCENTER", "675", MGE_DEFAULT, "Protection Center 675" },
 
+	/* Protection Station (TBR) */
+/*	{ "Protection Station", "500", MGE_DEFAULT, NULL }, */
+/*	{ "Protection Station", "650", MGE_DEFAULT, NULL }, */
+/*	{ "Protection Station", "800", MGE_DEFAULT, NULL }, */
+
 	/* Evolution models */
 	{ "Evolution", "500", MGE_DEFAULT, "Pulsar Evolution 500" },
 	{ "Evolution", "800", MGE_DEFAULT, "Pulsar Evolution 800" },
@@ -329,35 +362,48 @@ static models_name_t mge_model_names [] =
 	{ "Evolution", "3000XL", MGE_DEFAULT, "Pulsar Evolution 3000 XL" },
 
 	/* Newer Evolution models */
-	{ "Evolution", "650", MGE_EVOLUTION_650, "Evolution 650" },
-	{ "Evolution", "850", MGE_EVOLUTION_850, "Evolution 850" },
-	{ "Evolution", "1150", MGE_EVOLUTION_1150, "Evolution 1150" },
-	{ "Evolution", "S 1250", MGE_EVOLUTION_S_1250, "Evolution S 1250" },
-	{ "Evolution", "1550", MGE_EVOLUTION_1550, "Evolution 1550" },
-	{ "Evolution", "S 1750", MGE_EVOLUTION_S_1750, "Evolution S 1750" },
-	{ "Evolution", "2000", MGE_EVOLUTION_2000, "Evolution 2000" },
-	{ "Evolution", "S 2500", MGE_EVOLUTION_S_2500, "Evolution S 2500" },
-	{ "Evolution", "S 3000", MGE_EVOLUTION_S_3000, "Evolution S 3000" },
+	{ "Evolution", "650", MGE_EVOLUTION_650, NULL },
+	{ "Evolution", "850", MGE_EVOLUTION_850, NULL },
+	{ "Evolution", "1150", MGE_EVOLUTION_1150, NULL },
+	{ "Evolution", "S 1250", MGE_EVOLUTION_S_1250, NULL },
+	{ "Evolution", "1550", MGE_EVOLUTION_1550, NULL },
+	{ "Evolution", "S 1750", MGE_EVOLUTION_S_1750, NULL },
+	{ "Evolution", "2000", MGE_EVOLUTION_2000, NULL },
+	{ "Evolution", "S 2500", MGE_EVOLUTION_S_2500, NULL },
+	{ "Evolution", "S 3000", MGE_EVOLUTION_S_3000, NULL },
 
 	/* Pulsar M models */
-	{ "PULSAR M", "2200", MGE_PULSAR_M_2200, "Pulsar M 2200" },
-	{ "PULSAR M", "3000", MGE_PULSAR_M_3000, "Pulsar M 3000" },
-	{ "PULSAR M", "3000 XL", MGE_PULSAR_M_3000_XL, "Pulsar M 3000 XL" },
+	{ "PULSAR M", "2200", MGE_PULSAR_M_2200, NULL },
+	{ "PULSAR M", "3000", MGE_PULSAR_M_3000, NULL },
+	{ "PULSAR M", "3000 XL", MGE_PULSAR_M_3000_XL, NULL },
+	/* Eaton'ified names */
+	{ "EX", "2200", MGE_PULSAR_M_2200, NULL },
+	{ "EX", "3000", MGE_PULSAR_M_3000, NULL },
+	{ "EX", "3000 XL", MGE_PULSAR_M_3000, NULL },
 
-	/* Pulsar models */
-	{ "Pulsar", "700", MGE_DEFAULT, "Pulsar 700" },
-	{ "Pulsar", "1000", MGE_DEFAULT, "Pulsar 1000" },
-	{ "Pulsar", "1500", MGE_DEFAULT, "Pulsar 1500" },
-	{ "Pulsar", "1000 RT2U", MGE_DEFAULT, "Pulsar 1000 RT2U" },
-	{ "Pulsar", "1500 RT2U", MGE_DEFAULT, "Pulsar 1500 RT2U" },
+	/* Pulsar models (TBR) */
+/*	{ "Pulsar", "700", MGE_DEFAULT, NULL }, */
+/*	{ "Pulsar", "1000", MGE_DEFAULT, NULL }, */
+/*	{ "Pulsar", "1500", MGE_DEFAULT, NULL }, */
+/*	{ "Pulsar", "1000 RT2U", MGE_DEFAULT, NULL }, */
+/*	{ "Pulsar", "1500 RT2U", MGE_DEFAULT, NULL }, */
+	/* Eaton'ified names (TBR) */
+/*	{ "EX", "700", MGE_DEFAULT, NULL }, */
+/*	{ "EX", "1000", MGE_DEFAULT, NULL }, */
+/*	{ "EX", "1500", MGE_DEFAULT, NULL }, */
+/*	{ "EX", "1000 RT2U", MGE_DEFAULT, NULL }, */
+/*	{ "EX", "1500 RT2U", MGE_DEFAULT, NULL }, */
 
 	/* Pulsar MX models */
 	{ "PULSAR", "MX4000", MGE_DEFAULT, "Pulsar MX 4000 RT" },
 	{ "PULSAR", "MX5000", MGE_DEFAULT, "Pulsar MX 5000 RT" },
 
-	/* NOVA models */	
-	{ "NOVA AVR", "600", MGE_DEFAULT, "NOVA 600 AVR" },
-	{ "NOVA AVR", "1100", MGE_DEFAULT, "NOVA 1100 AVR" },
+	/* NOVA models */
+	{ "NOVA AVR", "500", MGE_DEFAULT, "Nova 500 AVR" },
+	{ "NOVA AVR", "600", MGE_DEFAULT, "Nova 600 AVR" },
+	{ "NOVA AVR", "625", MGE_DEFAULT, "Nova 625 AVR" },
+	{ "NOVA AVR", "1100", MGE_DEFAULT, "Nova 1100 AVR" },
+	{ "NOVA AVR", "1250", MGE_DEFAULT, "Nova 1250 AVR" },
 
 	/* EXtreme C (EMEA) */
 	{ "EXtreme", "700C", MGE_DEFAULT, "Pulsar EXtreme 700C" },
@@ -390,10 +436,8 @@ static models_name_t mge_model_names [] =
 	{ "GALAXY", "3000_20", MGE_DEFAULT, "Galaxy 3000 20 kVA" },
 	{ "GALAXY", "3000_30", MGE_DEFAULT, "Galaxy 3000 30 kVA" },
 
-	/* FIXME: To be completed (Comet, Galaxy, Esprit, ...) */
-
 	/* end of structure. */
-	{ NULL, NULL, MGE_DEFAULT, "Generic MGE HID model" }
+	{ NULL }
 };
 
 
@@ -403,10 +447,6 @@ static models_name_t mge_model_names [] =
 
 static hid_info_t mge_hid2nut[] =
 {
-	/* Server side variables */
-	{ "driver.version.internal", 0, 0, NULL, NULL, DRIVER_VERSION, HU_FLAG_ABSENT, NULL },
-	{ "driver.version.data", 0, 0, NULL, NULL, MGE_HID_VERSION, HU_FLAG_ABSENT, NULL },
-
 	/* Battery page */
 	{ "battery.charge", 0, 0, "UPS.PowerSummary.RemainingCapacity", NULL, "%.0f", 0, NULL },
 	{ "battery.charge.low", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerSummary.RemainingCapacityLimitSetting", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
@@ -429,9 +469,9 @@ static hid_info_t mge_hid2nut[] =
 	{ "ups.load.high", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.Flow.[4].ConfigPercentLoad", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "ups.delay.start", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerSummary.DelayBeforeStartup", NULL, DEFAULT_ONDELAY, HU_FLAG_ABSENT, NULL},
 	{ "ups.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerSummary.DelayBeforeShutdown", NULL, DEFAULT_OFFDELAY, HU_FLAG_ABSENT, NULL},
-	{ "ups.timer.start", 0, 0, "UPS.PowerSummary.DelayBeforeStartup", NULL, "%.0f", 0, NULL},
-	{ "ups.timer.shutdown", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", NULL, "%.0f", 0, NULL},
-	{ "ups.timer.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", NULL, "%.0f", 0, NULL},
+	{ "ups.timer.start", 0, 0, "UPS.PowerSummary.DelayBeforeStartup", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
+	{ "ups.timer.shutdown", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
+	{ "ups.timer.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
 	{ "ups.test.result", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "%s", HU_FLAG_SEMI_STATIC, test_read_info },
 	{ "ups.test.interval", ST_FLAG_RW | ST_FLAG_STRING, 8, "UPS.BatterySystem.Battery.TestPeriod", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "ups.beeper.status", 0 ,0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "%s", HU_FLAG_SEMI_STATIC, beeper_info },
@@ -543,9 +583,13 @@ static hid_info_t mge_hid2nut[] =
 	{ "output.powerfactor", 0, 0, "UPS.PowerConverter.Output.PowerFactor", NULL, "%s", 0, mge_powerfactor_conversion },
 
 	/* Outlet page (using MGE UPS SYSTEMS - PowerShare technology) */
-	{ "outlet.0.id", 0, 0, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
-	{ "outlet.0.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "Main Outlet", HU_FLAG_ABSENT, NULL },
-	{ "outlet.0.switchable", 0, 0, "UPS.OutletSystem.Outlet.[1].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
+	{ "outlet.id", 0, 0, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
+	{ "outlet.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "Main Outlet", HU_FLAG_ABSENT, NULL },
+	{ "outlet.switchable", 0, 0, "UPS.OutletSystem.Outlet.[1].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
+	/* The line below is the power consumption threshold on the master outlet
+	 * used as criteria to manage the slave outlets. Values: 10, 25 or 60 VA.
+	 * The default value is 25. */
+	{ "outlet.power", ST_FLAG_RW | ST_FLAG_STRING, 3, "UPS.OutletSystem.Outlet.[1].ConfigApparentPower", NULL, "%s", HU_FLAG_SEMI_STATIC, pegasus_threshold_info },
 	{ "outlet.1.id", 0, 0, "UPS.OutletSystem.Outlet.[2].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "outlet.1.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[2].OutletID", NULL, "PowerShare Outlet 1", HU_FLAG_ABSENT, NULL },
 	{ "outlet.1.switchable", 0, 0, "UPS.OutletSystem.Outlet.[2].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
@@ -557,7 +601,8 @@ static hid_info_t mge_hid2nut[] =
 	{ "outlet.1.delay.start", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.OutletSystem.Outlet.[2].StartupTimer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "outlet.2.id", 0, 0, "UPS.OutletSystem.Outlet.[3].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "outlet.2.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[3].OutletID", NULL, "PowerShare Outlet 2", HU_FLAG_ABSENT, NULL },
-	{ "outlet.2.switchable", 0, 0, "UPS.OutletSystem.Outlet.[3].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
+	/* needed for Pegasus to enable master/slave mode */
+	{ "outlet.2.switchable", ST_FLAG_RW | ST_FLAG_STRING, 3, "UPS.OutletSystem.Outlet.[3].PresentStatus.Switchable", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
 	{ "outlet.2.status", 0, 0, "UPS.OutletSystem.Outlet.[3].PresentStatus.SwitchOn/Off", NULL, "%s", 0, on_off_info },
 	{ "outlet.2.autoswitch.charge.low", ST_FLAG_RW | ST_FLAG_STRING, 3, "UPS.OutletSystem.Outlet.[3].RemainingCapacityLimit", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "outlet.2.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.OutletSystem.Outlet.[3].ShutdownTimer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
@@ -587,44 +632,56 @@ static hid_info_t mge_hid2nut[] =
 	{ "outlet.2.load.on", 0, 0, "UPS.OutletSystem.Outlet.[3].DelayBeforeStartup", NULL, "0", HU_TYPE_CMD, NULL },
 
 	/* end of structure. */
-	{ NULL, 0, 0, NULL, NULL, NULL, 0, NULL }
+	{ NULL }
 };
 
-/* All the logic for finely formatting the MGE model name */
-static const char *get_model_name(const char *iProduct, const char *iModel)
+/*
+ * All the logic for finely formatting the MGE model name and device
+ * type matching (used for device specific values or corrections).
+ * Returns pointer to (dynamically allocated) model name.
+ */
+static char *get_model_name(const char *iProduct, const char *iModel)
 {
 	models_name_t	*model = NULL;
 
 	upsdebugx(2, "get_model_name(%s, %s)\n", iProduct, iModel);
 
-	/* Search for formatting rules */
+	/* Search for device type and formatting rules */
 	for (model = mge_model_names; model->iProduct; model++) {
 		upsdebugx(2, "comparing with: %s", model->name);
 
-		if (strncmp(iProduct, model->iProduct, strlen(model->iProduct))) {
+		if (strcmp(iProduct, model->iProduct)) {
 			continue;
 		}
 
-		if (strncmp(iModel, model->iModel, strlen(model->iModel))) {
+		if (strcmp(iModel, model->iModel)) {
 			continue;
 		}
 
-		upsdebugx(2, "Found %s\n", model->name);
+		mge_type = model->type;
 		break;
 	}
 
-	mge_type = model->type;		/* enumerated model type */
+	if (!model->name) {
+		/*
+		 * Model not found or NULL (use default) so construct
+		 * model name by concatenation of iProduct and iModel
+		 */
+		char	buf[SMALLBUF];
+		snprintf(buf, sizeof(buf), "%s %s", iProduct, iModel);
+		return strdup(buf);
+	}
 
-	return model->name;
+	return strdup(model->name);
 }
 
 static char *mge_format_model(HIDDevice_t *hd) {
-	char	*product;
-	char	model[64];
+	char	product[SMALLBUF];
+	char	model[SMALLBUF];
 	double	value;
 
 	/* Get iProduct and iModel strings */
-	product = hd->Product ? hd->Product : "unknown";
+	snprintf(product, sizeof(product), "%s", hd->Product ? hd->Product : "unknown");
 
 	HIDGetItemString(udev, "UPS.PowerSummary.iModel", model, sizeof(model), mge_utab);
 
@@ -633,14 +690,11 @@ static char *mge_format_model(HIDDevice_t *hd) {
 		snprintf(model, sizeof(model), "%i", (int)value);
 	}
 
-	if (strlen(model) < 1) {
-		return product;
+	if (strlen(model) > 0) {
+		free(hd->Product);
+		hd->Product = get_model_name(product, model);
 	}
 
-	snprintf(model, sizeof(model), "%s", get_model_name(product, model));
-
-	free(hd->Product);
-	hd->Product = strdup(model);
 	return hd->Product;
 }
 
@@ -655,23 +709,31 @@ static char *mge_format_serial(HIDDevice_t *hd) {
 /* this function allows the subdriver to "claim" a device: return 1 if
  * the device is supported by this subdriver, else 0. */
 static int mge_claim(HIDDevice_t *hd) {
-	if (hd->VendorID != MGE_VENDORID) {
-		return 0;
-	}
-	switch (hd->ProductID)
-	{
-	case 0x0001:
-	case 0xffff:
-		return 1;	/* accept known UPSs */
 
-	default:
-		if (getval("productid")) {
+#ifndef SHUT_MODE
+	int status = is_usb_device_supported(mge_usb_device_table, hd->VendorID,
+								 hd->ProductID);
+
+	switch (status) {
+
+		case POSSIBLY_SUPPORTED:
+			/* by default, reject, unless the productid option is given */
+			if (getval("productid")) {
+				return 1;
+			}
+			possibly_supported("MGE", hd);
+			return 0;
+
+		case SUPPORTED:
 			return 1;
-		}
-		possibly_supported("MGE", hd);
-		return 0;
-		
+
+		case NOT_SUPPORTED:
+		default:
+			return 0;
 	}
+#else
+			return 1;
+#endif
 }
 
 subdriver_t mge_subdriver = {
