@@ -25,8 +25,6 @@
 #include "hidparser.h"
 #include "hidtypes.h"
 
-#define DRIVER_VERSION "0.66"
-
 #define DEFAULT_TIMEOUT 	3000
 #define MAX_STRING      	64
 
@@ -212,18 +210,30 @@ models_name_t models_names [] =
 	{ "PULSAR M", "2200", "Pulsar M 2200" },
 	{ "PULSAR M", "3000", "Pulsar M 3000" },
 	{ "PULSAR M", "3000 XL", "Pulsar M 3000 XL" },
+	/* Eaton'ified names */
+	{ "EX", "2200", "EX 2200" },
+	{ "EX", "3000", "EX 3000" },
+	{ "EX", "3000 XL", "EX 3000 XL" },
 	/* Pulsar models */
 	{ "Pulsar", "700", "Pulsar 700" },
 	{ "Pulsar", "1000", "Pulsar 1000" },
 	{ "Pulsar", "1500", "Pulsar 1500" },
 	{ "Pulsar", "1000 RT2U", "Pulsar 1000 RT2U" },
 	{ "Pulsar", "1500 RT2U", "Pulsar 1500 RT2U" },
+	/* Eaton'ified names */
+	{ "EX", "700", "EX 700" },
+	{ "EX", "1000", "EX 1000" },
+	{ "EX", "1500", "EX 1500" },
+	{ "EX", "1000 RT2U", "EX 1000 RT2U" },
+	{ "EX", "1500 RT2U", "EX 1500 RT2U" },
 	/* Pulsar MX models */
 	{ "PULSAR", "MX4000", "Pulsar MX 4000 RT" },
 	{ "PULSAR", "MX5000", "Pulsar MX 5000 RT" },
 	/* NOVA models */	
 	{ "NOVA AVR", "600", "NOVA 600 AVR" },
+	{ "NOVA AVR", "625", "Nova AVR 625" },
 	{ "NOVA AVR", "1100", "NOVA 1100 AVR" },
+	{ "NOVA AVR", "1250", "Nova AVR 1250" },
 	/* EXtreme C (EMEA) */
 	{ "EXtreme", "700C", "Pulsar EXtreme 700C" },
 	{ "EXtreme", "1000C", "Pulsar EXtreme 1000C" },
@@ -350,7 +360,6 @@ typedef struct {
 
 /* FIXME: should be shared with mgehid.h */
 static mge_info_item_t mge_info[] = {
-	{ "driver.version.internal", ST_FLAG_STRING, 5, NULL, "%s", DRIVER_VERSION, SHUT_FLAG_ABSENT | SHUT_FLAG_OK, NULL },
 	/* Battery page */
 	{ "battery.charge", 0, 0, "UPS.PowerSummary.RemainingCapacity", "%i", NULL, SHUT_FLAG_OK, NULL },
 	{ "battery.charge.low", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerSummary.RemainingCapacityLimitSetting",
@@ -383,10 +392,10 @@ static mge_info_item_t mge_info[] = {
 
 	/* Outlet page (using MGE UPS SYSTEMS - PowerShare technology) */
 	/* TODO: add an iterative semantic [%x] to factorise outlets */
-	{ "outlet.0.id", 0, 0, "UPS.OutletSystem.Outlet.[1].OutletID", "%i", NULL, SHUT_FLAG_OK, NULL },
-	{ "outlet.0.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[1].OutletID",
+	{ "outlet.id", 0, 0, "UPS.OutletSystem.Outlet.[1].OutletID", "%i", NULL, SHUT_FLAG_OK, NULL },
+	{ "outlet.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[1].OutletID",
 	  "%s", "Main Outlet", SHUT_FLAG_ABSENT | SHUT_FLAG_OK, NULL },
-	{ "outlet.0.switchable", 0, 0, "UPS.OutletSystem.Outlet.[1].PresentStatus.Switchable",
+	{ "outlet.switchable", 0, 0, "UPS.OutletSystem.Outlet.[1].PresentStatus.Switchable",
 	  "%i", NULL, SHUT_FLAG_OK, NULL },
 	{ "outlet.1.id", 0, 0, "UPS.OutletSystem.Outlet.[2].OutletID", "%i", NULL, SHUT_FLAG_OK, NULL },	
 	{ "outlet.1.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[2].OutletID",
