@@ -1,7 +1,7 @@
 /*
  * powercom.h - defines for the newpowercom.c driver
  *
- * $Id: powercom.h 294 2006-01-20 15:32:39Z keyson-guest $
+ * $Id: powercom.h 1240 2008-01-26 16:43:38Z clepple-guest $
  *
  * Copyrights:
  * (C) 2002 Simon Rozman <simon@rozman.net>
@@ -34,41 +34,6 @@
 /* nut includes */
 #include "timehead.h"
 
-/* general constants */
-enum general {
-	MAX_NUM_OF_BYTES_FROM_UPS = 16
-};
-
-/* values for sending to UPS */
-enum commands {
-	SEND_DATA    = '\x01',
-	BATTERY_TEST = '\x03',
-	SHUTDOWN     = '\xbc'
-};
-
-/* location of data in received string */
-enum data {
-	UPS_LOAD         = 0U,
-	BATTERY_CHARGE   = 1U,
-	INPUT_VOLTAGE    = 2U,
-	OUTPUT_VOLTAGE   = 3U,
-	INPUT_FREQUENCY  = 4U,
-	OUTPUT_FREQUENCY = 6U
-};
-
-/* status bits */
-enum status {
-	SUMMARY       = 0U,
-	MAINS_FAILURE = 1U,
-	LOW_BAT       = 2U, 
-	BAD_BAT       = 2U,
-	TEST          = 4U,
-	AVR_ON        = 8U,
-	OVERLOAD      = 32U,
-	OFF           = 128U,
-	STATUS_A      = 9U,
-	STATUS_B      = 10U
-};
 
 /* supported types */
 struct type {
@@ -94,7 +59,7 @@ struct type {
 	 */
 	
 	/* Some UPSs must have a minutes and a seconds arguments for 
-	 * the SHUTDOWN commands while others are known to work with the
+	 * the COUNTER commands while others are known to work with the
 	 * seconds argument alone.
 	 */
 	struct deley_for_power_kill {
