@@ -1,43 +1,39 @@
-/* powerp-bin.c	Model specific routines for CyberPower binary
-			protocol UPSes 
-
-   Copyright (C)
-	2007		Doug Reynolds <mav@wastegate.net>
-	2007-2008	Arjen de Korte <adkorte-guest@alioth.debian.org>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-*/
-
 /*
-   Throughout this driver, READ and WRITE comments are shown. These are
-   the typical commands to and replies from the UPS that was used for
-   decoding the protocol (with a serial logger).
+ * powerp-bin.c - Model specific routines for CyberPower binary
+ *                protocol UPSes 
+ *
+ * Copyright (C)
+ *	2007        Doug Reynolds <mav@wastegate.net>
+ *	2007-2008   Arjen de Korte <adkorte-guest@alioth.debian.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <math.h>
+/*
+ * Throughout this driver, READ and WRITE comments are shown. These are
+ * the typical commands to and replies from the UPS that was used for
+ * decoding the protocol (with a serial logger).
+ */
 
 #include "main.h"
 #include "serial.h"
 
-#include "powerpanel.h"
 #include "powerp-bin.h"
 
+#include <math.h>
+
 typedef struct {
 	unsigned char	start;
 	unsigned char	i_volt;
@@ -68,34 +64,34 @@ static unsigned char	powpan_answer[SMALLBUF];
 static const valtab_t	tran_high[] = {
 	{ "138", -9 }, { "139", -8 }, { "140", -7 }, { "141", -6 }, { "142", -5 },
 	{ "143", -4 }, { "144", -3 }, { "145", -2 }, { "146", -1 }, { "147",  0 },
-	{ NULL, 0 }
+	{ NULL }
 };
 
 /* OP series */
 static const valtab_t	tran_high_op[] = {
 	{ "140", -5 }, { "141", -4 }, { "142", -3 }, { "143", -2 }, { "144", -1 },
 	{ "145",  0 }, { "146", +1 }, { "147", +2 }, { "148", +3 }, { "149", +4 },
-	{ "150", +5 }, { NULL, 0 }
+	{ "150", +5 }, { NULL }
 };
 
 /* default */
 static const valtab_t	tran_low[] = {
 	{ "88",  0 }, { "89", +1 }, { "90", +2 }, { "91", +3 }, { "92", +4 },
 	{ "93", +5 }, { "94", +6 }, { "95", +7 }, { "96", +8 }, { "97", +9 },
-	{ NULL, 0 }
+	{ NULL }
 };
 
 /* OP series */
 static const valtab_t	tran_low_op[] = {
 	{ "85", -5 }, { "86", -4 }, { "87", -3 }, { "88", -2 }, { "89", -1 },
 	{ "90",  0 }, { "91", +1 }, { "92", +2 }, { "93", +3 }, { "94", +4 },
-	{ "95", +5 }, { NULL, 0 }
+	{ "95", +5 }, { NULL }
 };
 
 /* default */
 static const valtab_t	batt_low[] = {
 	{ "25", -6 }, { "30", -5 }, { "35", -3 }, { "40", -1 }, { "45",  0 },
-	{ "50", +2 }, { "55", +4 }, { "60", +6 }, { NULL, 0 }
+	{ "50", +2 }, { "55", +4 }, { "60", +6 }, { NULL }
 };
 
 /* OP series */
@@ -103,23 +99,23 @@ static const valtab_t	batt_low_op[] = {
 	{ "15", -8 }, { "18", -7 }, { "19", -6 }, { "20", -5 }, { "22", -4 },
 	{ "24", -3 }, { "25", -2 }, { "26", -1 }, { "28",  0 }, { "30", +1 },
 	{ "32", +2 }, { "34", +3 }, { "35", +4 }, { "36", +5 }, { "38", +6 },
-	{ "40", +7 }, { NULL, 0 }
+	{ "40", +7 }, { NULL }
 };
 
 /* default */
 static const valtab_t	out_volt[] = {
-	{ "110", -10 }, { "120",  0 }, { "130", +10 }, { NULL, 0 }
+	{ "110", -10 }, { "120",  0 }, { "130", +10 }, { NULL }
 };
 
 /* OP series */
 static const valtab_t	out_volt_op[] = {
 	{ "110", -10 }, { "115", -5 }, { "120",  0 }, { "124", +4 }, { "128", +8 },
-	{ "130", +10 }, { NULL, 0 }
+	{ "130", +10 }, { NULL }
 };
 
 static const valtab_t 	yes_no_info[] = {
 	{ "yes", 2 }, { "no", 0 },
-	{ NULL, 0 }
+	{ NULL }
 };
 
 static const struct {
@@ -135,14 +131,14 @@ static const struct {
 		{ "battery.charge.low", "R\010\r", "Q\010%c\r", batt_low },
 		{ "output.voltage.nominal", "R\030\r", "Q\030%c\r", out_volt },
 		{ "ups.start.battery", "R\017\r", "Q\017%c\r", yes_no_info },
-		{ NULL, NULL, NULL, NULL }
+		{ NULL }
 	}, {
 		/* OP series */
 		{ "input.transfer.high", "R\002\r", "Q\002%c\r", tran_high_op },
 		{ "input.transfer.low", "R\004\r", "Q\004%c\r", tran_low_op },
 		{ "battery.charge.low", "R\010\r", "Q\010%c\r", batt_low_op },
 		{ "output.voltage.nominal", "R\030\r", "Q\030%c\r", out_volt_op },
-		{ NULL, NULL, NULL, NULL }
+		{ NULL }
 	}	
 };
 
@@ -151,16 +147,15 @@ static const struct {
 	char	*command;
 	int	len;
 } cmdtab[] = {
-	{ "test.failure.start", "T\230\r", 3 },		/* 20 seconds test */
-	{ "test.failure.stop", "CT\r", 3 },
+	{ "test.battery.start.quick", "T\230\r", 3 },		/* 20 seconds test */
+	{ "test.battery.stop", "CT\r", 3 },
 	{ "beeper.toggle", "B\r", 2 },
 	{ "shutdown.reboot", "S\0\0R\0\1W\r", 8},
-	{ "shutdown.return", "S\0\0W\r", 5 },
+	/* the shutdown.stayoff command behaves
+	 * as shutdown.return when on battery */
+	{ "shutdown.stayoff", "S\0\0W\r", 5 },
 	{ "shutdown.stop", "C\r", 2 },
-/*
-	{ "shutdown.stayoff", "S\0\0\W\r", 5 },
- */
-	{ NULL, NULL, 0 }
+	{ NULL }
 };
 
 /* map UPS data to (approximated) input/output voltage */
@@ -205,20 +200,33 @@ static int powpan_command(const char *buf, size_t bufsize)
 
 	ser_flush_io(upsfd);
 
-	upsdebug_hex(3, "send", buf, bufsize);
+	ret = ser_send_buf_pace(upsfd, UPSDELAY, buf, bufsize);
 
-	ret = ser_send_buf_pace(upsfd, UPSDELAY, (unsigned char *)buf, bufsize);
-
-	if (ret < (int)bufsize) {
+	if (ret < 0) {
+		upsdebug_with_errno(3, "send");
 		return -1;
 	}
 
+	if (ret == 0) {
+		upsdebug_with_errno(3, "send: timeout");
+		return -1;
+	}
+
+	upsdebug_hex(3, "send", buf, bufsize);
+
 	usleep(100000);
 
-	ret = ser_get_buf_len(upsfd, powpan_answer, bufsize - 1, SER_WAIT_SEC, SER_WAIT_USEC);
+	ret = ser_get_buf_len(upsfd, powpan_answer, bufsize-1, SER_WAIT_SEC, SER_WAIT_USEC);
 
-	if (ret < 1) {
-		upsdebugx(3, "read: timed out");
+	if (ret < 0) {
+		upsdebug_with_errno(3, "read");
+		upsdebug_hex(4, "  \\_", buf, bufsize-1);
+		return -1;
+	}
+
+	if (ret == 0) {
+		upsdebugx(3, "read: timeout");
+		upsdebug_hex(4, "  \\_", buf, bufsize-1);
 		return -1;
 	}
 
@@ -242,11 +250,11 @@ static int powpan_instcmd(const char *cmdname, const char *extra)
 			return STAT_INSTCMD_HANDLED;
 		}
 
-		upslogx(LOG_ERR, "powpan_instcmd: command [%s] failed", cmdname);
-		return STAT_INSTCMD_UNKNOWN;
+		upslogx(LOG_ERR, "%s: command [%s] failed", __func__, cmdname);
+		return STAT_INSTCMD_FAILED;
 	}
 
-	upslogx(LOG_NOTICE, "powpan_instcmd: command [%s] unknown", cmdname);
+	upslogx(LOG_ERR, "%s: command [%s] not found", __func__, cmdname);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -276,7 +284,7 @@ static int powpan_setvar(const char *varname, const char *val)
 				vartab[type][i].map[j].command);
 
 			if ((powpan_command(command, 4) == 3) && (!memcmp(powpan_answer, command, 3))) {
-				dstate_setinfo(varname, val);
+				dstate_setinfo(varname, "%s", val);
 				return STAT_SET_HANDLED;
 			}
 
@@ -297,6 +305,9 @@ static void powpan_initinfo()
 	int	i, j;
 	char	*s;
 
+	dstate_setinfo("ups.delay.start", "%d", 45);
+	dstate_setinfo("ups.delay.shutdown", "%d", 0);	/* almost immediate */
+
 	/*
 	 * NOTE: The reply is already in the buffer, since the F\r command
 	 * was used for autodetection of the UPS. No need to do it again.
@@ -330,7 +341,7 @@ static void powpan_initinfo()
 				continue;
 			}
 
-			dstate_setinfo(vartab[type][i].var, vartab[type][i].map[j].val);
+			dstate_setinfo(vartab[type][i].var, "%s", vartab[type][i].map[j].val);
 			break;
 		}
 	
@@ -343,12 +354,12 @@ static void powpan_initinfo()
 		dstate_setflags(vartab[type][i].var, ST_FLAG_RW);
 
 		for (j = 0; vartab[type][i].map[j].val != 0; j++) {
-			dstate_addenum(vartab[type][i].var, vartab[type][i].map[j].val);
+			dstate_addenum(vartab[type][i].var, "%s", vartab[type][i].map[j].val);
 		}
 	}
 
-	upsh.instcmd = powpan_instcmd;
-	upsh.setvar = powpan_setvar;
+	dstate_addcmd("shutdown.stayoff");
+	dstate_addcmd("shutdown.reboot");
 }
 
 static int powpan_status(status_t *status)
@@ -362,49 +373,57 @@ static int powpan_status(status_t *status)
 	 * READ #VVL.CTF.....\r
         *      01234567890123
 	 */
-	upsdebug_hex(3, "send", "D\r", 2);
-
 	ret = ser_send_pace(upsfd, UPSDELAY, "D\r");
-	if (ret < 2) {
+
+	if (ret < 0) {
+		upsdebug_with_errno(3, "send");
 		return -1;
 	}
 
+	if (ret == 0) {
+		upsdebug_with_errno(3, "send: timeout");
+		return -1;
+	}
+
+	upsdebug_hex(3, "send", "D\r", 2);
+
 	usleep(200000);
 
-	ret = ser_get_buf_len(upsfd, (unsigned char *)status, sizeof(*status), SER_WAIT_SEC, SER_WAIT_USEC);
-	if (ret < 1) {
-		upsdebugx(3, "read: timed out");
+	ret = ser_get_buf_len(upsfd, status, sizeof(*status), SER_WAIT_SEC, SER_WAIT_USEC);
+
+	if (ret < 0) {
+		upsdebug_with_errno(3, "read");
+		upsdebug_hex(4, "  \\_", status, sizeof(*status));
+		return -1;
+	}
+
+	if (ret == 0) {
+		upsdebugx(3, "read: timeout");
+		upsdebug_hex(4, "  \\_", status, sizeof(*status));
 		return -1;
 	}
 
 	upsdebug_hex(3, "read", status, ret);
 
-	if (ret < (int)sizeof(*status)) {
-		upsdebugx(4, "Short status read");
-		return -1;
-	}
-	
 	if ((status->flags[0] + status->flags[1]) != 255) {
-		upsdebugx(4, "Checksum flags[0..1] failed");
+		upsdebugx(4, "  \\_ : checksum flags[0..1] failed");
 		return -1;
 	}
 
 	if ((status->flags[2] + status->flags[3]) != 255) {
-		upsdebugx(4, "Checksum flags[2..3] failed");
+		upsdebugx(4, "  \\_ : checksum flags[2..3] failed");
 		return -1;
 	}
 
 	return 0;
 }
 
-static void powpan_updateinfo()
+static int powpan_updateinfo()
 {
 	status_t	status;
 
 	if (powpan_status(&status)) {
-		ser_comm_fail("Status read failed!");
-		dstate_datastale();
-		return;
+		return -1;
 	}
 
 	switch (type)
@@ -453,12 +472,17 @@ static void powpan_updateinfo()
 
 	/* !OB && !TEST */
 	if (!(status.flags[0] & 0x84)) {
-		if ((status.i_volt > 0.50 * status.o_volt) && (status.i_volt < 0.98 * status.o_volt)) {
-			status_set("BOOST");
-		}
 
-		if ((status.o_volt > 0.50 * status.i_volt) && (status.o_volt < 0.98 * status.i_volt)) {
+		if (status.o_volt < 0.5 * status.i_volt) {
+			upsdebugx(2, "%s: output voltage too low", __func__);
+		} else if (status.o_volt < 0.95 * status.i_volt) {
 			status_set("TRIM");
+		} else if (status.o_volt < 1.05 * status.i_volt) {
+			upsdebugx(2, "%s: appears to be in BYPASS state", __func__);
+		} else if (status.o_volt < 1.5 * status.i_volt) {
+			status_set("BOOST"); 
+		} else {
+			upsdebugx(2, "%s: output voltage too high", __func__);
 		}
 	}
 
@@ -472,78 +496,57 @@ static void powpan_updateinfo()
 
 	status_commit();
 
-	ser_comm_good();
-	dstate_dataok();
-
-	return;
-}
-
-static void powpan_shutdown()
-{
-	status_t	status;
-	int		i;
-
-	for (i = 0; i < MAXTRIES; i++) {
-
-		if (powpan_status(&status)) {
-			continue;
-		}
-
-		/*
-		 * We're still on battery...
-		 */
-		if (status.flags[0] & 0x80) {
-			break;
-		}
- 
-		/*
-		 * Apparently, the power came back already, so just reboot.
-		 */
-		if (powpan_instcmd("shutdown.reboot", NULL) == STAT_INSTCMD_HANDLED) {
-			upslogx(LOG_INFO, "Rebooting now...");
-			return;
-		}
-	}
-
-	for (i = 0; i < MAXTRIES; i++) {
-
-		/*
-		 * ...send wait for return.
-		 */
-		if (powpan_instcmd("shutdown.return", NULL) == STAT_INSTCMD_HANDLED) {
-			upslogx(LOG_INFO, "Waiting for power to return...");
-			return;
-		}
-	}
-
-	upslogx(LOG_ERR, "Shutdown command failed!");
+	return (status.flags[0] & 0x80) ? 1 : 0;
 }
 
 static int powpan_initups()
 {
 	int	ret, i;
 
-	upsdebugx(1, "Trying binary protocol...");
+	upsdebugx(1, "Trying %s protocol...", powpan_binary.version);
 
 	ser_set_speed(upsfd, device_path, B1200);
 
+	/* This fails for many devices, so don't bother to complain */
 	ser_send_pace(upsfd, UPSDELAY, "\r\r");
 
 	for (i = 0; i < MAXTRIES; i++) {
 
+		ser_flush_io(upsfd);
+
 		/*
 		 * WRITE F\r
 		 * READ .PR2200    .x.<.1100
 		 *      01234567890123456789
 		 */
-		ser_flush_io(upsfd);
-		ser_send_pace(upsfd, UPSDELAY, "F\r");
-		usleep(100000);
-		ret = ser_get_line(upsfd, (char *)powpan_answer, sizeof(powpan_answer),
+		ret = ser_send_pace(upsfd, UPSDELAY, "F\r");
+
+		if (ret < 0) {
+			upsdebug_with_errno(3, "send");
+			continue;
+		}
+
+		if (ret == 0) {
+			upsdebug_with_errno(3, "send: timeout");
+			continue;
+		}
+
+		upsdebug_hex(3, "send", "F\r", 2);
+
+		usleep(200000);
+
+		ret = ser_get_line(upsfd, powpan_answer, sizeof(powpan_answer),
 			ENDCHAR, IGNCHAR, SER_WAIT_SEC, SER_WAIT_USEC);
 
-		if (ret < 1) {
-			upsdebugx(3, "read: timed out");
+		if (ret < 0) {
+			upsdebug_with_errno(3, "read");
+			upsdebug_hex(4, "  \\_", powpan_answer, strlen((char *)powpan_answer));
+			continue;
+		}
+
+		if (ret == 0) {
+			upsdebugx(3, "read: timeout");
+			upsdebug_hex(4, "  \\_", powpan_answer, strlen((char *)powpan_answer));
 			continue;
 		}
 
@@ -564,6 +567,14 @@ static int powpan_initups()
 			type = OP;
 		}
 
+		if (getval("ondelay")) {
+			fatalx(EXIT_FAILURE, "Setting 'ondelay' not supported by %s driver", powpan_binary.version);
+		}
+
+		if (getval("offdelay")) {
+			fatalx(EXIT_FAILURE, "Setting 'offdelay' not supported by %s driver", powpan_binary.version);
+		}
+
 		return ret;
 	}
 
@@ -572,8 +583,9 @@ static int powpan_initups()
 
 subdriver_t powpan_binary = {
 	"binary",
+	powpan_instcmd,
+	powpan_setvar,
 	powpan_initups,
 	powpan_initinfo,
-	powpan_updateinfo,
-	powpan_shutdown
+	powpan_updateinfo
 };
