@@ -11,42 +11,15 @@
  * - others using SafeNet software and serial interface
  *
  * Status:
- *  20031015/Revision 0.1 - Arjen de Korte <arjen@de-korte.org>
- *   - initial release (entirely based on reverse engineering the
- *     serial data stream)
- *  20031022/Revision 0.2 - Arjen de Korte <arjen@de-korte.org>
- *   - status polling command is now "random" (just like the
- *     SafeNet (Windows) driver
- *   - low battery status is "LB" (not "BL")
- *  20031228/Revision 0.3 - Arjen de Korte <arjen@de-korte.org>
- *   - instant command 'shutdown.return' added
- *   - added 'manufacturer', 'modelname' and 'serialnumber'
- *     commandline parameters
- *   - removed experimental driver flag
- *   - documentation & code cleanup
- *  20060108/Revision 0.4 - Arjen de Korte <arjen@de-korte.org>
- *   - minor changes to make sure state changes are not missed
- *   - log errors when instant commands can't be handled
- *   - crude hardware detection which looks for DSR=1
- *  20060128/Revision 0.5 - Arjen de Korte <arjen@de-korte.org>
- *   - removed TRUE/FALSE defines, which were not really
- *     improving the code anyway
- *  20060131/Revision 1.0 - Arjen de Korte <arjen@de-korte.org>
- *   - put all UPS commands in header file (for easy retrieval)
- *   - remove log errors when instant commands can't be handled
- *     (doesn't work reliably on at least one flavor)
- *  20061229/Revision 1.1 - Arjen de Korte <arjen@de-korte.org>
- *   - flush the serial input buffer before sending commands
- *     (to get rid of unsollicited serial PnP replies)
- *  20070211/Revision 1.2 - Arjen de Korte <arjen@de-korte.org>
- *   - wait for three consecutive failed polls before declaring
- *     data stale in order not to bother clients with temporary
- *     problems
- *  20070304/Revision 1.3 - Arjen de Korte <arjen@de-korte.org>
- *   - in battery test mode (CAL state) stop the test when the
- *     the low battery state is reached
+ *  20081102/Revision 1.41 - Arjen de Korte <adkorte-guest@alioth.debian.org>
+ *   - allow more time for reading reply to command
+ *  20081106/Revision 1.5 - Arjen de Korte <adkorte-guest@alioth.debian.org>
+ *   - changed communication with UPS
+ *   - improved handling of battery & system test
+ *  20081228/Revision 1.6 - Arjen de Korte <adkorte-guest@alioth.debian.org>
+ *   - add ondelay and offdelay
  *
- * Copyright (C) 2003-2006  Arjen de Korte <arjen@de-korte.org>
+ * Copyright (C) 2003-2008  Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,14 +36,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-#include <stdlib.h>
-#include <ctype.h>
-
 #include "main.h"
 #include "serial.h"
 #include "safenet.h"
 
-#define DRV_VERSION	"1.4"
+#define DRIVER_NAME	"Generic SafeNet UPS driver"
+#define DRIVER_VERSION	"1.6"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Arjen de Korte <adkorte-guest@alioth.debian.org>",
+	DRV_STABLE,
+	{ NULL }
+};
 
 /*
  * Here we keep the last known status of the UPS
@@ -80,10 +60,13 @@ static union	{
 	struct safenet		status;
 } ups;
 
+static int ondelay = 1;		/* minutes */
+static int offdelay = 30;	/* seconds */
+
 static int safenet_command(const char *command)
 {
 	char	reply[32];
-	int	i;
+	int	i, ret;
 
 	/*
 	 * Get rid of whatever is in the in- and output buffers.
@@ -94,29 +77,49 @@ static int safenet_command(const char *command)
 	 * Send the command and read back the status line. When we just send
 	 * a status polling command, it will return the actual status.
 	 */
-	ser_send_pace(upsfd, 10000, command);
-	upsdebugx(3, "UPS command %s", command);
+	ret = ser_send(upsfd, "%s", command);
+
+	if (ret < 0) {
+		upsdebug_with_errno(3, "send");
+		return -1;
+	}
+
+	if (ret == 0) {
+		upsdebug_with_errno(3, "send: timeout");
+		return -1;
+	}
+
+	upsdebugx(3, "send: %.*s", strcspn(command, "\r"), command);
 
 	/*
-	 * Read the reply from the UPS. Allow twice the time needed to read 12
-	 * bytes (120 bits (including start & stop bits) / 1200 baud = 100ms).
+	 * Read the reply from the UPS.
 	 */
-	ser_get_line(upsfd, reply, sizeof(reply), '\r', "", 0, 200000);
-	upsdebugx(3, "UPS answers %s", (strlen(reply)>0) ? reply : "[INVALID]");
+	ret = ser_get_buf(upsfd, reply, sizeof(reply), 1, 0);
+
+	if (ret < 0) {
+		upsdebug_with_errno(3, "read");
+		return -1;
+	}
+
+	if (ret == 0) {
+		upsdebugx(3, "read: timeout");
+		return -1;
+	}
+
+	upsdebugx(3, "read: %.*s", strcspn(reply, "\r"), reply);
 
 	/*
 	 * We check if the reply looks like a valid status.
 	 */
-
-	if ((strlen(reply) != 11) || (reply[0] != '$') || (strspn(reply+1, "AB") != 10)) {
-		return(-1);
+	if ((ret != 12) || (reply[0] != '$') || (strspn(reply+1, "AB") != 10)) {
+		return -1;
 	}
 
-	for (i=0; i<10; i++) {
+	for (i = 0; i < 10; i++) {
 		ups.reply[i] = ((reply[i+1] == 'B') ? 1 : 0);
 	}
 
-	return(0);
+	return 0;
 }
 
 static void safenet_update()
@@ -141,27 +144,18 @@ static void safenet_update()
 		status_set("RB");
 	}
 
-	/*
-	 * This is not actually an indication of the UPS in the BYPASS state, it is
-	 * an indication that the UPS failed the self diagnostics (which is NOT the
-	 * same as a replace battery warning). Since the result is the same (no
-	 * backup in case of mains failure) we give it this status. Introduce a new
-	 * status here?
-	 */
-	if (ups.status.systemfail) {
-		status_set("BYPASS");
-	}
-
-	/*
-	 * This is not actually an indication of the UPS in the CAL state, it is an
-	 * indication that the UPS is in the system test state. The result of this
-	 * test may indicate a battery failure, so this is the closest we can get.
-	 * Introduce a new status here?
-	 */
 	if (ups.status.systemtest) {
 		status_set("CAL");
 	}
 
+	alarm_init();
+
+	if (ups.status.systemfail) {
+		alarm_set("System selftest fail!");
+	}
+
+	alarm_commit();
+
 	status_commit();
 }
 
@@ -185,43 +179,55 @@ static int instcmd(const char *cmdname, const char *extra)
 	 * Start the UPS selftest
 	 */
 	if (!strcasecmp(cmdname, "test.battery.start")) {
-		safenet_command(COM_BATT_TEST);
-		return STAT_INSTCMD_HANDLED;
+		if (safenet_command(COM_BATT_TEST)) {
+			return STAT_INSTCMD_FAILED;
+		} else {
+			return STAT_INSTCMD_HANDLED;
+		}
 	}
 
 	/*
 	 * Stop the UPS selftest
 	 */
 	if (!strcasecmp(cmdname, "test.battery.stop")) {
-		safenet_command(COM_STOP_TEST);
-		return STAT_INSTCMD_HANDLED;
+		if (safenet_command(COM_STOP_TEST)) {
+			return STAT_INSTCMD_FAILED;
+		} else {
+			return STAT_INSTCMD_HANDLED;
+		}
 	}
 
 	/*
 	 * Start simulated mains failure
 	 */
 	if (!strcasecmp (cmdname, "test.failure.start")) {
-		safenet_command(COM_MAINS_TEST);
-		return STAT_INSTCMD_HANDLED;
+		if (safenet_command(COM_MAINS_TEST)) {
+			return STAT_INSTCMD_FAILED;
+		} else {
+			return STAT_INSTCMD_HANDLED;
+		}
 	}
 
 	/*
 	 * Stop simulated mains failure
 	 */
 	if (!strcasecmp (cmdname, "test.failure.stop")) {
-		safenet_command(COM_STOP_TEST);
-		return STAT_INSTCMD_HANDLED;
+		if (safenet_command(COM_STOP_TEST)) {
+			return STAT_INSTCMD_FAILED;
+		} else {
+			return STAT_INSTCMD_HANDLED;
+		}
 	}
 
 	/*
 	 * If beeper is off, toggle beeper state (so it should be ON after this)
 	 */
 	if (!strcasecmp(cmdname, "beeper.enable")) {
-		if (ups.status.silenced) {
-			safenet_command(COM_TOGGLE_BEEP);
+		if (ups.status.silenced && safenet_command(COM_TOGGLE_BEEP)) {
+			return STAT_INSTCMD_FAILED;
+		} else {
+			return STAT_INSTCMD_HANDLED;
 		}
-
-		return STAT_INSTCMD_HANDLED;
 	}
 
 	/*
@@ -230,43 +236,55 @@ static int instcmd(const char *cmdname, const char *extra)
 	 * event automatically (no way to stop this, besides side cutters)
 	 */
 	if (!strcasecmp(cmdname, "beeper.mute")) {
-		if (!ups.status.silenced) {
-			safenet_command(COM_TOGGLE_BEEP);
+		if (!ups.status.silenced && safenet_command(COM_TOGGLE_BEEP)) {
+			return STAT_INSTCMD_FAILED;
+		} else {
+			return STAT_INSTCMD_HANDLED;
 		}
-
-		return STAT_INSTCMD_HANDLED;
 	}
 
 	/*
 	 * Toggle beeper state unconditionally
 	 */
 	if (!strcasecmp(cmdname, "beeper.toggle")) {
-		safenet_command(COM_TOGGLE_BEEP);
+		if (safenet_command(COM_TOGGLE_BEEP)) {
+			return STAT_INSTCMD_FAILED;
+		} else {
+			return STAT_INSTCMD_HANDLED;
+		}
 
-		return STAT_INSTCMD_HANDLED;
 	}
 
 	/*
-	 * Shutdown immediately and wait for the power to return
+	 * Shutdown and wait for the power to return
 	 */
 	if (!strcasecmp(cmdname, "shutdown.return")) {
-		safenet_command(SHUTDOWN_RETURN);
+		char	command[] = SHUTDOWN_RETURN;
+		
+		command[4] += ((offdelay % 1000) / 100);
+		command[5] += ((offdelay % 100) / 10);
+		command[6] +=  (offdelay % 10);
+
+		safenet_command(command);
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	/*
-	 * Shutdown immediately and reboot after 1 minute
+	 * Shutdown and reboot
 	 */
 	if (!strcasecmp(cmdname, "shutdown.reboot")) {
-		safenet_command(SHUTDOWN_REBOOT);
-		return STAT_INSTCMD_HANDLED;
-	}
+		char	command[] = SHUTDOWN_REBOOT;
 
-	/*
-	 * Shutdown in 20 seconds and reboot after 1 minute
-	 */
-	if (!strcasecmp(cmdname, "shutdown.reboot.graceful")) {
-		safenet_command(GRACEFUL_REBOOT);
+		command[3] += ((offdelay % 1000) / 100);
+		command[4] += ((offdelay % 100) / 10);
+		command[5] +=  (offdelay % 10);
+
+		command[7] += ((ondelay % 10000) / 1000);
+		command[8] += ((ondelay % 1000) / 100);
+		command[9] += ((ondelay % 100) / 10);
+		command[10] += (ondelay % 10);
+
+		safenet_command(command);
 		return STAT_INSTCMD_HANDLED;
 	}
 
@@ -279,7 +297,7 @@ void upsdrv_initinfo(void)
 	int	retry = 3;
 	char	*v;
 
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
+	dstate_setinfo("driver.version.internal", "%s", DRIVER_VERSION);
 	
 	usleep(100000);
 
@@ -312,6 +330,9 @@ void upsdrv_initinfo(void)
 	dstate_setinfo("ups.model", "%s", ((v = getval("modelname")) != NULL) ? v : "unknown");
 	dstate_setinfo("ups.serial", "%s", ((v = getval("serialnumber")) != NULL) ? v : "unknown");
 
+	dstate_setinfo("ups.delay.start", "%d", 60 * ondelay);
+	dstate_setinfo("ups.delay.shutdown", "%d", offdelay);
+
 	/*
 	 * These are the instant commands we support.
 	 */
@@ -326,7 +347,6 @@ void upsdrv_initinfo(void)
 	dstate_addcmd ("beeper.toggle");
 	dstate_addcmd ("shutdown.return");
 	dstate_addcmd ("shutdown.reboot");
-	dstate_addcmd ("shutdown.reboot.graceful");
 
 	upsh.instcmd = instcmd;
 }
@@ -380,7 +400,17 @@ void upsdrv_updateinfo(void)
 	retry = 0;
 
 	if (ups.status.systemtest && ups.status.batterylow) {
-		safenet_command(COM_STOP_TEST);
+
+		/*
+		 * Don't update status after stopping battery test, to
+		 * allow UPS to update the status flags (OB+LB glitch)
+		 */
+		if (safenet_command(COM_STOP_TEST)) {
+			upslogx(LOG_WARNING, "Can't terminate battery test!");
+		} else {
+			upslogx(LOG_INFO, "Battery test finished");
+			return;
+		}
 	}
 
 	safenet_update();
@@ -414,9 +444,9 @@ void upsdrv_shutdown(void)
 	 * we need to check the status of the UPS here.
 	 */
 	if (ups.status.onbattery) {
-		safenet_command(SHUTDOWN_RETURN);
+		instcmd("shutdown.return", NULL);
 	} else {
-		safenet_command(SHUTDOWN_REBOOT);
+		instcmd("shutdown.reboot", NULL);
 	}
 }
 
@@ -429,26 +459,68 @@ void upsdrv_makevartable(void)
 	addvar(VAR_VALUE, "manufacturer", "manufacturer [unknown]");
 	addvar(VAR_VALUE, "modelname", "modelname [unknown]");
 	addvar(VAR_VALUE, "serialnumber", "serialnumber [unknown]");
-}
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Generic SafeNet UPS driver %s (%s)\n\n", DRV_VERSION, UPS_VERSION);
+	addvar(VAR_VALUE, "ondelay", "Delay before UPS startup (minutes)");
+	addvar(VAR_VALUE, "offdelay", "Delay before UPS shutdown (seconds)");
 }
 
 void upsdrv_initups(void)
 {
+	struct termios	tio;
+	const char	*val;
+
 	/*
 	 * Open and lock the serial port and set the speed to 1200 baud.
 	 */
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B1200);
 
+	if (tcgetattr(upsfd, &tio)) {
+		fatal_with_errno(EXIT_FAILURE, "tcgetattr");
+	}
+
+	/*
+	 * Use canonical mode input processing (to read reply line)
+	 */
+	tio.c_lflag |= ICANON;	/* Canonical input (erase and kill processing) */
+
+	tio.c_cc[VEOF] = _POSIX_VDISABLE;
+	tio.c_cc[VEOL] = '\r';
+	tio.c_cc[VERASE] = _POSIX_VDISABLE;
+	tio.c_cc[VINTR]  = _POSIX_VDISABLE;
+	tio.c_cc[VKILL]  = _POSIX_VDISABLE;
+	tio.c_cc[VQUIT]  = _POSIX_VDISABLE;
+	tio.c_cc[VSUSP]  = _POSIX_VDISABLE;
+	tio.c_cc[VSTART] = _POSIX_VDISABLE;
+	tio.c_cc[VSTOP]  = _POSIX_VDISABLE;
+
+	if (tcsetattr(upsfd, TCSANOW, &tio)) {
+		fatal_with_errno(EXIT_FAILURE, "tcsetattr");
+	}
+
 	/*
 	 * Set DTR and clear RTS to provide power for the serial interface.
 	 */
 	ser_set_dtr(upsfd, 1);
 	ser_set_rts(upsfd, 0);
+
+	val = getval("ondelay");
+	if (val) {
+		ondelay = strtol(val, NULL, 10);
+	}
+
+	if ((ondelay < 0) || (ondelay > 9999)) {
+		fatalx(EXIT_FAILURE, "Start delay '%d' out of range [0..9999]", ondelay);
+	}
+
+	val = getval("offdelay");
+	if (val) {
+		offdelay = strtol(val, NULL, 10);
+	}
+
+	if ((offdelay < 0) || (offdelay > 999)) {
+		fatalx(EXIT_FAILURE, "Shutdown delay '%d' out of range [0..999]", offdelay);
+	}
 }
 
 void upsdrv_cleanup(void)
