@@ -19,8 +19,8 @@
 
 #include "common.h"
 #include "timehead.h"
-
 #include "serial.h"
+#include "main.h"
 
 #include <grp.h>
 #include <pwd.h>
@@ -34,10 +34,7 @@
 #include <libutil.h>
 #endif
 
-
-	extern	int	do_lock_port, exit_flag;
-
-	static	unsigned int	comm_failures = 0;
+	static unsigned int	comm_failures = 0;
 
 static void ser_open_error(const char *port)
 {
@@ -237,166 +234,115 @@ int ser_close(int fd, const char *port)
 
 int ser_send_char(int fd, unsigned char ch)
 {
-	return write(fd, &ch, 1);
+	return ser_send_buf_pace(fd, 0, &ch, 1);
+}
+
+static int send_formatted(int fd, const char *fmt, va_list va, unsigned long d_usec)
+{
+	int	ret;
+	char	buf[LARGEBUF];
+
+	ret = vsnprintf(buf, sizeof(buf), fmt, va);
+
+	if (ret >= (int)sizeof(buf)) {
+		upslogx(LOG_WARNING, "vsnprintf needed more than %d bytes", sizeof(buf));
+	}
+
+	return ser_send_buf_pace(fd, d_usec, buf, strlen(buf));
 }
 
 /* send the results of the format string with d_usec delay after each char */
-unsigned int ser_send_pace(int fd, unsigned long d_usec, const char *fmt, ...)
+int ser_send_pace(int fd, unsigned long d_usec, const char *fmt, ...)
 {
 	int	ret;
-	char	buf[1024], *p;
 	va_list	ap;
-	unsigned int	sent = 0;
 
 	va_start(ap, fmt);
 
-	ret = vsnprintf(buf, sizeof(buf), fmt, ap);
+	ret = send_formatted(fd, fmt, ap, d_usec);
 
 	va_end(ap);
 
-	if ((ret < 1) || (ret >= (int) sizeof(buf)))
-		upslogx(LOG_WARNING, "ser_send_pace: vsnprintf needed more "
-			"than %d bytes", sizeof(buf));
-
-	for (p = buf; *p; p++) {
-		if (write(fd, p, 1) != 1)
-			return -1;
-
-		if (d_usec)
-			usleep(d_usec);
-
-		sent++;
-	}
-
-	return sent;
+	return ret;
 }
 
 /* send the results of the format string with no delay */
-unsigned int ser_send(int fd, const char *fmt, ...)
+int ser_send(int fd, const char *fmt, ...)
 {
 	int	ret;
-	char	buf[1024];
 	va_list	ap;
 
 	va_start(ap, fmt);
 
-	ret = vsnprintf(buf, sizeof(buf), fmt, ap);
+	ret = send_formatted(fd, fmt, ap, 0);
 
 	va_end(ap);
 
-	if ((ret < 1) || (ret >= (int) sizeof(buf)))
-		upslogx(LOG_WARNING, "ser_send: vsnprintf needed more "
-			"than %d bytes", sizeof(buf));
-
-	ret = write(fd, buf, strlen(buf));
-
 	return ret;
 }
 
 /* send buflen bytes from buf with no delay */
-int ser_send_buf(int fd, const unsigned char *buf, size_t buflen)
+int ser_send_buf(int fd, const void *buf, size_t buflen)
 {
-	return (write(fd, buf, buflen));
+	return ser_send_buf_pace(fd, 0, buf, buflen);
 }
 
 /* send buflen bytes from buf with d_usec delay after each char */
-int ser_send_buf_pace(int fd, unsigned long d_usec, const unsigned char *buf, 
+int ser_send_buf_pace(int fd, unsigned long d_usec, const void *buf,
 	size_t buflen)
 {
 	int	ret;
-	unsigned int	i;
+	size_t	sent;
 
-	for (i = 0; i < buflen; i++) {
-		ret = ser_send_char(fd, buf[i]);
+	for (sent = 0; sent < buflen; sent += ret) {
 
-		if (ret != 1)
+		ret = write(fd, &((char *)buf)[sent], (d_usec == 0) ? (buflen - sent) : 1);
+
+		if (ret < 1) {
 			return ret;
+		}
 
 		usleep(d_usec);
 	}
 
-	return buflen;
+	return sent;
 }
 
-static int get_buf(int fd, unsigned char *buf, size_t buflen, long d_sec, long d_usec)
+int ser_get_char(int fd, void *ch, long d_sec, long d_usec)
 {
-	int	ret;
-	fd_set	rfds;
-	struct	timeval	tv; 
-
-	FD_ZERO(&rfds);
-	FD_SET(fd, &rfds);
-
-	tv.tv_sec = d_sec;
-	tv.tv_usec = d_usec;
-
-	ret = select(fd + 1, &rfds, NULL, NULL, &tv);
-
-	if (ret == -1) {
-		if (errno != EINTR)
-			upslog_with_errno(LOG_ERR, "select fd %d", fd);
-
-		return -1;
-	}
-
-	if (ret == 0)
-		return -1;	/* timeout */
-
-	ret = read(fd, buf, buflen);
-
-	if (ret < 1)
-		return -1;	/* read error */
-
-	return ret;
+	return select_read(fd, ch, 1, d_sec, d_usec);
 }
 
-int ser_get_char(int fd, unsigned char *ch, long d_sec, long d_usec)
+int ser_get_buf(int fd, void *buf, size_t buflen, long d_sec, long d_usec)
 {
-	return get_buf(fd, ch, 1, d_sec, d_usec);
+	memset(buf, '\0', buflen);
+
+	return select_read(fd, buf, buflen, d_sec, d_usec);
 }
 
 /* keep reading until buflen bytes are received or a timeout occurs */
-int ser_get_buf_len(int fd, unsigned char *buf, size_t buflen, long d_sec, long d_usec)
+int ser_get_buf_len(int fd, void *buf, size_t buflen, long d_sec, long d_usec)
 {
-	int	       i, ret;
-	unsigned char  tmp[64];
-	size_t	       count = 0, tmplen;
+	int	ret;
+	size_t	recv;
 
 	memset(buf, '\0', buflen);
 
-	/* don't use all of tmp if the caller has a small buffer */
-	tmplen = sizeof(tmp);
+	for (recv = 0; recv < buflen; recv += ret) {
 
-	if (buflen < tmplen)
-		tmplen = buflen;
+		ret = select_read(fd, &((char *)buf)[recv], buflen - recv, d_sec, d_usec);
 
-	while (count < buflen) {
-		ret = get_buf(fd, tmp, tmplen, d_sec, d_usec);
-
-		if (ret < 1)
-			return -1;
-
-		for (i = 0; i < ret; i++) {
-			if (count == buflen) 
-				return count;
-
-			buf[count++] = tmp[i];
+		if (ret < 1) {
+			return ret;
 		}
-
-		/* make sure we don't read too much next time */
-		tmplen = sizeof(tmp);
-
-		if ((buflen - count) < tmplen)
-			tmplen = buflen - count;
-		
 	}
 
-	return count;
+	return recv;
 }
 
 /* reads a line up to <endchar>, discarding anything else that may follow,
    with callouts to the handler if anything matches the alertset */
-int ser_get_line_alert(int fd, char *buf, size_t buflen, char endchar,
+int ser_get_line_alert(int fd, void *buf, size_t buflen, char endchar,
 	const char *ignset, const char *alertset, void handler(char ch), 
 	long d_sec, long d_usec)
 {
@@ -409,15 +355,15 @@ int ser_get_line_alert(int fd, char *buf, size_t buflen, char endchar,
 	maxcount = buflen - 1;		/* for trailing \0 */
 
 	while (count < maxcount) {
-		ret = get_buf(fd, (unsigned char*)tmp, sizeof(tmp), d_sec, d_usec);
+		ret = select_read(fd, tmp, sizeof(tmp), d_sec, d_usec);
 
-		if (ret < 1)
-			return -1;
+		if (ret < 1) {
+			return ret;
+		}
 
 		for (i = 0; i < ret; i++) {
 
 			if ((count == maxcount) || (tmp[i] == endchar)) {
-				buf[count] = '\0';
 				return count;
 			}
 
@@ -431,17 +377,15 @@ int ser_get_line_alert(int fd, char *buf, size_t buflen, char endchar,
 				continue;
 			}
 
-			buf[count++] = tmp[i];
+			((char *)buf)[count++] = tmp[i];
 		}
 	}
 
-	buf[count] = '\0';
-
 	return count;
 }
 
 /* as above, only with no alertset handling (just a wrapper) */
-int ser_get_line(int fd, char *buf, size_t buflen, char endchar,
+int ser_get_line(int fd, void *buf, size_t buflen, char endchar,
 	const char *ignset, long d_sec, long d_usec)
 {
 	return ser_get_line_alert(fd, buf, buflen, endchar, ignset, "", NULL,
@@ -453,7 +397,7 @@ int ser_flush_in(int fd, const char *ignset, int verbose)
 	int	ret, extra = 0;
 	char	ch;
 
-	while ((ret = ser_get_char(fd, (unsigned char*)&ch, 0, 0)) > 0) {
+	while ((ret = ser_get_char(fd, &ch, 0, 0)) > 0) {
 
 		if (strchr(ignset, ch))
 			continue;
@@ -495,13 +439,6 @@ void ser_comm_fail(const char *fmt, ...)
 		((comm_failures % SER_ERR_LIMIT) != 0))
 		return;
 
-	/* generic message if the caller hasn't elaborated */
-	if (!fmt) {
-		upslogx(LOG_WARNING, "Communications with UPS lost"
-			" - check cabling");
-		return;
-	}
-
 	va_start(ap, fmt);
 	ret = vsnprintf(why, sizeof(why), fmt, ap);
 	va_end(ap);
