@@ -2,9 +2,9 @@
  *
  *  Based on NetSNMP API (Simple Network Management Protocol V1-2)
  *
- *  Copyright (C) 2002-2006 
- *  			Arnaud Quette <arnaud.quette@free.fr>
- *  			Dmitry Frolov <frolov@riss-telecom.ru>
+ *  Copyright (C) 
+ *   2002-2008  Arnaud Quette <arnaud.quette@free.fr>
+ *   2002-2006	Dmitry Frolov <frolov@riss-telecom.ru>
  *  			J.W. Hoogervorst <jeroen@hoogervorst.net>
  *  			Niels Baggesen <niels@baggesen.net>
  *
@@ -34,6 +34,8 @@
 
 /* include all known mib2nut lookup tables */
 #include "apccmib.h"
+#include "eaton-aphel-mib.h"
+#include "raritan-mib.h"
 #include "ietfmib.h"
 #include "mgemib.h"
 #include "netvisionmib.h"
@@ -50,6 +52,12 @@ mib2nut_info_t mib2nut[] = {
 		PW_OID_MODEL_NAME, pw_mib },
 	{ "ietf", IETF_MIB_VERSION, IETF_OID_POWER_STATUS,
 		IETF_OID_MFR_NAME, ietf_mib },
+	{ "aphel_genesisII", EATON_APHEL_MIB_VERSION, "",
+		APHEL1_OID_MODEL_NAME, eaton_aphel_genesisII_mib },
+	{ "aphel_revelation", EATON_APHEL_MIB_VERSION, "",
+		APHEL2_OID_MODEL_NAME, eaton_aphel_revelation_mib },
+	{ "raritan", RARITAN_MIB_VERSION, "",
+		RARITAN_OID_MODEL_NAME, raritan_mib },
 	{ NULL }
 };
 
@@ -58,7 +66,23 @@ snmp_info_t *snmp_info;
 const char *mibname;
 const char *mibvers;
 
-time_t lastpoll;
+#define DRIVER_NAME	"Generic SNMP UPS driver"
+#define DRIVER_VERSION		"0.44"
+
+/* driver description structure */
+upsdrv_info_t	upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Arnaud Quette <arnaud.quette@free.fr>\n" \
+	"Dmitry Frolov <frolov@riss-telecom.ru>\n" \
+	"J.W. Hoogervorst <jeroen@hoogervorst.net>\n" \
+	"Niels Baggesen <niels@baggesen.net>",
+	DRV_STABLE,
+	{ NULL }
+};
+/* FIXME: integrate MIBs info? do the same as for usbhid-ups! */
+
+time_t lastpoll = 0;
 
 /* ---------------------------------------------
  * driver functions implementations
@@ -72,13 +96,15 @@ void upsdrv_initinfo(void)
 
 	snprintf(version, sizeof version, "%s (mib: %s %s)",
 		DRIVER_VERSION, mibname, mibvers);
-	dstate_setinfo("driver.version.internal", version);
-	
+	dstate_setinfo("driver.version.internal", "%s", version);
+
 	/* add instant commands to the info database. */
 	for (su_info_p = &snmp_info[0]; su_info_p->info_type != NULL ; su_info_p++)			
+	{
 		su_info_p->flags |= SU_FLAG_OK;
 		if (SU_TYPE(su_info_p) == SU_TYPE_CMD)
 			dstate_addcmd(su_info_p->info_type);
+	}
 
 	/* setup handlers for instcmd and setvar functions */
 	upsh.setvar = su_setvar;
@@ -92,9 +118,6 @@ void upsdrv_initinfo(void)
 		dstate_dataok();
 	else		
 		dstate_datastale();
-
-	/* store timestamp */
-	lastpoll = time(NULL);
 }
 
 void upsdrv_updateinfo(void)
@@ -104,12 +127,16 @@ void upsdrv_updateinfo(void)
 	/* only update every pollfreq */
 	if (time(NULL) > (lastpoll + pollfreq)) {
 
+		status_init();
+
 		/* update all dynamic info fields */
 		if (snmp_ups_walk(SU_WALKMODE_UPDATE))
 			dstate_dataok();
 		else		
 			dstate_datastale();	
-	
+
+		status_commit();
+
 		/* store timestamp */
 		lastpoll = time(NULL);
 	}
@@ -117,10 +144,15 @@ void upsdrv_updateinfo(void)
 
 void upsdrv_shutdown(void)
 {
-	/* TODO: su_shutdown_ups(); */
-	
-	/* replace with a proper shutdown function */
-	fatalx(EXIT_FAILURE, "shutdown not supported");
+	/*
+	This driver will probably never support this. In order to
+	be any use, the driver should be called near the end of
+	the system halt script. By that time we in all likelyhood
+	we won't have network capabilities anymore, so we could
+	never send this command to the UPS. This is not an error,
+	but a limitation of the interface used.
+	*/
+	fatalx(EXIT_SUCCESS, "SNMP doesn't support shutdown in system halt script");
 }
 
 void upsdrv_help(void)
@@ -145,14 +177,6 @@ void upsdrv_makevartable(void)
 	    "Disable transfer OIDs (use on APCC Symmetras)");
 }
 
-void upsdrv_banner(void)
-{
-	upslogx(1,"Network UPS Tools - Multi-MIBS SNMP UPS driver %s (%s)", 
-		DRIVER_VERSION, UPS_VERSION);
-	
-	experimental_driver = 1;
-}
-
 void upsdrv_initups(void)
 {
 	snmp_info_t *su_info_p;
@@ -187,7 +211,8 @@ void upsdrv_initups(void)
 		upslogx(0, "Detected %s on host %s (mib: %s %s)",
 			 model, device_path, mibname, mibvers);
 	else
-		fatalx(EXIT_FAILURE, "%s MIB wasn't found on %s", mibs, g_snmp_sess.peername);   
+		fatalx(EXIT_FAILURE, "%s MIB wasn't found on %s", mibs, g_snmp_sess.peername);
+		/* No supported device detected */
 }
 
 void upsdrv_cleanup(void)
@@ -212,7 +237,7 @@ void nut_snmp_init(const char *type, const char *hostname, const char *version,
 	snmp_sess_init(&g_snmp_sess);
 
 	g_snmp_sess.peername = xstrdup(hostname);
-	g_snmp_sess.community = xstrdup(community);
+	g_snmp_sess.community = (unsigned char *)xstrdup(community);
 	g_snmp_sess.community_len = strlen(community);
 	if (strcmp(version, "v1") == 0)
 		g_snmp_sess.version = SNMP_VERSION_1;
@@ -248,9 +273,11 @@ struct snmp_pdu *nut_snmp_get(const char *OID)
 	size_t name_len = MAX_OID_LEN;
 	static unsigned int numerr = 0;
 
+	upsdebugx(3, "nut_snmp_get(%s)", OID);
+
 	/* create and send request. */
 	if (!snmp_parse_oid(OID, name, &name_len)) {
-		upslogx(LOG_ERR, "[%s] nut_snmp_get: %s: %s",
+		upsdebugx(2, "[%s] nut_snmp_get: %s: %s",
 			upsname?upsname:device_name, OID, snmp_api_errstring(snmp_errno));
 		return NULL;
 	}
@@ -299,7 +326,7 @@ bool_t nut_snmp_get_str(const char *OID, char *buf, size_t buf_len, info_lkp_t *
 {
 	size_t len = 0;
 	struct snmp_pdu *pdu;
-
+	
 	/* zero out buffer. */
 	memset(buf, 0, buf_len);
 
@@ -316,6 +343,7 @@ bool_t nut_snmp_get_str(const char *OID, char *buf, size_t buf_len, info_lkp_t *
 		buf[len] = '\0';
 		break;
 	case ASN_INTEGER:
+	case ASN_COUNTER:
 	case ASN_GAUGE:
 		if(oid2info) {
 			const char *str;
@@ -367,6 +395,7 @@ bool_t nut_snmp_get_int(const char *OID, long *pval)
 		free(buf);
 		break;
 	case ASN_INTEGER:
+	case ASN_COUNTER:
 	case ASN_GAUGE:
 		value = *pdu->variables->val.integer;
 		break;
@@ -517,7 +546,7 @@ void su_setinfo(const char *type, const char *value, int flags, int auxdata)
 		return;
 
 	if (strcasecmp(type, "ups.status")) {
-		dstate_setinfo(type, value);
+		dstate_setinfo(type, "%s", value);
 		dstate_setflags(type, flags);
 		dstate_setaux(type, auxdata);
 	}
@@ -532,9 +561,7 @@ void su_status_set(snmp_info_t *su_info_p, long value)
 	if ((info_value = su_find_infoval(su_info_p->oid2info, value)) != NULL)
 	{
 		if (strcmp(info_value, "")) {
-			status_init();
 			status_set(info_value);
-			status_commit();
 		}
 	}
 	/* TODO: else */
@@ -554,8 +581,9 @@ snmp_info_t *su_find_info(const char *type)
 }
 
 /* Load the right snmp_info_t structure matching mib parameter */
-void load_mib2nut(const char *mib)
+bool_t load_mib2nut(const char *mib)
 {
+	bool_t ret = FALSE;
 	mib2nut_info_t *mp = mib2nut;
 	upsdebugx(2, "SNMP UPS driver : entering load_mib2nut(%s)", mib);
 	
@@ -570,8 +598,10 @@ void load_mib2nut(const char *mib)
 			upsdebugx(1, "load_mib2nut: trying %s mib", mp->mib_name);
 			status = nut_snmp_get_str(mp->oid_auto_check,
 						buf, sizeof buf, NULL);
-			if (status)
+			if (status) {
+				ret = TRUE;
 				break;
+			}
 		}
 		mp++;
 	}
@@ -582,8 +612,15 @@ void load_mib2nut(const char *mib)
 		mibvers = mp->mib_version;
 		upsdebugx(1, "load_mib2nut: using %s mib", mibname);
 	}
-	else
-		fatalx(EXIT_FAILURE, "Unknown mibs value: %s", mib);
+	else {
+		/* Did we find something or is it really an unknown mib */
+		/* we assume (ret == FALSE) */
+		if (strcmp(mib, "auto") != 0)
+			fatalx(EXIT_FAILURE, "Unknown mibs value: %s", mib);
+		else
+			fatalx(EXIT_FAILURE, "No supported device detected");
+	}
+	return ret;
 }
 
 /* find the OID value matching that INFO_* value */
@@ -646,6 +683,10 @@ bool_t snmp_ups_walk(int mode)
 	
 	for (su_info_p = &snmp_info[0]; su_info_p->info_type != NULL ; su_info_p++) {
 
+		/* Check if we are asked to stop (reactivity++). */
+		if (exit_flag != 0)
+			return TRUE;
+
 		/* skip instcmd. */
 		if (SU_TYPE(su_info_p) == SU_TYPE_CMD) {
 			upsdebugx(1, "SU_CMD_MASK => %s", su_info_p->OID);
@@ -704,11 +745,14 @@ bool_t snmp_ups_walk(int mode)
 		}
 
 		if (su_info_p->flags & SU_OUTPHASES) {
+			upsdebugx(1, "Check outphases");
 		    	if (output_phases == 0) continue;
+			upsdebugx(1, "outphases is set");
 			if (su_info_p->flags & SU_OUTPUT_1) {
 			    	if (output_phases == 1)
 					su_info_p->flags &= ~SU_OUTPHASES;
 				else {
+					upsdebugx(1, "outphases is not 1");
 					su_info_p->flags &= ~SU_FLAG_OK;
 					continue;
 				}
@@ -717,6 +761,7 @@ bool_t snmp_ups_walk(int mode)
 			    	if (output_phases == 3)
 					su_info_p->flags &= ~SU_OUTPHASES;
 				else {
+					upsdebugx(1, "outphases is not 3");
 				    	su_info_p->flags &= ~SU_FLAG_OK;
 					continue;
 				}
@@ -800,6 +845,8 @@ bool_t su_ups_get(snmp_info_t *su_info_p)
 			/* no response, or units == F */
 			if ((su == FALSE) || (units == APCC_IEM_FAHRENHEIT))
 				temp = (value - 32) / 1.8;
+			else
+				temp = value;
 		}
 		else {
 			temp = value * su_info_p->info_len;
@@ -827,7 +874,7 @@ bool_t su_ups_get(snmp_info_t *su_info_p)
 			    	upsdebugx(1, "setvar %s", su_info_p->OID);
 			    	*su_info_p->setvar = value;
 			}
-			snprintf(buf, sizeof(buf), "%.1f", value * su_info_p->info_len);
+			snprintf(buf, sizeof(buf), "%.2f", value * su_info_p->info_len);
 		}
 	} else {
 		status = nut_snmp_get_str(su_info_p->OID, buf, sizeof(buf), su_info_p->oid2info);
@@ -853,16 +900,14 @@ int su_setvar(const char *varname, const char *val)
 
 	su_info_p = su_find_info(varname);
 
-	if (su_info_p == NULL || su_info_p->info_type == NULL ||
-		!(su_info_p->flags & SU_FLAG_OK))
-	{
-		upslogx(LOG_ERR, "su_setvar: info element unavailable %s", varname);
+	if (!su_info_p || !su_info_p->info_type || !(su_info_p->flags & SU_FLAG_OK)) {
+		upsdebugx(2, "su_setvar: info element unavailable %s", varname);
 		return STAT_SET_UNKNOWN;
 	}
 
 	if (!(su_info_p->info_flags & ST_FLAG_RW) || su_info_p->OID == NULL) {
-		upslogx(LOG_ERR, "su_setvar: not writable %s", varname);
-		return STAT_SET_UNKNOWN; /* STAT_SET_UNHANDLED would be better */
+		upsdebugx(2, "su_setvar: not writable %s", varname);
+		return STAT_SET_INVALID;
 	}
 
 	/* set value. */
@@ -872,10 +917,12 @@ int su_setvar(const char *varname, const char *val)
 		ret = nut_snmp_set_int(su_info_p->OID, strtol(val, NULL, 0));
 	}
 
-	if (ret == FALSE)
-		upslogx(LOG_ERR, "su_setvar: cannot set value %s for %s", val, su_info_p->OID);
-	else
-		upsdebugx(1, "su_setvar: sucessfully set %s to \"%s\"", su_info_p->info_type, val);
+	if (ret == FALSE) {
+		upsdebugx(1, "su_setvar: cannot set value %s for %s", val, su_info_p->OID);
+		return STAT_SET_FAILED;
+	}
+
+	upsdebugx(1, "su_setvar: sucessfully set %s to \"%s\"", su_info_p->info_type, val);
 
 	/* update info array. */
 	su_setinfo(varname, val, su_info_p->info_flags, su_info_p->info_len);
@@ -894,27 +941,43 @@ int su_instcmd(const char *cmdname, const char *extradata)
 
 	su_info_p = su_find_info(cmdname);
 
-	if ((su_info_p->info_type == NULL) || !(su_info_p->flags & SU_FLAG_OK) ||
-		(su_info_p->OID == NULL))
-	{
-		upslogx(LOG_ERR, "su_instcmd: %s unavailable", cmdname);
+	if (!su_info_p || !su_info_p->info_type || !(su_info_p->flags & SU_FLAG_OK)) {
+		upsdebugx(2, "su_instcmd: %s unavailable", cmdname);
 		return STAT_INSTCMD_UNKNOWN;
 	}
 
+	if (extradata) {
+		int	ret = STAT_SET_INVALID;
+
+		if (strcasecmp(cmdname, "shutdown.return")) {
+			ret = su_setvar("ups.delay.start", extradata);
+		}
+		if (strcasecmp(cmdname, "shutdown.stayoff")) {
+			ret = su_setvar("ups.delay.shutdown", extradata);
+		}
+		if (strcasecmp(cmdname, "shutdown.reboot")) {
+			ret = su_setvar("ups.delay.reboot", extradata);
+		}
+
+		if (ret == STAT_SET_HANDLED) {
+			extradata = NULL;
+		}
+	}
+
 	/* set value. */
 	if (su_info_p->info_flags & ST_FLAG_STRING) {
-		status = nut_snmp_set_str(su_info_p->OID, su_info_p->dfl);
+		status = nut_snmp_set_str(su_info_p->OID, extradata ? extradata : su_info_p->dfl);
 	} else {
-		status = nut_snmp_set_int(su_info_p->OID, su_info_p->info_len);
+		status = nut_snmp_set_int(su_info_p->OID, extradata ? atoi(extradata) : su_info_p->info_len);
 	}
 
 	if (status == FALSE) {
-		upslogx(LOG_ERR, "su_instcmd: cannot set value for %s", cmdname);
-		return STAT_INSTCMD_UNKNOWN;
-	} else {
-		upsdebugx(1, "su_instcmd: successfully sent command %s", cmdname);
-		return STAT_INSTCMD_HANDLED;
-	}	
+		upsdebugx(1, "su_instcmd: cannot set value for %s", cmdname);
+		return STAT_INSTCMD_FAILED;
+	}
+
+	upsdebugx(1, "su_instcmd: successfully sent command %s", cmdname);
+	return STAT_INSTCMD_HANDLED;
 }
 
 /* TODO: complete rewrite */
