@@ -31,8 +31,6 @@
 
 */
 
-#define DRV_VERSION "0.60"
-
 #include <ctype.h>
 #include <stdio.h>
 
@@ -41,6 +39,18 @@
 #include "solis.h"
 #include "timehead.h"
 
+#define DRIVER_NAME	"Microsol Solis UPS driver"
+#define DRIVER_VERSION	"0.61"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Silvino B. Magalhães <sbm2yk@gmail.com>",
+	DRV_STABLE,
+	{ NULL }
+};
+
 #define false 0
 #define true 1
 #define ENDCHAR 13	/* replies end with CR */
@@ -865,7 +875,7 @@ static void getbaseinfo(void)
 
 	/* dummy read attempt to sync - throw it out */
 	snprintf(mycmd, sizeof(mycmd), "%c%c",CMD_UPSCONT, ENDCHAR);
-	ser_send(upsfd, mycmd);
+	ser_send(upsfd, "%s", mycmd);
 
 	/* trying detect solis model */
 	while ( ( !detected ) && ( j < 20 ) )  {
@@ -879,9 +889,9 @@ static void getbaseinfo(void)
 
 		j++;
 		if( tam == 25)
-			CommReceive(Pacote, tam);
+			CommReceive((char *)Pacote, tam);
 		else
-			 CommReceive(temp, tam);
+			 CommReceive((char *)temp, tam);
 	} /* while end */
 
 	if( (!detected) ) {
@@ -993,7 +1003,7 @@ static void getupdateinfo(void)
 	temp[0] = 0; /* flush temp buffer */
 	tam = ser_get_buf_len(upsfd, temp, pacsize, 3, 0);
 
-	CommReceive(temp, tam);
+	CommReceive((char *)temp, tam);
 
 }
 
@@ -1025,7 +1035,6 @@ void upsdrv_initinfo(void)
 	getbaseinfo();
 
 	upsh.instcmd = instcmd;
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 }
 
 void upsdrv_updateinfo(void)
@@ -1121,13 +1130,6 @@ void upsdrv_makevartable(void)
 	
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Microsol Solis UPS driver %s (%s)\n", 
-		DRV_VERSION, UPS_VERSION);
-        printf("by Silvino B. Magalhaes for Microsol - sbm2yk@gmail.com\n\n");
-}
-
 void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
