@@ -29,10 +29,11 @@
 #include "main.h"
 #include "extstate.h" /* for ST_FLAG_STRING */
 #include "common.h"
+#include "usb-common.h"
 
-#define TRIPPLITE_HID_VERSION "TrippLite HID 0.2 (experimental)"
+#define TRIPPLITE_HID_VERSION "TrippLite HID 0.4"
+/* FIXME: experimental flag to be put in upsdrv_info */
 
-#define TRIPPLITE_VENDORID 0x09ae 
 
 /* For some devices, the reported battery voltage is off by
  * factor of 10 (due to an error in the report descriptor),
@@ -41,6 +42,53 @@
  */
 static double	battery_scale = 1.0;
 
+/* Specific handlers for USB device matching */
+static void *battery_scale_1dot0()
+{
+	/* FIXME: we could remove this one since it's the default! */
+	battery_scale = 1.0;
+	return NULL;
+}
+static void *battery_scale_0dot1()
+{
+	battery_scale = 0.1;
+	return NULL;
+}
+
+/* TrippLite */
+#define TRIPPLITE_VENDORID 0x09ae 
+
+/* Hewlett Packard */
+#define HP_VENDORID 0x03f0
+
+/* USB IDs device table */
+static usb_device_id_t tripplite_usb_device_table[] = {
+	/* e.g. TrippLite AVR550U */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1003), battery_scale_0dot1 },
+	/* e.g. TrippLite AVR750U */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1007), battery_scale_0dot1 },
+	/* e.g. TrippLite OMNI1000LCD */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x2005), battery_scale_0dot1 },
+	/* e.g. TrippLite OMNI900LCD */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x2007), battery_scale_0dot1 },
+	/* e.g. TrippLite smart2200RMXL2U */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x3012), battery_scale_1dot0 },
+	/* e.g. ? */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x3014), battery_scale_1dot0 },
+	/* e.g. TrippLite SmartOnline SU1500RTXL2UA (older unit?) */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x4001), battery_scale_1dot0 },
+	/* e.g. TrippLite SmartOnline SU6000RT4U? */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x4002), battery_scale_1dot0 },
+	/* e.g. TrippLite SmartOnline SU1500RTXL2ua */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x4003), battery_scale_1dot0 },
+
+	/* HP R/T 2200 INTL (like SMART2200RMXL2U) */
+	{ USB_DEVICE(HP_VENDORID, 0x1f0a), battery_scale_1dot0 },
+	
+	/* Terminating entry */
+	{ -1, -1, NULL }
+};
+
 /* returns statically allocated string - must not use it again before
    done with result! */
 static char *tripplite_chemistry_fun(double value)
@@ -123,23 +171,47 @@ static hid_info_t tripplite_hid2nut[] = {
 #ifdef USBHID_UPS_TRIPPLITE_DEBUG
 
 	/* unmapped variables - meaning unknown */
-	{ "UPS.ffff0010.[1].ffff007d", 0, 0, "UPS.ffff0010.[1].ffff007d", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.[1].ffff00c0", 0, 0, "UPS.ffff0015.[1].ffff00c0", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.[1].ffff00c1", 0, 0, "UPS.ffff0015.[1].ffff00c1", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.[1].ffff00c2", 0, 0, "UPS.ffff0015.[1].ffff00c2", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.[1].ffff00c3", 0, 0, "UPS.ffff0015.[1].ffff00c3", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.[1].ffff00c4", 0, 0, "UPS.ffff0015.[1].ffff00c4", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.[1].ffff00c5", 0, 0, "UPS.ffff0015.[1].ffff00c5", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.[1].ffff00d2", 0, 0, "UPS.ffff0015.[1].ffff00d2", NULL, "%.0f", 0, NULL },
-	{ "UPS.OutletSystem.Outlet.ffff0091", 0, 0, "UPS.OutletSystem.Outlet.ffff0091", NULL, "%.0f", 0, NULL },
-	{ "UPS.OutletSystem.Outlet.ffff00c7", 0, 0, "UPS.OutletSystem.Outlet.ffff00c7", NULL, "%.0f", 0, NULL },
+	{ "UPS.Flow.0xffff0097", 0, 0, "UPS.Flow.0xffff0097", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff0075", 0, 0, "UPS.0xffff0010.[1].0xffff0075", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff0076", 0, 0, "UPS.0xffff0010.[1].0xffff0076", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff007c", 0, 0, "UPS.0xffff0010.[1].0xffff007c", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff007d", 0, 0, "UPS.0xffff0010.[1].0xffff007d", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e0", 0, 0, "UPS.0xffff0010.[1].0xffff00e0", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e1", 0, 0, "UPS.0xffff0010.[1].0xffff00e1", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e2", 0, 0, "UPS.0xffff0010.[1].0xffff00e2", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e3", 0, 0, "UPS.0xffff0010.[1].0xffff00e3", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e4", 0, 0, "UPS.0xffff0010.[1].0xffff00e4", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e5", 0, 0, "UPS.0xffff0010.[1].0xffff00e5", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e6", 0, 0, "UPS.0xffff0010.[1].0xffff00e6", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e7", 0, 0, "UPS.0xffff0010.[1].0xffff00e7", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0010.[1].0xffff00e8", 0, 0, "UPS.0xffff0010.[1].0xffff00e8", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00c0", 0, 0, "UPS.0xffff0015.[1].0xffff00c0", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00c1", 0, 0, "UPS.0xffff0015.[1].0xffff00c1", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00c2", 0, 0, "UPS.0xffff0015.[1].0xffff00c2", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00c3", 0, 0, "UPS.0xffff0015.[1].0xffff00c3", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00c4", 0, 0, "UPS.0xffff0015.[1].0xffff00c4", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00c5", 0, 0, "UPS.0xffff0015.[1].0xffff00c5", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00d2", 0, 0, "UPS.0xffff0015.[1].0xffff00d2", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00d3", 0, 0, "UPS.0xffff0015.[1].0xffff00d3", NULL, "%.0f", 0, NULL },
+	{ "UPS.0xffff0015.[1].0xffff00d6", 0, 0, "UPS.0xffff0015.[1].0xffff00d6", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff0056", 0, 0, "UPS.OutletSystem.Outlet.0xffff0056", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff0081", 0, 0, "UPS.OutletSystem.Outlet.0xffff0081", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff0091", 0, 0, "UPS.OutletSystem.Outlet.0xffff0091", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff0093", 0, 0, "UPS.OutletSystem.Outlet.0xffff0093", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff0095", 0, 0, "UPS.OutletSystem.Outlet.0xffff0095", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff0096", 0, 0, "UPS.OutletSystem.Outlet.0xffff0096", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff0098", 0, 0, "UPS.OutletSystem.Outlet.0xffff0098", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff00a2", 0, 0, "UPS.OutletSystem.Outlet.0xffff00a2", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff00a4", 0, 0, "UPS.OutletSystem.Outlet.0xffff00a4", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff00a7", 0, 0, "UPS.OutletSystem.Outlet.0xffff00a7", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff00a9", 0, 0, "UPS.OutletSystem.Outlet.0xffff00a9", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff00aa", 0, 0, "UPS.OutletSystem.Outlet.0xffff00aa", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff00ab", 0, 0, "UPS.OutletSystem.Outlet.0xffff00ab", NULL, "%.0f", 0, NULL },
+	{ "UPS.OutletSystem.Outlet.0xffff00ac", 0, 0, "UPS.OutletSystem.Outlet.0xffff00ac", NULL, "%.0f", 0, NULL },
+	{ "UPS.PowerSummary.iOEMInformation", 0, 0, "UPS.PowerSummary.iOEMInformation", NULL, "%s", HU_FLAG_STATIC, stringid_conversion },
 
 #endif /* USBHID_UPS_TRIPPLITE_DEBUG */
 
-	/* Server side variables */
-	{ "driver.version.internal", ST_FLAG_STRING, sizeof(DRIVER_VERSION), NULL, NULL, DRIVER_VERSION, HU_FLAG_ABSENT, NULL },
-	{ "driver.version.data", ST_FLAG_STRING, sizeof(TRIPPLITE_HID_VERSION), NULL, NULL, TRIPPLITE_HID_VERSION, HU_FLAG_ABSENT, NULL },
-	
 	/* Battery page */
 	{ "battery.charge", 0, 0, "UPS.PowerSummary.RemainingCapacity", NULL, "%.0f", 0, NULL },
 	{ "battery.charge", 0, 0, "UPS.BatterySystem.Battery.RemainingCapacity", NULL, "%.0f", 0, NULL },
@@ -154,9 +226,9 @@ static hid_info_t tripplite_hid2nut[] = {
 	/* UPS page */
 	{ "ups.delay.start", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, DEFAULT_ONDELAY, HU_FLAG_ABSENT, NULL},
 	{ "ups.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, DEFAULT_OFFDELAY, HU_FLAG_ABSENT, NULL},
-	{ "ups.timer.start", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, "%.0f", 0, NULL},
-	{ "ups.timer.shutdown", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, "%.0f", 0, NULL},
-	{ "ups.timer.reboot", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeReboot", NULL, "%.0f", 0, NULL },
+	{ "ups.timer.start", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
+	{ "ups.timer.shutdown", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
+	{ "ups.timer.reboot", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeReboot", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL },
 	{ "ups.test.result", 0, 0, "UPS.BatterySystem.Test", NULL, "%s", 0, test_read_info },
 	{ "ups.beeper.status", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "%s", 0, beeper_info },
 	{ "ups.power.nominal", 0, 0, "UPS.Flow.ConfigApparentPower", NULL, "%.0f", HU_FLAG_STATIC, NULL },
@@ -260,42 +332,52 @@ static char *tripplite_format_serial(HIDDevice_t *hd) {
 /* this function allows the subdriver to "claim" a device: return 1 if
  * the device is supported by this subdriver, else 0. */
 static int tripplite_claim(HIDDevice_t *hd) {
-	if (hd->VendorID != TRIPPLITE_VENDORID) {
-		return 0;
-	}
 
-	/* accept any known UPS - add devices here as needed.
-	   Remember: also update scripts/udev/nutusb-ups.rules.in
-	   and scripts/hotplug/libhid.usermap */
-	switch (hd->ProductID)
+	int status = is_usb_device_supported(tripplite_usb_device_table, hd->VendorID,
+								 hd->ProductID);
+
+	switch (status)
 	{
-	case 0x1003:  /* e.g. AVR550U */
-	case 0x2005:  /* e.g. OMNI1000LCD */
-	case 0x2007:  /* e.g. OMNI900LCD */
-		battery_scale = 0.1;
-		return 1;
+	case POSSIBLY_SUPPORTED:
 
-	case 0x3012:  /* e.g. smart2200RMXL2U */
-	case 0x4002:  /* e.g. SmartOnline SU6000RT4U? */
-	case 0x4003:  /* e.g. SmartOnline SU1500RTXL2ua */
-		battery_scale = 1.0;
-		return 1;
+		switch (hd->VendorID)
+		{
+		/*
+		 * this vendor makes lots of USB devices that are
+		 * not a UPS, so don't use possibly_supported here
+		 */
+		case HP_VENDORID:
+			return 0;
+	
+		/* reject known non-HID devices */
+		/* not all Tripp Lite products are HID, some are "serial over USB". */
+		case TRIPPLITE_VENDORID:
+			if (hd->ProductID == 0x0001) {
+				/* e.g. SMART550USB, SMART3000RM2U */
+				upsdebugx(0, "This Tripp Lite device (%04x/%04x) is not supported by usbhid-ups.\n"
+						 "Please use the tripplite_usb driver instead.\n",
+						 hd->VendorID, hd->ProductID);
+			} else {
+				possibly_supported("Tripp Lite", hd);
+			}
 
-	/* reject known non-HID devices */
-	/* not all Tripp Lite products are HID, some are "serial over USB". */
-	case 0x0001:  /* e.g. SMART550USB, SMART3000RM2U */
-		upsdebugx(0,
-"This Tripp Lite device (%04x/%04x) is not supported by usbhid-ups.\n"
-"Please use the tripplite_usb driver instead.\n",
-					 hd->VendorID, hd->ProductID);
-		return 0;
+			return 0;
 
-	/* by default, reject, unless the productid option is given */
-	default:
-		if (getval("productid")) {
-			return 1;
+		/* by default, reject, unless the productid option is given */
+		default:
+			if (getval("productid")) {
+				return 1;
+			}
+
+			possibly_supported("Belkin", hd);
+			return 0;
 		}
-		possibly_supported("Tripp Lite", hd);
+
+	case SUPPORTED:
+		return 1;
+
+	case NOT_SUPPORTED:
+	default:
 		return 0;
 	}
 }
