@@ -8,7 +8,7 @@
    Copyright (C) 1999  Russell Kroll <rkroll@exploits.org>
    Copyright (C) 2001  Rickard E. (Rik) Faith <faith@alephnull.com>
    Copyright (C) 2004  Nicholas J. Kain <nicholas@kain.us>
-   Copyright (C) 2005-2007  Charles Lepple <clepple+nut@gmail.com>
+   Copyright (C) 2005-2008  Charles Lepple <clepple+nut@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-#define DRV_VERSION "0.11"
 
 /* % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
  *
@@ -80,12 +79,12 @@
  * :Q     ->            (while online: reboot)
  * :R     -> R<01><FF>  (query flags for conditions that cause a reset?)
  * :S     -> S100_Z0    (status?)
- * :T     -> T7D2581    (temperature?)
+ * :T     -> T7D2581    (temperature, frequency)
  * :U     -> U<FF><FF>  (unit ID, 1-65535)
  * :V     -> V1062XX	(outlets in groups of 2-2-4, with the groups of 2
  * 			 individually switchable.)
  * :W_    -> W_		(watchdog)
- * :Z     -> Z		(reset for max/min? - apparently not)
+ * :Z     -> Z		(reset for max/min; takes a moment to complete)
  * 
  * % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
  *
@@ -131,11 +130,11 @@
 POD ("Plain Old Documentation") - run through pod2html or perldoc. See
 perlpod(1) for more information.
 
-pod2man --name='TRIPPLITE_USB' --section=8 --release=' ' --center='Network UPS Tools (NUT)' tripplite_usb.c
+pod2man --name='TRIPPLITE_USB' --section=8 --release='$Rev: 1695 $' --center='Network UPS Tools (NUT)' tripplite_usb.c
 
 =head1 NAME
 
-tripplite_usb - Driver for older Tripp Lite USB UPSes (non-PDC HID)
+tripplite_usb - Driver for older Tripp Lite USB UPSes (not PDC HID)
 
 =head1 NOTE
 
@@ -148,8 +147,8 @@ nutupsdrv(8).
 This driver should work with older Tripp Lite UPSes which are detected as USB
 HID-class devices, but are not true HID Power-Device Class devices.  So far,
 the devices supported by tripplite_usb have product ID 0001, and the newer
-units (such as those with "LCD" in the model name) with product ID 2001 will
-work with the usbhid-ups driver instead.  Please report success or failure to
+units (such as those with "LCD" in the model name) with product ID 2001 require
+the usbhid-ups driver instead.  Please report success or failure to
 the nut-upsuser mailing list.  A key piece of information is the protocol
 number, returned in ups.debug.0.  Also, be sure to turn on debugging (C<-DDD>)
 for more informative log messages.  If your Tripp Lite UPS uses a serial port,
@@ -171,6 +170,22 @@ This driver has been tested with the following models:
 
 =back
 
+If you have used Tripp Lite's PowerAlert software to connect to your UPS, there
+is a good chance that tripplite_usb(8) will work if it uses one of the
+following protocols:
+
+=over 
+
+=item * Protocol 0004
+
+=item * Protocol 1001
+
+=item * Protocol 2001
+
+=item * Protocol 3003
+
+=back
+
 =head1 EXTRA ARGUMENTS
 
 This driver supports the following optional setting in the ups.conf(5) file
@@ -224,6 +239,19 @@ For more information on regular expressions, see regex(7)
 This variable is the same as the C<offdelay> setting, but it can be changed at
 runtime by upsrw(8).
 
+=item ups.id
+
+Some SMARTPRO models feature an ID that can be set and retrieved. If your UPS
+supports this feature, this variable will be listed in the output of upsrw(8).
+
+=item outlet.1.switch
+
+Some Tripp Lite units have a switchable outlet (usually outlet #1) which can be
+turned on and off by writing C<1> or C<0>, respectively, to C<outlet.1.switch>.
+If your unit has multiple switchable outlets, substitute the outlet number for
+"1" in the variable name. Be sure to test this first - there is no other way to
+be certain that the number used by the driver matches the label on the unit.
+
 =back
 
 =head1 KNOWN ISSUES AND BUGS
@@ -283,12 +311,65 @@ The NUT (Network UPS Tools) home page: http://www.networkupstools.org/
 #include <math.h>
 #include <ctype.h>
 #include <usb.h>
+#include "usb-common.h"
+
+#define DRIVER_NAME		"Tripp Lite OMNIVS / SMARTPRO driver"
+#define DRIVER_VERSION	"0.20"
+
+/* driver description structure */
+upsdrv_info_t	upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Charles Lepple <clepple+nut@gmail.com>\n" \
+	"Russell Kroll <rkroll@exploits.org>\n" \
+	"Rickard E. (Rik) Faith <faith@alephnull.com>\n" \
+	"Nicholas J. Kain <nicholas@kain.us>",
+	DRV_EXPERIMENTAL,
+	{ NULL }
+};
+
+/* TrippLite */
+#define TRIPPLITE_VENDORID 0x09ae 
+
+/* USB IDs device table */
+static usb_device_id_t tripplite_usb_device_table[] = {
+	/* e.g. OMNIVS1000, SMART550USB, ... */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x0001), NULL },
+	
+	/* Terminating entry */
+	{ -1, -1, NULL }
+};
+
+static int subdriver_match_func(USBDevice_t *hd, void *privdata)
+{
+	switch (is_usb_device_supported(tripplite_usb_device_table, hd->VendorID, hd->ProductID))
+	{
+	case SUPPORTED:
+		return 1;
+
+	case POSSIBLY_SUPPORTED:
+		/* by default, reject, unless the productid option is given */
+		if (getval("productid")) {
+			return 1;
+		}
+	case NOT_SUPPORTED:
+	default:
+		return 0;
+	}
+}
+
+static USBDeviceMatcher_t subdriver_matcher = {
+	&subdriver_match_func,
+	NULL,
+	NULL
+};
 
 static enum tl_model_t {
 	TRIPP_LITE_UNKNOWN = 0,
 	TRIPP_LITE_OMNIVS,
 	TRIPP_LITE_OMNIVS_2001,
-	TRIPP_LITE_SMARTPRO
+	TRIPP_LITE_SMARTPRO,
+	TRIPP_LITE_SMART_0004
 } tl_model = TRIPP_LITE_UNKNOWN;
 
 /*!@brief If a character is not printable, return a dot. */
@@ -296,7 +377,7 @@ static enum tl_model_t {
 
 #define ENDCHAR 13
 
-#define MAX_SEND_TRIES 3
+#define MAX_SEND_TRIES 10
 #define SEND_WAIT_SEC 0
 #define SEND_WAIT_NSEC (1000*1000*100)
 
@@ -358,8 +439,6 @@ static int reconnect_ups(void)
 	upsdebugx(2, "= device has been disconnected, try to reconnect =");
 	upsdebugx(2, "==================================================");
 
-	upsdrv_cleanup();
-
 	ret = comm_driver->open(&udev, &curDevice, reopen_matcher, NULL);
 	if (ret < 1) {
 		upslogx(LOG_INFO, "Reconnecting to UPS failed; will retry later...");
@@ -404,9 +483,9 @@ static int hex2d(const unsigned char *start, unsigned int len)
 	unsigned char buf[32];
 	buf[31] = '\0';
 
-	strncpy(buf, start, (len < (sizeof buf) ? len : (sizeof buf - 1)));
+	strncpy((char *)buf, (char *)start, (len < (sizeof buf) ? len : (sizeof buf - 1)));
 	if(len < sizeof(buf)) buf[len] = '\0';
-	return strtol(buf, NULL, 16);
+	return strtol((char *)buf, NULL, 16);
 }
 
 /*!@brief Dump message in both hex and ASCII
@@ -423,24 +502,30 @@ static const char *hexascdump(unsigned char *msg, size_t len)
 
 	bufp = buf;
 	buf[0] = 0;
+
+	/* Dump each byte in hex: */
 	for(i=0; i<len; i++) {
-		bufp += sprintf(bufp, "%02x ", msg[i]);
+		bufp += sprintf((char *)bufp, "%02x ", msg[i]);
 	}
-#if 1
+
+	/* Dump single-quoted string with printable version of each byte: */
 	*bufp++ = '\'';
 	for(i=0; i<len; i++) {
 		*bufp++ = toprint(msg[i]);
 	}
 	*bufp++ = '\'';
-#endif
+
 	*bufp++ = '\0';
 
-	return buf;
+	return (char *)buf;
 }
 
 enum tl_model_t decode_protocol(unsigned int proto)
 {
 	switch(proto) {
+		case 0x0004:
+			upslogx(3, "Using older SMART protocol (%x)", proto);
+			return TRIPP_LITE_SMART_0004;
 		case 0x1001:
 			upslogx(3, "Using OMNIVS protocol (%x)", proto);
 			return TRIPP_LITE_OMNIVS;
@@ -515,12 +600,10 @@ void usb_comm_fail(int res, const char *msg)
 		case -EBUSY:
 			upslogx(LOG_WARNING, "%s: Device claimed by another process", msg);
 			fatalx(EXIT_FAILURE, "Terminating: EBUSY");
-			upsdrv_cleanup();
 			break;
 
 		default:
 			upslogx(LOG_WARNING, "%s: Device detached? (error %d: %s)", msg, res, usb_strerror());
-			upsdrv_cleanup();
 
 			upslogx(LOG_NOTICE, "Reconnect attempt #%d", ++try);
 			hd = NULL;
@@ -598,8 +681,9 @@ static int send_cmd(const unsigned char *msg, size_t msg_len, unsigned char *rep
 			upsdebugx(7, "send_cmd recv_try %d", recv_try+1);
 			ret = comm_driver->get_interrupt(udev, reply, sizeof(buffer_out), RECV_WAIT_MSEC);
 			if(ret != sizeof(buffer_out)) {
-				upslogx(1, "libusb_get_interrupt() returned %d instead of %u",
-					ret, (unsigned)(sizeof(buffer_out)));
+				upslogx(1, "libusb_get_interrupt() returned %d instead of %u while sending %s",
+					ret, (unsigned)(sizeof(buffer_out)),
+					hexascdump(buffer_out, sizeof(buffer_out)));
 			}
 			done = (ret == sizeof(buffer_out)) && (buffer_out[1] == reply[0]);
 		}
@@ -632,7 +716,7 @@ void debug_message(const char *msg, int len)
 
 	snprintf(var_name, sizeof(var_name), "ups.debug.%c", *msg);
 
-	ret = send_cmd(msg, len, (char *)tmp_value, sizeof(tmp_value));
+	ret = send_cmd((unsigned char *)msg, len, tmp_value, sizeof(tmp_value));
 	if(ret <= 0) {
 		sprintf(err_msg, "Error reading '%c' value", *msg);
 		usb_comm_fail(ret, err_msg);
@@ -730,11 +814,12 @@ static int control_outlet(int outlet_id, int state)
 	int ret;
 
 	switch(tl_model) {
-		case TRIPP_LITE_SMARTPRO:
+		case TRIPP_LITE_SMARTPRO:   /* tested */
+		case TRIPP_LITE_SMART_0004: /* untested */
 			snprintf(k_cmd, sizeof(k_cmd)-1, "N%02X", 5);
-			ret = send_cmd(k_cmd, strlen(k_cmd) + 1, buf, sizeof buf);
+			ret = send_cmd((unsigned char *)k_cmd, strlen(k_cmd) + 1, (unsigned char *)buf, sizeof buf);
 			snprintf(k_cmd, sizeof(k_cmd)-1, "K%d%d", outlet_id, state & 1);
-			ret = send_cmd(k_cmd, strlen(k_cmd) + 1, buf, sizeof buf);
+			ret = send_cmd((unsigned char *)k_cmd, strlen(k_cmd) + 1, (unsigned char *)buf, sizeof buf);
 
 			if(ret != 8) {
 				upslogx(LOG_ERR, "Could not set outlet %d to state %d, ret = %d", outlet_id, state, ret);
@@ -754,14 +839,14 @@ static int instcmd(const char *cmdname, const char *extra)
 {
 	unsigned char buf[10];
 
-	if(tl_model == TRIPP_LITE_SMARTPRO) {
+	if(tl_model == TRIPP_LITE_SMARTPRO || tl_model == TRIPP_LITE_SMART_0004) {
 		if (!strcasecmp(cmdname, "test.battery.start")) {
-			send_cmd("A", 2, buf, sizeof buf);
+			send_cmd((unsigned char *)"A", 2, buf, sizeof buf);
 			return STAT_INSTCMD_HANDLED;
 		}
 
-		if(!strcasecmp(cmdname, "ups.debug.send_Z")) {
-			return (send_cmd("Z", 2, buf, sizeof buf) == 2) ? STAT_INSTCMD_HANDLED : STAT_INSTCMD_UNKNOWN;
+		if(!strcasecmp(cmdname, "reset.input.minmax")) {
+			return (send_cmd((unsigned char *)"Z", 2, buf, sizeof buf) == 2) ? STAT_INSTCMD_HANDLED : STAT_INSTCMD_UNKNOWN;
 		}
 	}
 
@@ -771,7 +856,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	if (!strcasecmp(cmdname, "load.on")) {
 		return control_outlet(0, 0) ? STAT_INSTCMD_HANDLED : STAT_INSTCMD_UNKNOWN;
 	}
-	/* TODO: add code for individual outlets */
+	/* code for individual outlets is in setvar() */
 #if 0
 	if (!strcasecmp(cmdname, "shutdown.reboot")) {
 		do_reboot_now();
@@ -799,14 +884,32 @@ static int setvar(const char *varname, const char *val)
 {
 	if (!strcasecmp(varname, "ups.delay.shutdown")) {
 		offdelay = atoi(val);
-		dstate_setinfo("ups.delay.shutdown", val);
+		dstate_setinfo("ups.delay.shutdown", "%d", offdelay);
+		return STAT_SET_HANDLED;
+	}
+
+	if (unit_id >= 0 && !strcasecmp(varname, "ups.id")) {
+                int new_unit_id, ret;
+		unsigned char J_msg[] = "J__", buf[9];
+
+		new_unit_id = atoi(val);
+		J_msg[1] = new_unit_id >> 8;
+		J_msg[2] = new_unit_id & 0xff;
+                ret = send_cmd(J_msg, sizeof(J_msg), buf, sizeof(buf));
+
+		if(ret <= 0) {
+			upslogx(LOG_NOTICE, "Could not set Unit ID (return code: %d).", ret);
+			return STAT_SET_UNKNOWN;
+		}
+
+		dstate_setinfo("ups.id", "%s", val);
 		return STAT_SET_HANDLED;
 	}
 
 	if(!strncmp(varname, "outlet.", strlen("outlet."))) {
 		char outlet_name[80];
 		char index_str[10], *first_dot, *next_dot;
-		int index_chars, index, state;
+		int index_chars, index, state, ret;
 
 		first_dot = strstr(varname, ".");
 		next_dot = strstr(first_dot + 1, ".");
@@ -830,9 +933,14 @@ static int setvar(const char *varname, const char *val)
 		upslogx(LOG_DEBUG, "outlet.%d.switch = %s -> %d", index, val, state);
 
 		snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.switch", index);
-		dstate_setinfo(outlet_name, "%d", state);
 
-		return control_outlet(index, state) ? STAT_SET_HANDLED : STAT_SET_UNKNOWN;
+		ret = control_outlet(index, state);
+		if(ret) {
+			dstate_setinfo(outlet_name, "%d", state);
+			return STAT_SET_HANDLED;
+		} else {
+			return STAT_SET_UNKNOWN;
+		}
 	}
 
 #if 0
@@ -854,9 +962,9 @@ void upsdrv_initinfo(void)
 {
 	const unsigned char proto_msg[] = "\0", f_msg[] = "F", p_msg[] = "P",
 		s_msg[] = "S", u_msg[] = "U", v_msg[] = "V", w_msg[] = "W\0";
-	char *model, *model_end, proto_string[5 + sizeof("protocol ")];
+	char *model, *model_end;
 	unsigned char proto_value[9], f_value[9], p_value[9], s_value[9],
-	     u_value[9], v_value[9], w_value[9], buf[256];
+	     u_value[9], v_value[9], w_value[9];
 	int  va, ret;
 	unsigned int proto_number = 0;
 
@@ -884,11 +992,9 @@ void upsdrv_initinfo(void)
 	tl_model = decode_protocol(proto_number);
 
 	if(tl_model == TRIPP_LITE_UNKNOWN)
-		dstate_setinfo("ups.debug.0", hexascdump(proto_value+1, 7));
+		dstate_setinfo("ups.debug.0", "%s", hexascdump(proto_value+1, 7));
 
-	snprintf(proto_string, sizeof(proto_string), "protocol %04x", proto_number);
-
-	dstate_setinfo("ups.firmware.aux", proto_string);
+	dstate_setinfo("ups.firmware.aux", "protocol %04x", proto_number);
 
 	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
 
@@ -903,7 +1009,14 @@ void upsdrv_initinfo(void)
 
 	/* Get nominal power: */
 	ret = send_cmd(p_msg, sizeof(p_msg), p_value, sizeof(p_value)-1);
-	va = strtol(p_value+1, NULL, 10);
+	va = strtol((char *)(p_value+1), NULL, 10);
+
+	if(tl_model == TRIPP_LITE_SMART_0004) {
+		dstate_setinfo("ups.debug.P","%s", hexascdump(p_value+1, 7));
+		va *= 10; /* TODO: confirm? */
+	}
+
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
 
 	/* trim "TRIPP LITE" from beginning of model */
 	model = strdup(hd->Product);
@@ -921,7 +1034,7 @@ void upsdrv_initinfo(void)
 		*model_end = '\0';
 	}
 
-	dstate_setinfo("ups.model", model);
+	dstate_setinfo("ups.model", "%s", model);
 
 	dstate_setinfo("ups.power.nominal", "%d", va);
 
@@ -930,7 +1043,7 @@ void upsdrv_initinfo(void)
         /* Fetch firmware version: */
 	ret = send_cmd(f_msg, sizeof(f_msg), f_value, sizeof(f_value)-1);
 
-	toprint_str(f_value+1, 6);
+	toprint_str((char *)(f_value+1), 6);
 	f_value[7] = 0;
 
 	dstate_setinfo("ups.firmware", "F%s", f_value+1);
@@ -952,15 +1065,14 @@ void upsdrv_initinfo(void)
 		int i;
 		char outlet_name[80];
 
-		outlet_name[79] = 0;
 		for(i = 1; i <= switchable_load_banks + 1; i++) {
-			snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.id", i);
+			snprintf(outlet_name, sizeof(outlet_name), "outlet.%d.id", i);
 			dstate_setinfo(outlet_name, "%d", i);
 
-			snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.desc", i);
+			snprintf(outlet_name, sizeof(outlet_name), "outlet.%d.desc", i);
 			dstate_setinfo(outlet_name, "Load %d", i);
 
-			snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.switchable", i);
+			snprintf(outlet_name, sizeof(outlet_name), "outlet.%d.switchable", i);
 			if( i <= switchable_load_banks ) {
 				dstate_setinfo(outlet_name, "1");
 				snprintf(outlet_name, sizeof(outlet_name)-1, "outlet.%d.switch", i);
@@ -981,7 +1093,7 @@ void upsdrv_initinfo(void)
 
 	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
 
-	if(tl_model != TRIPP_LITE_OMNIVS) {
+	if(tl_model != TRIPP_LITE_OMNIVS && tl_model != TRIPP_LITE_SMART_0004) {
 		/* Unit ID might not be supported by all models: */
 		ret = send_cmd(u_msg, sizeof(u_msg), u_value, sizeof(u_value)-1);
 		if(ret <= 0) {
@@ -990,10 +1102,16 @@ void upsdrv_initinfo(void)
 			unit_id = (int)((unsigned)(u_value[1]) << 8) 
 				| (unsigned)(u_value[2]);
 		}
+
+		if(tl_model == TRIPP_LITE_SMART_0004) {
+			debug_message("U", 2);
+		}
 	}
 
 	if(unit_id >= 0) {
 		dstate_setinfo("ups.id", "%d", unit_id);
+		dstate_setflags("ups.id", ST_FLAG_RW | ST_FLAG_STRING);
+		dstate_setaux("ups.id", 5);
 		upslogx(LOG_DEBUG,"Unit ID: %d", unit_id);
 	}
 
@@ -1003,26 +1121,23 @@ void upsdrv_initinfo(void)
 	dstate_setinfo("battery.voltage.nominal", "%d", battery_voltage_nominal);
 	dstate_setinfo("ups.debug.load_banks", "%d", switchable_load_banks);
 
-	snprintf(buf, sizeof buf, "%d", offdelay);
-	dstate_setinfo("ups.delay.shutdown", buf);
+	dstate_setinfo("ups.delay.shutdown", "%d", offdelay);
 	dstate_setflags("ups.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING);
 	dstate_setaux("ups.delay.shutdown", 3);
 
 #if 0
-	snprintf(buf, sizeof buf, "%d", startdelay);
-	dstate_setinfo("ups.delay.start", buf);
+	dstate_setinfo("ups.delay.start", "%d", startdelay);
 	dstate_setflags("ups.delay.start", ST_FLAG_RW | ST_FLAG_STRING);
 	dstate_setaux("ups.delay.start", 8);
 
-	snprintf(buf, sizeof buf, "%d", bootdelay);
-	dstate_setinfo("ups.delay.reboot", buf);
+	dstate_setinfo("ups.delay.reboot", "%d", bootdelay);
 	dstate_setflags("ups.delay.reboot", ST_FLAG_RW | ST_FLAG_STRING);
 	dstate_setaux("ups.delay.reboot", 3);
 #endif
 
-	if(tl_model == TRIPP_LITE_SMARTPRO) {
+	if(tl_model == TRIPP_LITE_SMARTPRO || tl_model == TRIPP_LITE_SMART_0004) {
 		dstate_addcmd("test.battery.start");
-		dstate_addcmd("ups.debug.send_Z");
+		dstate_addcmd("reset.input.minmax");
 	}
 
 	dstate_addcmd("shutdown.return");
@@ -1044,8 +1159,6 @@ void upsdrv_initinfo(void)
 
 	printf("Attached to %s %s\n",
 			dstate_getinfo("ups.mfr"), dstate_getinfo("ups.model"));
-
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 }
 
 void upsdrv_shutdown(void)
@@ -1067,6 +1180,8 @@ void upsdrv_updateinfo(void)
 
 	status_init();
 
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
+
 	/* General status (e.g. "S10") */
 	ret = send_cmd(s_msg, sizeof(s_msg), s_value, sizeof(s_value));
 	if(ret <= 0) {
@@ -1079,6 +1194,8 @@ void upsdrv_updateinfo(void)
 		dstate_setinfo("ups.debug.S","%s", hexascdump(s_value+1, 7));
 	}
 
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
+
 	if(tl_model == TRIPP_LITE_OMNIVS) {
 		switch(s_value[2]) {
 			case '0':
@@ -1107,8 +1224,9 @@ void upsdrv_updateinfo(void)
 		}
 	}
 
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
 
-	if(tl_model == TRIPP_LITE_OMNIVS_2001) {
+	if(tl_model == TRIPP_LITE_SMARTPRO || tl_model == TRIPP_LITE_OMNIVS_2001 || tl_model == TRIPP_LITE_SMART_0004) {
 		switch(s_value[2]) {
 			case '0':
 				dstate_setinfo("battery.test.status", "Battery OK");
@@ -1137,53 +1255,26 @@ void upsdrv_updateinfo(void)
 				break;
 		}
 
-		/* Online/on battery: */
-		if(s_value[4] & 1) {
-			status_set("OB");
+
+		if(s_value[4] & 4) {
+			status_set("OFF");
 		} else {
-			status_set("OL");
+			/* Online/on battery: */
+			if(s_value[4] & 1) {
+				status_set("OB");
+			} else {
+				status_set("OL");
+			}
+		}
+
+		/* This may not be right... */
+		if(tl_model == TRIPP_LITE_SMARTPRO) {
+			battery_charge = (unsigned)(s_value[5]);
+			dstate_setinfo("battery.charge",  "%u", battery_charge);
 		}
 	}
 
-	if(tl_model == TRIPP_LITE_SMARTPRO) {
-		switch(s_value[2]) {
-			case '0':
-				dstate_setinfo("battery.test.status", "Battery OK");
-				break;
-			case '1':
-				dstate_setinfo("battery.test.status", "Battery bad - replace");
-				break;
-			case '2':
-				status_set("CAL");
-				break;
-			case '3':
-				status_set("OVER");
-				dstate_setinfo("battery.test.status", "Overcurrent?");
-				break;
-			case '4':
-				/* The following message is confusing, and may not be accurate: */
-				/* dstate_setinfo("battery.test.status", "Battery state unknown"); */
-				break;
-			case '5':
-				status_set("OVER");
-				dstate_setinfo("battery.test.status", "Battery fail - overcurrent?");
-				break;
-			default:
-				upslogx(LOG_ERR, "Unknown value for s[2]: 0x%02x", s_value[2]);
-				dstate_datastale();
-				break;
-		}
-
-		/* Online/on battery: */
-		if(s_value[4] & 1) {
-			status_set("OB");
-		} else {
-			status_set("OL");
-		}
-
-		battery_charge = (unsigned)(s_value[5]);
-		dstate_setinfo("battery.charge",  "%u", battery_charge);
-	}
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
 
 	switch(s_value[1]) {
 		case '0':
@@ -1204,6 +1295,8 @@ void upsdrv_updateinfo(void)
 
 	status_commit();
 
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
+
 	if( tl_model == TRIPP_LITE_OMNIVS || tl_model == TRIPP_LITE_OMNIVS_2001 ) {
 		ret = send_cmd(b_msg, sizeof(b_msg), b_value, sizeof(b_value));
 		if(ret <= 0) {
@@ -1231,7 +1324,9 @@ void upsdrv_updateinfo(void)
 		dstate_setinfo("battery.charge",  "%3d", bp);
 	}
 
-	if( tl_model == TRIPP_LITE_SMARTPRO ) {
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
+
+	if( tl_model == TRIPP_LITE_SMARTPRO || tl_model == TRIPP_LITE_SMART_0004 ) {
 		ret = send_cmd(d_msg, sizeof(d_msg), d_value, sizeof(d_value));
 		if(ret <= 0) {
 			dstate_datastale();
@@ -1246,10 +1341,14 @@ void upsdrv_updateinfo(void)
 
 		dstate_setinfo("battery.voltage", "%.2f", bv);
 
-		/* battery charge state is left as an exercise for the reader */
+		/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
 
 		ret = send_cmd(m_msg, sizeof(m_msg), m_value, sizeof(m_value));
-		dstate_setinfo("ups.debug.M", "%s", hexascdump(m_value+1, 7));
+
+                if(m_value[5] != 0x0d) { /* we only expect 4 hex digits */
+			dstate_setinfo("ups.debug.M", "%s", hexascdump(m_value+1, 7));
+		}
+
 		if(ret <= 0) {
 			dstate_datastale();
 			usb_comm_fail(ret, "Error reading M (min/max input voltage)");
@@ -1259,6 +1358,8 @@ void upsdrv_updateinfo(void)
 		dstate_setinfo("input.voltage.minimum", "%3d", hex2d(m_value+1, 2));
 		dstate_setinfo("input.voltage.maximum", "%3d", hex2d(m_value+3, 2));
 
+		/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
+
 		ret = send_cmd(t_msg, sizeof(t_msg), t_value, sizeof(t_value));
 		dstate_setinfo("ups.debug.T", "%s", hexascdump(t_value+1, 7));
 		if(ret <= 0) {
@@ -1267,22 +1368,31 @@ void upsdrv_updateinfo(void)
 			return;
 		}
 
-		freq = hex2d(t_value + 3, 3);
-		dstate_setinfo("input.frequency", "%.1f", freq / 10.0);
+		if( tl_model == TRIPP_LITE_SMARTPRO ) {
+			freq = hex2d(t_value + 3, 3);
+			dstate_setinfo("input.frequency", "%.1f", freq / 10.0);
 
-		switch(t_value[6]) {
-			case '1':
-				dstate_setinfo("input.frequency.nominal", "%d", 60);
-				break;
-			case '0':
-				dstate_setinfo("input.frequency.nominal", "%d", 50);
-				break;
+			switch(t_value[6]) {
+				case '1':
+					dstate_setinfo("input.frequency.nominal", "%d", 60);
+					break;
+				case '0':
+					dstate_setinfo("input.frequency.nominal", "%d", 50);
+					break;
+			}
                 }
 
+		if( tl_model == TRIPP_LITE_SMART_0004 ) {
+			freq = hex2d(t_value + 3, 4);
+			dstate_setinfo("input.frequency", "%.1f", freq / 10.0);
+		}
+
 		/* I'm guessing this is a calibration constant of some sort. */
 		dstate_setinfo("ups.temperature", "%.1f", (unsigned)(hex2d(t_value+1, 2)) * 0.3636 - 21);
 	}
 
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
+
 	ret = send_cmd(l_msg, sizeof(l_msg), l_value, sizeof(l_value));
 	if(ret <= 0) {
 		dstate_datastale();
@@ -1298,11 +1408,17 @@ void upsdrv_updateinfo(void)
 		case TRIPP_LITE_SMARTPRO:
 			dstate_setinfo("ups.load", "%d", hex2d(l_value+1, 2));
 			break;
+		case TRIPP_LITE_SMART_0004:
+			dstate_setinfo("ups.load", "%d", hex2d(l_value+1, 2));
+			dstate_setinfo("ups.debug.L","%s", hexascdump(l_value+1, 7));
+			break;
 		default:
 			dstate_setinfo("ups.debug.L","%s", hexascdump(l_value+1, 7));
 			break;
 	}
 
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
+
 	if(tl_model != TRIPP_LITE_OMNIVS && tl_model != TRIPP_LITE_OMNIVS_2001) {
 		debug_message("D", 2);
 
@@ -1311,9 +1427,9 @@ void upsdrv_updateinfo(void)
 			debug_message("V", 2); /* Probably not necessary - seems to be static */
 			debug_message("M", 2);
 			debug_message("T", 2);
+			debug_message("P", 2);
 		}
 		/* debug_message("U", 2); */
-		/* debug_message("Z", 2); */
 	}
 
 	dstate_dataok();
@@ -1348,14 +1464,6 @@ void upsdrv_makevartable(void)
 #endif
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Tripp Lite OMNIVS and SMARTPRO driver %s (%s)\n",
-			DRV_VERSION, UPS_VERSION);
-
-	experimental_driver = 1;
-}
-
 /*!@brief Initialize UPS and variables from ups.conf
  *
  * @todo Allow binding based on firmware version (which seems to vary wildly
@@ -1367,9 +1475,8 @@ void upsdrv_initups(void)
 	int r;
 
 	/* process the UPS selection options */
-	regex_array[0] = "09AE" /* getval("vendorid") */;
+	regex_array[0] = NULL; /* handled by USB IDs device table */
 	regex_array[1] = getval("productid");
-	if(!regex_array[1]) regex_array[1] = "0001";
 	regex_array[2] = getval("vendor"); /* vendor string */
 	regex_array[3] = getval("product"); /* product string */
 	regex_array[4] = getval("serial"); /* probably won't see this */
@@ -1382,6 +1489,9 @@ void upsdrv_initups(void)
 		fatalx(EXIT_FAILURE, "invalid regular expression: %s", regex_array[r]);
 	}
 
+	/* link the matchers */
+	regex_matcher->next = &subdriver_matcher;
+
 	/* Search for the first supported UPS matching the regular
 	 *            expression */
 	r = comm_driver->open(&udev, &curDevice, regex_matcher, NULL);
@@ -1393,6 +1503,9 @@ void upsdrv_initups(void)
 	
 	upslogx(1, "Detected a UPS: %s/%s", hd->Vendor ? hd->Vendor : "unknown", hd->Product ? hd->Product : "unknown");
 
+	dstate_setinfo("ups.vendorid", "%04x", hd->VendorID);
+	dstate_setinfo("ups.productid", "%04x", hd->ProductID);
+
 	/* create a new matcher for later reopening */
 	r = USBNewExactMatcher(&reopen_matcher, hd);
 	if (r) {
