@@ -124,7 +124,18 @@
 
 #include "main.h"
 #include "serial.h"
-#include "tripplitesu.h"
+
+#define DRIVER_NAME		"Tripp Lite SmartOnline driver"
+#define DRIVER_VERSION	"0.03"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Allan N. Hessenflow <allanh@kallisti.com>",
+	DRV_EXPERIMENTAL,
+	{ NULL }
+};
 
 #define MAX_RESPONSE_LENGTH 256
 
@@ -205,60 +216,95 @@ static struct {
 #define WATCHDOG                     "WDG" /* poll/set */
 
 
-static int do_command(char type, const char *command,
-                      const char *parameters, char *response)
+static int do_command(char type, const char *command, const char *parameters, char *response)
 {
-	char buffer[5];
-	int count;
-	char *ptr;
+	char	buffer[SMALLBUF];
+	int	count, ret;
 
-	count = 0;
-	ser_flush_in(upsfd, "", nut_debug_level);
+	ser_flush_io(upsfd);
 
-	if (ser_send(upsfd, "~00%c%03d%s%s", type,
-	            strlen(command) + strlen(parameters), command,
-	            parameters) <= 0)
+	if (response) {
+		*response = '\0';
+	}
+
+	snprintf(buffer, sizeof(buffer), "~00%c%03d%s%s", type, strlen(command) + strlen(parameters), command, parameters);
+
+	ret = ser_send_pace(upsfd, 10000, "%s", buffer);
+	if (ret <= 0) {
+		upsdebug_with_errno(3, "do_command: send [%s]", buffer);
 		return -1;
+	}
 
-	if (ser_get_buf_len(upsfd, buffer, 4, 3, 0) <= 0)
+	upsdebugx(3, "do_command: %d bytes sent [%s] -> OK", ret, buffer);
+
+	ret = ser_get_buf_len(upsfd, (unsigned char *)buffer, 4, 3, 0);
+	if (ret < 0) {
+		upsdebug_with_errno(3, "do_command: read");
 		return -1;
-	buffer[4] = '\0';
+	}
+	if (ret == 0) {
+		upsdebugx(3, "do_command: read -> TIMEOUT");
+		return -1;
+	}
+
+	buffer[ret] = '\0';
+	upsdebugx(3, "do_command: %d byted read [%s]", ret, buffer);
 
 	if (!strcmp(buffer, "~00D")) {
-		if (ser_get_buf_len(upsfd, buffer, 3, 3, 0) <= 0)
+
+		ret = ser_get_buf_len(upsfd, (unsigned char *)buffer, 3, 3, 0);
+		if (ret < 0) {
+			upsdebug_with_errno(3, "do_command: read");
 			return -1;
-		buffer[3] = '\0';
+		}
+		if (ret == 0) {
+			upsdebugx(3, "do_command: read -> TIMEOUT");
+			return -1;
+		}
+
+		buffer[ret] = '\0';
+		upsdebugx(3, "do_command: %d bytes read [%s]", ret, buffer);
+
 		count = atoi(buffer);
-		if (count >= MAX_RESPONSE_LENGTH || (count && !response))
+		if (count >= MAX_RESPONSE_LENGTH) {
+			upsdebugx(3, "do_command: response exceeds expected size!");
 			return -1;
+		}
+
+		if (count && !response) {
+			upsdebugx(3, "do_command: response not expected!");
+			return -1;
+		}
+
 		if (count == 0) {
-			if (response)
-				*response = '\0';
 			return 0;
 		}
 
-		if (ser_get_buf_len(upsfd, response, count, 3, 0) <= 0)
+		ret = ser_get_buf_len(upsfd, (unsigned char *)response, count, 3, 0);
+		if (ret < 0) {
+			upsdebug_with_errno(3, "do_command: read");
 			return -1;
-		response[count] = '\0';
+		}
+		if (ret == 0) {
+			upsdebugx(3, "do_command: read -> TIMEOUT");
+			return -1;
+		}
+
+		response[ret] = '\0';
+		upsdebugx(3, "do_command: %d bytes read [%s]", ret, response);
+
 		/* Tripp Lite pads their string responses with spaces.
 		   I don't like that, so I remove them.  This is safe to
 		   do with all responses for this protocol, so I just
 		   do that here. */
-		ptr = response + strlen(response) - 1;
-		while (ptr > response && *ptr == ' ')
-			ptr--;
-		*(ptr + 1) = '\0';
-		ser_comm_good();
-		return count;
+		rtrim(response, ' ');
+
+		return ret;
 	}
+
 	if (!strcmp(buffer, "~00A")) {
-		if (response)
-			*response = '\0';
-		ser_comm_good();
-		return count;
+		return 0;
 	}
-	if (!strcmp(buffer, "~00R"))
-		ser_comm_good();
 
 	return -1;
 }
@@ -535,7 +581,6 @@ void upsdrv_initinfo(void)
 		        device_path);
 	min_low_transfer = max_low_transfer = 0;
 	min_high_transfer = max_high_transfer = 0;
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 
 	/* get all the read-only fields here */
 	if (do_command(POLL, MANUFACTURER, "", response) > 0)
@@ -801,14 +846,6 @@ void upsdrv_makevartable(void)
 	addvar(VAR_VALUE, "lowbatt", "Set low battery level, in percent");
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Tripp Lite SmartOnline driver %s (%s)\n\n", 
-	       DRV_VERSION, UPS_VERSION);
-
-	experimental_driver = 1;
-}
-
 void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
