@@ -1,6 +1,8 @@
 /* upsd.c - watches ups state files and answers queries 
 
-   Copyright (C) 1999  Russell Kroll <rkroll@exploits.org>
+   Copyright (C)
+	1999	Russell Kroll <rkroll@exploits.org>
+	2008	Arjen de Korte <adkorte-guest@alioth.debian.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,14 +29,9 @@
 #include <sys/un.h>
 #include <sys/socket.h>
 #include <netdb.h>
-
-#ifdef HAVE_SSL
-#include <openssl/err.h>
-#include <openssl/ssl.h>
-#endif
+#include <poll.h>
 
 #include "user.h"
-#include "access.h"
 #include "ctype.h"
 #include "stype.h"
 #include "ssl.h"
@@ -42,6 +39,12 @@
 #include "desc.h"
 #include "neterr.h"
 
+#ifdef HAVE_WRAP
+#include <tcpd.h>
+int	allow_severity = LOG_INFO;
+int	deny_severity = LOG_WARNING;
+#endif	/* HAVE_WRAP */
+
 	/* externally-visible settings and pointers */
 
 	upstype_t	*firstups = NULL;
@@ -49,6 +52,9 @@
 	/* default 15 seconds before data is marked stale */
 	int	maxage = 15;
 
+	/* default FD_SETSIZE connections allowed */
+	int	maxconn = FD_SETSIZE;
+
 	/* preloaded to STATEPATH in main, can be overridden via upsd.conf */
 	char	*statepath = NULL;
 
@@ -56,8 +62,10 @@
 	char	*datapath = NULL;
 
 	/* everything else */
+	const char	*progname;
 
 static ctype_t	*firstclient = NULL;
+/* static ctype_t	*lastclient = NULL; */
 
 	/* default is to listen on all local interfaces */
 static stype_t	*firstaddr = NULL;
@@ -66,21 +74,34 @@ static stype_t	*firstaddr = NULL;
 static int 	opt_af = AF_UNSPEC;
 #endif
 
-	/* signal handlers */
-static struct sigaction	sa;
-static sigset_t	nut_upsd_sigmask;
+typedef enum {
+	DRIVER = 1,
+	CLIENT,
+	SERVER
+} handler_type_t;
+
+typedef struct {
+	handler_type_t	type;
+	void		*data;
+} handler_t;
+
+	/* pollfd  */
+static struct pollfd	*fds = NULL;
+static handler_t	*handler = NULL;
 
 	/* pid file */
-static	char	pidfn[SMALLBUF];
+static char	pidfn[SMALLBUF];
 
 	/* set by signal handlers */
-static	int	reload_flag = 0, exit_flag = 0;
+static int	reload_flag = 0, exit_flag = 0;
 
 #ifdef	HAVE_IPV6
-static const char *inet_ntopW (struct sockaddr_storage *s) {
+static const char *inet_ntopW (struct sockaddr_storage *s)
+{
 	static char str[40];
 
-	switch (s->ss_family) {
+	switch (s->ss_family)
+	{
 	case AF_INET:
 		return inet_ntop (AF_INET, &(((struct sockaddr_in *)s)->sin_addr), str, 16);
 	case AF_INET6:
@@ -101,7 +122,7 @@ upstype_t *get_ups_ptr(const char *name)
 		return NULL;
 	}
 
-	for (tmp = firstups; tmp != NULL; tmp = tmp->next) {
+	for (tmp = firstups; tmp; tmp = tmp->next) {
 		if (!strcasecmp(tmp->name, name)) {
 			return tmp;
 		}
@@ -130,125 +151,93 @@ static void ups_data_ok(upstype_t *ups)
 		return;
 	}
 
-	upslogx(LOG_NOTICE, "UPS [%s] data is no longer stale", ups->name);
 	ups->stale = 0;
-}
 
-/* make sure this UPS is connected and has fresh data */
-static void check_ups(upstype_t *ups)
-{
-	/* sanity checks */
-	if ((!ups) || (!ups->fn)) {
-		return;
-	}
-
-	/* see if we need to (re)connect to the socket */
-	if (ups->sock_fd == -1) {
-		ups->sock_fd = sstate_connect(ups);
-	}
-
-	/* throw some warnings if it's not feeding us data any more */
-	if (sstate_dead(ups, maxage)) {
-		ups_data_stale(ups);
-	} else {
-		ups_data_ok(ups);
-	}
+	upslogx(LOG_NOTICE, "UPS [%s] data is no longer stale", ups->name);
 }
 
 /* add another listening address */
 void listen_add(const char *addr, const char *port)
 {
-	stype_t	*stmp, *last;
+	stype_t	*server;
 
 	/* don't change listening addresses on reload */
 	if (reload_flag) {
 		return;
 	}
 
-	stmp = last = firstaddr;
-
-	/* find end of linked list */
-	while (stmp != NULL) {
-		last = stmp;
-		stmp = stmp->next;
-	}
-
 	/* grab some memory and add the info */
-	stmp = xmalloc(sizeof(stype_t));
-	stmp->addr = xstrdup(addr);
-	stmp->port = xstrdup(port);
-	stmp->sock_fd = -1;
-	stmp->next = NULL;
+	server = xcalloc(1, sizeof(*server));
+	server->addr = xstrdup(addr);
+	server->port = xstrdup(port);
+	server->sock_fd = -1;
+	server->next = firstaddr;
 
-	if (last == NULL) {
-		firstaddr = stmp;
-	} else {
-		last->next = stmp;
-	}
+	firstaddr = server;
 
-	upsdebugx(3, "listen_add: added %s:%s", stmp->addr, stmp->port);
+	upsdebugx(3, "listen_add: added %s:%s", server->addr, server->port);
 }
 
 /* create a listening socket for tcp connections */
-static void setuptcp(stype_t *serv)
+static void setuptcp(stype_t *server)
 {
 #ifndef	HAVE_IPV6
 	struct hostent		*host;
-	struct sockaddr_in	server;
+	struct sockaddr_in	sockin;
 	int	res, one = 1;
 
-	host = gethostbyname(serv->addr);
+	host = gethostbyname(server->addr);
 
-	if (host == NULL) {
+	if (!host) {
 		struct  in_addr	listenaddr;
 
-		if (!inet_aton(serv->addr, &listenaddr)) {
+		if (!inet_aton(server->addr, &listenaddr)) {
 			fatal_with_errno(EXIT_FAILURE, "inet_aton");
 		}
 
 		host = gethostbyaddr(&listenaddr, sizeof(listenaddr), AF_INET);
 
-		if (host == NULL) {
+		if (!host) {
 			fatal_with_errno(EXIT_FAILURE, "gethostbyaddr");
 		}
 	}
 
-	if ((serv->sock_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+	if ((server->sock_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 		fatal_with_errno(EXIT_FAILURE, "socket");
 	}
 
-	res = setsockopt(serv->sock_fd, SOL_SOCKET, SO_REUSEADDR, (void *) &one, sizeof(one));
+	res = setsockopt(server->sock_fd, SOL_SOCKET, SO_REUSEADDR, (void *) &one, sizeof(one));
 
 	if (res != 0) {
 		fatal_with_errno(EXIT_FAILURE, "setsockopt(SO_REUSEADDR)");
 	}
 
-	memset(&server, '\0', sizeof(server));
-	server.sin_family = AF_INET;
-	server.sin_port = htons(atoi(serv->port));
+	memset(&sockin, '\0', sizeof(sockin));
+	sockin.sin_family = AF_INET;
+	sockin.sin_port = htons(atoi(server->port));
 
-	memcpy(&server.sin_addr, host->h_addr, host->h_length);
+	memcpy(&sockin.sin_addr, host->h_addr, host->h_length);
 
-	if (bind(serv->sock_fd, (struct sockaddr *) &server, sizeof(server)) == -1) {
-		fatal_with_errno(EXIT_FAILURE, "Can't bind TCP port %s", serv->port);
+	if (bind(server->sock_fd, (struct sockaddr *) &sockin, sizeof(sockin)) == -1) {
+		fatal_with_errno(EXIT_FAILURE, "Can't bind TCP port %s", server->port);
 	}
 
-	if ((res = fcntl(serv->sock_fd, F_GETFL, 0)) == -1) {
+	if ((res = fcntl(server->sock_fd, F_GETFL, 0)) == -1) {
 		fatal_with_errno(EXIT_FAILURE, "fcntl(get)");
 	}
 
-	if (fcntl(serv->sock_fd, F_SETFL, res | O_NDELAY) == -1) {
+	if (fcntl(server->sock_fd, F_SETFL, res | O_NDELAY) == -1) {
 		fatal_with_errno(EXIT_FAILURE, "fcntl(set)");
 	}
 
-	if (listen(serv->sock_fd, 16)) {
+	if (listen(server->sock_fd, 16)) {
 		fatal_with_errno(EXIT_FAILURE, "listen");
 	}
 #else
 	struct addrinfo		hints, *res, *ai;
 	int	v = 0, one = 1;
 
-	upsdebugx(3, "setuptcp: try to bind to %s port %s", serv->addr, serv->port);
+	upsdebugx(3, "setuptcp: try to bind to %s port %s", server->addr, server->port);
 
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_flags		= AI_PASSIVE;
@@ -256,15 +245,15 @@ static void setuptcp(stype_t *serv)
 	hints.ai_socktype	= SOCK_STREAM;
 	hints.ai_protocol	= IPPROTO_TCP;
 
-        if ((v = getaddrinfo(serv->addr, serv->port, &hints, &res)) != 0) {
+	if ((v = getaddrinfo(server->addr, server->port, &hints, &res)) != 0) {
 		if (v == EAI_SYSTEM) {
-                        fatal_with_errno(EXIT_FAILURE, "getaddrinfo");
+			fatal_with_errno(EXIT_FAILURE, "getaddrinfo");
 		}
 
-                fatalx(EXIT_FAILURE, "getaddrinfo: %s", gai_strerror(v));
-        }
+		fatalx(EXIT_FAILURE, "getaddrinfo: %s", gai_strerror(v));
+	}
 
-       for (ai = res; ai != NULL; ai = ai->ai_next) {
+	for (ai = res; ai; ai = ai->ai_next) {
 		int sock_fd;
 
 		if ((sock_fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0) {
@@ -296,7 +285,7 @@ static void setuptcp(stype_t *serv)
 			continue;
 		}
 
-		serv->sock_fd = sock_fd;
+		server->sock_fd = sock_fd;
 		break;
 	}
 
@@ -304,10 +293,10 @@ static void setuptcp(stype_t *serv)
 #endif
 
 	/* don't fail silently */
-	if (serv->sock_fd < 0) {
-		fatalx(EXIT_FAILURE, "not listening on %s port %s", serv->addr, serv->port);
+	if (server->sock_fd < 0) {
+		fatalx(EXIT_FAILURE, "not listening on %s port %s", server->addr, server->port);
 	} else {
-		upslogx(LOG_INFO, "listening on %s port %s", serv->addr, serv->port);
+		upslogx(LOG_INFO, "listening on %s port %s", server->addr, server->port);
 	}
 
 	return;
@@ -320,7 +309,7 @@ static void declogins(const char *upsname)
 
 	ups = get_ups_ptr(upsname);
 
-	if (ups == NULL) {
+	if (!ups) {
 		upslogx(LOG_INFO, "Tried to decrement invalid ups name (%s)", upsname);
 		return;
 	}
@@ -333,56 +322,44 @@ static void declogins(const char *upsname)
 }
 
 /* disconnect a client connection and free all related memory */
-static void delclient(ctype_t *dclient)
+static void client_disconnect(ctype_t *client)
 {
-	ctype_t	*tmp, *last;
-
-	if (dclient == NULL) {
+	if (!client) {
 		return;
 	}
 
-	last = NULL;
-	tmp = firstclient;
+	upsdebugx(2, "Disconnect from %s", client->addr);
 
-	while (tmp != NULL) {
-		if (tmp == dclient) {		/* found it */
-			shutdown(dclient->fd, 2);
-			close(dclient->fd);
+	shutdown(client->sock_fd, 2);
+	close(client->sock_fd);
 
-			free(tmp->addr);
-
-			if (tmp->loginups != NULL) {
-				declogins(tmp->loginups);
-				free(tmp->loginups);
-			}
-
-			free(tmp->username);
-			free(tmp->password);
-
-#ifdef HAVE_SSL
-			if (tmp->ssl) {
-				SSL_free(tmp->ssl);
-			}
-#endif
-
-			pconf_finish(&tmp->ctx);
-
-			if (last == NULL) {	/* deleting first entry */
-				firstclient = tmp->next;
-			} else {
-				last->next = tmp->next;
-			}
-
-			free(tmp);
-			return;
-		}
-
-		last = tmp;
-		tmp = tmp->next;
+	if (client->loginups) {
+		declogins(client->loginups);
 	}
 
-	/* not found?! */
-	upslogx(LOG_WARNING, "Tried to delete client struct that doesn't exist!");
+	ssl_finish(client);
+
+	pconf_finish(&client->ctx);
+
+	if (client->prev) {
+		client->prev->next = client->next;
+	} else {
+		/* deleting first entry */
+		firstclient = client->next;
+	}
+
+	if (client->next) {
+		client->next->prev = client->prev;
+	} else {
+		/* deleting last entry */
+		/* lastclient = client->prev; */
+	}
+
+	free(client->addr);
+	free(client->loginups);
+	free(client->password);
+	free(client->username);
+	free(client);
 
 	return;
 }
@@ -398,10 +375,6 @@ int sendback(ctype_t *client, const char *fmt, ...)
 		return 0;
 	}
 
-	if (client->delete) {
-		return 0;
-	}
-
 	va_start(ap, fmt);
 	vsnprintf(ans, sizeof(ans), fmt, ap);
 	va_end(ap);
@@ -411,14 +384,14 @@ int sendback(ctype_t *client, const char *fmt, ...)
 	if (client->ssl) {
 		res = ssl_write(client, ans, len);
 	} else {
-		res = write(client->fd, ans, len);
+		res = write(client->sock_fd, ans, len);
 	}
 
-	upsdebugx(2, "write: [destfd=%d] [len=%d] [%s]", client->fd, len, rtrim(ans, '\n'));
+	upsdebugx(2, "write: [destfd=%d] [len=%d] [%s]", client->sock_fd, len, rtrim(ans, '\n'));
 
 	if (len != res) {
 		upslog_with_errno(LOG_NOTICE, "write() failed for %s", client->addr);
-		client->delete = 1;
+		client->last_heard = 0;
 		return 0;	/* failed */
 	}
 
@@ -440,32 +413,28 @@ int send_err(ctype_t *client, const char *errtype)
 /* disconnect anyone logged into this UPS */
 void kick_login_clients(const char *upsname)
 {
-	ctype_t   *tmp, *next;
+	ctype_t	*client, *cnext;
 
-	tmp = firstclient;
+	for (client = firstclient; client; client = cnext) {
 
-	while (tmp) {
-		next = tmp->next;
+		cnext = client->next;
 
 		/* if it's not logged in, don't check it */
-		if (!tmp->loginups) {
-			tmp = next;
+		if (!client->loginups) {
 			continue;
 		}
 
-		if (!strcmp(tmp->loginups, upsname)) {
-			upslogx(LOG_INFO, "Kicking client %s (was on UPS [%s])\n", tmp->addr, upsname);
-			delclient(tmp);
+		if (!strcmp(client->loginups, upsname)) {
+			upslogx(LOG_INFO, "Kicking client %s (was on UPS [%s])\n", client->addr, upsname);
+			client_disconnect(client);
 		}
-
-		tmp = next;
 	}
 }
 
 /* make sure a UPS is sane - connected, with fresh data */
 int ups_available(const upstype_t *ups, ctype_t *client)
 {
-	if (ups->sock_fd == -1) {
+	if (ups->sock_fd < 0) {
 		send_err(client, NUT_ERR_DRIVER_NOT_CONNECTED);
 		return 0;
 	}
@@ -484,6 +453,10 @@ static void check_command(int cmdnum, ctype_t *client, int numarg,
 	const char **arg)
 {
 	if (netcmds[cmdnum].flags & FLAG_USER) {
+#ifdef HAVE_WRAP
+		struct request_info	req;
+#endif	/* HAVE_WRAP */
+
 		if (!client->username) {
 			send_err(client, NUT_ERR_USERNAME_REQUIRED);
 			return;
@@ -493,6 +466,16 @@ static void check_command(int cmdnum, ctype_t *client, int numarg,
 			send_err(client, NUT_ERR_PASSWORD_REQUIRED);
 			return;
 		}
+
+#ifdef HAVE_WRAP
+		request_init(&req, RQ_DAEMON, progname, RQ_CLIENT_ADDR, client->addr, RQ_USER, client->username, 0);
+
+		if (!hosts_access(&req)) {
+			/* tcp-wrappers says access should be denied */
+			send_err(client, NUT_ERR_ACCESS_DENIED);
+			return;
+		}
+#endif	/* HAVE_WRAP */
 	}
 
 	/* looks good - call the command */
@@ -504,42 +487,15 @@ static void parse_net(ctype_t *client)
 {
 	int	i;
 
-	/* see if this client is still allowed to talk to us */
-	if (!access_check(&client->sock)) {
-		send_err(client, NUT_ERR_ACCESS_DENIED);
-		client->delete = 1;
-		return;
-	}
-
-	/* paranoia */
-	client->rq[client->rqpos] = '\0';
-
-	/* by this point we should always have a usable line */
-	if (pconf_line(&client->ctx, client->rq) != 1) {
-
-		/* shouldn't happen */
-		upslogx(LOG_WARNING, "pconf_line couldn't handle the input");
-		send_err(client, NUT_ERR_UNKNOWN_COMMAND);
-		return;
-	}
-
-	if (pconf_parse_error(&client->ctx)) {
-		upsdebugx(4, "parse error on net read");
-		send_err(client, NUT_ERR_UNKNOWN_COMMAND);
-		return;
-	}
-
 	/* shouldn't happen */
 	if (client->ctx.numargs < 1) {
 		send_err(client, NUT_ERR_UNKNOWN_COMMAND);
 		return;
 	}
 
-	for (i = 0; netcmds[i].name != NULL; i++) {
+	for (i = 0; netcmds[i].name; i++) {
 		if (!strcasecmp(netcmds[i].name, client->ctx.arglist[0])) {
-			check_command(i, client, client->ctx.numargs, 
-			(const char **) client->ctx.arglist);
-
+			check_command(i, client, client->ctx.numargs, (const char **) client->ctx.arglist);
 			return;
 		}
 	}
@@ -549,139 +505,99 @@ static void parse_net(ctype_t *client)
 	send_err(client, NUT_ERR_UNKNOWN_COMMAND);
 }
 
-/* scan the list of UPSes for sanity */
-static void check_every_ups(void)
-{
-	upstype_t *ups;
-
-	ups = firstups;
-
-	while (ups != NULL) {
-		check_ups(ups);
-		ups = ups->next;
-	}
-}
-
 /* answer incoming tcp connections */
-static void answertcp(stype_t *serv)
-#ifndef	HAVE_IPV6
+static void client_connect(stype_t *server)
 {
+#ifndef	HAVE_IPV6
 	struct	sockaddr_in csock;
 #else
-{
 	struct	sockaddr_storage csock;
 #endif
-	int	acc;
-	ctype_t	*tmp, *last;
 	socklen_t	clen;
+	int		fd;
+	ctype_t		*client;
 
 	clen = sizeof(csock);
-	acc = accept(serv->sock_fd, (struct sockaddr *) &csock, &clen);
+	fd = accept(server->sock_fd, (struct sockaddr *) &csock, &clen);
 
-	if (acc < 0) {
+	if (fd < 0) {
 		return;
 	}
 
-	if (!access_check(&csock)) {
-		upslogx(LOG_NOTICE, "Rejecting TCP connection from %s", 
-#ifndef	HAVE_IPV6
-	 		inet_ntoa(csock.sin_addr));
-#else
-			inet_ntopW(&csock));
-#endif
-		shutdown(acc, shutdown_how);
-		close(acc);
-		return;
-	}
+	client = xcalloc(1, sizeof(*client));
 
-	last = tmp = firstclient;
+	client->sock_fd = fd;
 
-	while (tmp != NULL) {
-		last = tmp;
-		tmp = tmp->next;
-	}
-
-	tmp = xmalloc(sizeof(ctype_t));
-
-	tmp->fd = acc;
-	tmp->delete = 0;
+	time(&client->last_heard);
 
 #ifndef	HAVE_IPV6
-	tmp->addr = xstrdup(inet_ntoa(csock.sin_addr));
-	memcpy(&tmp->sock, &csock, sizeof(struct sockaddr_in));
+	client->addr = xstrdup(inet_ntoa(csock.sin_addr));
 #else
-	tmp->addr = xstrdup(inet_ntopW(&csock));
-	memcpy(&tmp->sock, &csock, sizeof(struct sockaddr_storage));
+	client->addr = xstrdup(inet_ntopW(&csock));
 #endif
 
-	tmp->rqpos = 0;
-	memset(tmp->rq, '\0', sizeof(tmp->rq));
+	pconf_init(&client->ctx, NULL);
 
-	pconf_init(&tmp->ctx, NULL);
-
-	tmp->loginups = NULL;		/* for upsmon */
-	tmp->username = NULL;
-	tmp->password = NULL;
-
-	tmp->ssl = NULL;
-	tmp->ssl_connected = 0;
-
-	tmp->next = NULL;
-
-	if (last == NULL) {
- 		firstclient = tmp;
-	} else {
-		last->next = tmp;
+	if (firstclient) {
+		firstclient->prev = client;
+		client->next = firstclient;
 	}
 
-	upslogx(LOG_DEBUG, "Connection from %s", tmp->addr);
+	firstclient = client;
+
+/*
+	if (lastclient) {
+		client->prev = lastclient;
+		lastclient->next = client;
+	}
+
+	lastclient = client;
+ */
+	upsdebugx(2, "Connect from %s", client->addr);
 }
 
 /* read tcp messages and handle them */
-static void readtcp(ctype_t *client)
+static void client_readline(ctype_t *client)
 {
 	char	buf[SMALLBUF];
 	int	i, ret;
 
-	memset(buf, '\0', sizeof(buf));
-
 	if (client->ssl) {
 		ret = ssl_read(client, buf, sizeof(buf));
 	} else {
-		ret = read(client->fd, buf, sizeof(buf));
+		ret = read(client->sock_fd, buf, sizeof(buf));
 	}
 
-	if (ret < 1) {
-		upslogx(LOG_INFO, "Host %s disconnected (read failure)", client->addr);
-		delclient(client);
+	if (ret < 0) {
+		upsdebug_with_errno(2, "Disconnect %s (read failure)", client->addr);
+		client_disconnect(client);
 		return;
 	}
 
-	/* if an overflow will happen, then clear the queue */
-	if ((ret + client->rqpos) >= sizeof(client->rq)) {
-		memset(client->rq, '\0', sizeof(client->rq));
-		client->rqpos = 0;
+	if (ret == 0) {
+		upsdebugx(2, "Disconnect %s (no data available)", client->addr);
+		client_disconnect(client);
+		return;
 	}
 
 	/* fragment handling code */
-
 	for (i = 0; i < ret; i++) {
+
 		/* add to the receive queue one by one */
-		client->rq[client->rqpos++] = buf[i];
-
-		/* parse on linefeed ("blah blah\n") */
-		if (buf[i] == 10) {	/* LF */
+		switch (pconf_char(&client->ctx, buf[i]))
+		{
+		case 1:
+			time(&client->last_heard);	/* command received */
 			parse_net(client);
+			continue;
 
-			/* bail out if the connection closed in parse_net */
-			if (client->delete) {
-				delclient(client);
-				return;
-			}
-			
-			/* reset queue */
-			client->rqpos = 0;
-			memset(client->rq, '\0', sizeof(client->rq));
+		case 0:
+			continue;	/* haven't gotten a line yet */
+
+		default:
+			/* parse error */
+			upslogx(LOG_NOTICE, "Parse error on sock: %s", client->ctx.errmsg);
+			return;
 		}
 	}
 
@@ -690,62 +606,64 @@ static void readtcp(ctype_t *client)
 
 void server_load(void)
 {
-	stype_t	*serv;
+	stype_t	*server;
 
 	/* default behaviour if no LISTEN addres has been specified */
-	if (firstaddr == NULL) {
-		listen_add("0.0.0.0", string_const(PORT));
+	if (!firstaddr) {
+#ifdef	HAVE_IPV6
+		if (opt_af != AF_INET) {
+			listen_add("::1", string_const(PORT));
+		}
+
+		if (opt_af != AF_INET6) {
+			listen_add("127.0.0.1", string_const(PORT));
+		}
+#else
+		listen_add("127.0.0.1", string_const(PORT));
+#endif
 	}
 
-	for (serv = firstaddr; serv != NULL; serv = serv->next) {
-		setuptcp(serv);
+	for (server = firstaddr; server; server = server->next) {
+		setuptcp(server);
 	}
 }
 
 void server_free(void)
 {
-	stype_t	*stmp, *snext;
+	stype_t	*server, *snext;
 
 	/* cleanup server fds */
-	stmp = firstaddr;
+	for (server = firstaddr; server; server = snext) {
+		snext = server->next;
 
-	while (stmp) {
-		snext = stmp->next;
+		if (server->sock_fd != -1) {
+			close(server->sock_fd);
+		}
 
-		if (stmp->sock_fd != -1)
-			close(stmp->sock_fd);
-
-		free(stmp->addr);
-		free(stmp->port);
-		free(stmp);
-
-		stmp = snext;
+		free(server->addr);
+		free(server->port);
+		free(server);
 	}
 
 	firstaddr = NULL;
 }
 
-static void upsd_cleanup(void)
+static void client_free(void)
 {
-	ctype_t	*tmpcli, *tmpnext;
-	upstype_t	*ups, *unext;
+	ctype_t		*client, *cnext;
 
 	/* cleanup client fds */
-	tmpcli = firstclient;
-
-	while (tmpcli) {
-		tmpnext = tmpcli->next;
-		delclient(tmpcli);
-		tmpcli = tmpnext;
+	for (client = firstclient; client; client = cnext) {
+		cnext = client->next;
+		client_disconnect(client);
 	}
+}
 
-	if (strcmp(pidfn, "") != 0) {
-		unlink(pidfn);
-	}
+void driver_free(void)
+{
+	upstype_t	*ups, *unext;
 
-	ups = firstups;
-
-	while (ups) {
+	for (ups = firstups; ups; ups = unext) {
 		unext = ups->next;
 
 		if (ups->sock_fd != -1) {
@@ -754,135 +672,195 @@ static void upsd_cleanup(void)
 
 		sstate_infofree(ups);
 		sstate_cmdfree(ups);
+
 		pconf_finish(&ups->sock_ctx);
 
 		free(ups->fn);
 		free(ups->name);
 		free(ups->desc);
 		free(ups);
+	}
+}
 
-		ups = unext;
+static void upsd_cleanup(void)
+{
+	if (strlen(pidfn) > 0) {
+		unlink(pidfn);
 	}
 
 	/* dump everything */
 
-	acl_free();
-	access_free();
 	user_flush();
 	desc_free();
+	
 	server_free();
+	client_free();
+	driver_free();
 
 	free(statepath);
 	free(datapath);
 	free(certfile);
+
+	free(fds);
+	free(handler);
+}
+
+void poll_reload(void)
+{
+	int	ret;
+
+	ret = sysconf(_SC_OPEN_MAX);
+
+	if (ret < maxconn) {
+		fatalx(EXIT_FAILURE, "Maximum number of connections limited to %d [requested %d]", ret, maxconn);
+	}
+
+	fds = xrealloc(fds, maxconn * sizeof(*fds));
+	handler = xrealloc(handler, maxconn * sizeof(*handler));
 }
 
 /* service requests and check on new data */
 static void mainloop(void)
 {
-	fd_set	rfds;
-	struct	timeval	tv;
-	int	res, maxfd = -1;
-	ctype_t	*tmpcli, *tmpnext;
-	upstype_t	*utmp, *unext;
-	stype_t	*stmp, *snext;
+	int	i, ret, nfds = 0;
+
+	upstype_t	*ups;
+	ctype_t		*client, *cnext;
+	stype_t		*server;
+	time_t	now;
+
+	time(&now);
 
 	if (reload_flag) {
 		conf_reload();
+		poll_reload();
 		reload_flag = 0;
 	}
-	
-	tv.tv_sec = 2;
-	tv.tv_usec = 0;
 
-	FD_ZERO(&rfds);
+	/* scan through driver sockets */
+	for (ups = firstups; ups && (nfds < maxconn); ups = ups->next) {
 
-	/* scan through servers and add to FD_SET */
-	for (stmp = firstaddr; stmp != NULL; stmp = stmp->next) {
-		if (stmp->sock_fd != -1) {
-			FD_SET(stmp->sock_fd, &rfds);
-
-			if (stmp->sock_fd > maxfd) {
-				maxfd = stmp->sock_fd;
-			}
+		/* see if we need to (re)connect to the socket */
+		if (ups->sock_fd < 0) {
+			ups->sock_fd = sstate_connect(ups);
+			continue;
 		}
+
+		/* throw some warnings if it's not feeding us data any more */
+		if (sstate_dead(ups, maxage)) {
+			ups_data_stale(ups);
+		} else {
+			ups_data_ok(ups);
+		}
+
+		fds[nfds].fd = ups->sock_fd;
+		fds[nfds].events = POLLIN;
+
+		handler[nfds].type = DRIVER;
+		handler[nfds].data = ups;
+
+		nfds++;
 	}
 
-	/* scan through clients and add to FD_SET */
-	for (tmpcli = firstclient; tmpcli != NULL; tmpcli = tmpcli->next) {
-		if (tmpcli->fd != -1) {
-			FD_SET(tmpcli->fd, &rfds);
+	/* scan through client sockets */
+	for (client = firstclient; client; client = cnext) {
 
-			if (tmpcli->fd > maxfd) {
-				maxfd = tmpcli->fd;
-			}
+		cnext = client->next;
+
+		if (difftime(now, client->last_heard) > 60) {
+			/* shed clients after 1 minute of inactivity */
+			client_disconnect(client);
+			continue;
 		}
+
+		if (nfds >= maxconn) {
+			/* ignore clients that we are unable to handle */
+			continue;
+		}
+
+		fds[nfds].fd = client->sock_fd;
+		fds[nfds].events = POLLIN;
+
+		handler[nfds].type = CLIENT;
+		handler[nfds].data = client;
+
+		nfds++;
 	}
 
-	/* also add new driver sockets */
-	for (utmp = firstups; utmp != NULL; utmp = utmp->next) {
-		if (utmp->sock_fd != -1) {
-			FD_SET(utmp->sock_fd, &rfds);
+	/* scan through server sockets */
+	for (server = firstaddr; server && (nfds < maxconn); server = server->next) {
 
-			if (utmp->sock_fd > maxfd) {
-				maxfd = utmp->sock_fd;
-			}
-		}
-	}
-	
-	res = select(maxfd + 1, &rfds, NULL, NULL, &tv);
-
-	if (res > 0) {
-		/* scan servers for activity */
-		stmp = firstaddr;
-
-		while (stmp) {
-			snext = stmp->next;
-
-			if (stmp->sock_fd != -1) {
-				if (FD_ISSET(stmp->sock_fd, &rfds)) {
-					answertcp(stmp);
-				}
-			}
-
-			stmp = snext;
-		}
-		
-		/* scan clients for activity */
-		tmpcli = firstclient;
-
-		while (tmpcli != NULL) {
-
-			/* preserve for later since delclient may run */
-			tmpnext = tmpcli->next;
-
-			if (tmpcli->fd != -1) {
-				if (FD_ISSET(tmpcli->fd, &rfds)) {
-					readtcp(tmpcli);
-				}
-			}
-
-			tmpcli = tmpnext;
+		if (server->sock_fd < 0) {
+			continue;
 		}
 
-		/* now scan ups sockets for activity */
-		utmp = firstups;
+		fds[nfds].fd = server->sock_fd;
+		fds[nfds].events = POLLIN;
 
-		while (utmp) {
-			unext = utmp->next;
+		handler[nfds].type = SERVER;
+		handler[nfds].data = server;
 
-			if (utmp->sock_fd != -1) {
-				if (FD_ISSET(utmp->sock_fd, &rfds)) {
-					sstate_sock_read(utmp);
-				}
-
-			}
-
-			utmp = unext;
-		}
+		nfds++;
 	}
 
-	check_every_ups();
+	upsdebugx(2, "%s: polling %d filedescriptors", __func__, nfds);
+
+	ret = poll(fds, nfds, 2000);
+
+	if (ret == 0) {
+		upsdebugx(2, "%s: no data available", __func__);
+		return;
+	}
+
+	if (ret < 0) {
+		upslog_with_errno(LOG_ERR, "%s", __func__);
+		return;
+	}
+
+	for (i = 0; i < nfds; i++) {
+
+		if (fds[i].revents & (POLLHUP|POLLERR|POLLNVAL)) {
+
+			switch(handler[i].type)
+			{
+			case DRIVER:
+				sstate_disconnect((upstype_t *)handler[i].data);
+				break;
+			case CLIENT:
+				client_disconnect((ctype_t *)handler[i].data);
+				break;
+			case SERVER:
+				upsdebugx(2, "%s: server disconnected", __func__);
+				break;
+			default:
+				upsdebugx(2, "%s: <unknown> disconnected", __func__);
+				break;
+			}
+
+			continue;
+		}
+
+		if (fds[i].revents & POLLIN) {
+
+			switch(handler[i].type)
+			{
+			case DRIVER:
+				sstate_readline((upstype_t *)handler[i].data);
+				break;
+			case CLIENT:
+				client_readline((ctype_t *)handler[i].data);
+				break;
+			case SERVER:
+				client_connect((stype_t *)handler[i].data);
+				break;
+			default:
+				upsdebugx(2, "%s: <unknown> has data available", __func__);
+				break;
+			}
+
+			continue;
+		}
+	}
 }
 
 static void help(const char *progname) 
@@ -896,7 +874,6 @@ static void help(const char *progname)
 	printf("		 - reload: reread configuration files\n");
 	printf("		 - stop: stop process and exit\n");
 	printf("  -D		raise debugging level\n");
-	printf("  -f		stay in the foreground for testing\n");
 	printf("  -h		display this help\n");
 	printf("  -r <dir>	chroots to <dir>\n");
 	printf("  -u <user>	switch to <user> (if started as root)\n");
@@ -919,13 +896,15 @@ static void set_exit_flag(int sig)
 	exit_flag = sig;
 }
 
-/* basic signal setup to ignore SIGPIPE */
-static void setupsignals(void)
+static void setup_signals(void)
 {
-	sigemptyset(&nut_upsd_sigmask);
-	sa.sa_mask = nut_upsd_sigmask;
+	struct sigaction	sa;
+
+	sigemptyset(&sa.sa_mask);
+	sigaddset(&sa.sa_mask, SIGHUP);
 	sa.sa_flags = 0;
 
+	/* basic signal setup to ignore SIGPIPE */
 	sa.sa_handler = SIG_IGN;
 	sigaction(SIGPIPE, &sa, NULL);
 
@@ -955,17 +934,16 @@ void check_perms(const char *fn)
 	if (st.st_mode & (S_IROTH | S_IXOTH)) {
 		upslogx(LOG_WARNING, "%s is world readable", fn);
 	}
-}	
+}
 
 int main(int argc, char **argv)
 {
 	int	i, cmd = 0;
-	int	do_background = 1;
-	char	*progname, *chroot_path = NULL;
+	char	*chroot_path = NULL;
 	const char	*user = NULL;
 	struct passwd	*new_uid = NULL;
 
-	progname = argv[0];
+	progname = xbasename(argv[0]);
 
 	/* pick up a default from configure --with-user */
 	user = RUN_AS_USER;
@@ -992,9 +970,6 @@ int main(int argc, char **argv)
 			case 'r':
 				chroot_path = optarg;
 				break;
-			case 'f':
-				do_background = 0;
-				break;
 			case 'u':
 				user = optarg;
 				break;
@@ -1015,7 +990,6 @@ int main(int argc, char **argv)
 				break;
 
 			case 'D':
-				do_background = 0;
 				nut_debug_level++;
 				break;
 
@@ -1047,7 +1021,9 @@ int main(int argc, char **argv)
 		help(progname);
 	}
 
-	setupsignals();
+	atexit(upsd_cleanup);
+
+	setup_signals();
 
 	open_syslog("upsd");
 
@@ -1079,6 +1055,7 @@ int main(int argc, char **argv)
 	/* handle ups.conf */
 	read_upsconf();
 	upsconf_add(0);		/* 0 = initial */
+	poll_reload();
 
 	if (num_ups == 0) {
 		fatalx(EXIT_FAILURE, "Fatal error: at least one UPS must be defined in ups.conf");
@@ -1092,20 +1069,17 @@ int main(int argc, char **argv)
 	/* handle upsd.users */
 	user_load();
 
-	if (do_background) {
+	if (!nut_debug_level) {
 		background();
-
 		writepid(pidfn);
 	} else {
-		memset(&pidfn, '\0', sizeof(pidfn));
+		memset(pidfn, 0, sizeof(pidfn));
 	}
 
-	while (exit_flag == 0) {
+	while (!exit_flag) {
 		mainloop();
 	}
 
 	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
-	upsd_cleanup();
-
-	exit(EXIT_SUCCESS);
+	return EXIT_SUCCESS;
 }
